/* ############################ MALP License ############################## */
/* # Fri Jan 18 14:00:00 CET 2013                                         # */
/* # Copyright or (C) or Copr. Commissariat a l'Energie Atomique          # */
/* #                                                                      # */
/* # This software is governed by the CeCILL-C license under French law   # */
/* # and abiding by the rules of distribution of free software.  You can  # */
/* # use, modify and/ or redistribute the software under the terms of     # */
/* # the CeCILL-C license as circulated by CEA, CNRS and INRIA at the     # */
/* # following URL http://www.cecill.info.                                # */
/* #                                                                      # */
/* # The fact that you are presently reading this means that you have     # */
/* # had knowledge of the CeCILL-C license and that you accept its        # */
/* # terms.                                                               # */
/* #                                                                      # */
/* # Authors:                                                             # */
/* #   - BESNARD Jean-Baptiste jean-baptiste.besnard@cea.fr               # */
/* #                                                                      # */
/* ######################################################################## */
#include <svut.h>
#include <iostream>
#include <stdint.h>

using namespace svUnitTest;

#include "Bit_Array.h"

using namespace std;

SVUT_DECLARE_FLAT_TEST(Bit_Array,Init_Release)
{
	struct Bit_Array ba;
	
	Bit_Array_init( &ba, 4241 );
	
	SVUT_ASSERT_EQUAL( ba.size, 4241u/8u + 1u );
	SVUT_ASSERT_EQUAL( ba.real_size, 4241u );
	SVUT_ASSERT_NOT_NULL( ba.array );

	Bit_Array_release( &ba );
	
	SVUT_ASSERT_EQUAL( ba.size, 0u );
	SVUT_ASSERT_NULL( ba.array );
	SVUT_ASSERT_EQUAL( ba.real_size, 0u );
}

SVUT_DECLARE_FLAT_TEST(Bit_Array,Bit_Array_size)
{
	struct Bit_Array ba;
	
	Bit_Array_init( &ba, 4241 );
	
	uint64_t ret = Bit_Array_size( &ba );
	SVUT_ASSERT_EQUAL( ret, 4241u );

	Bit_Array_release( &ba );
	
	ret = Bit_Array_size( &ba );
	SVUT_ASSERT_EQUAL( ret, 0u );
}



SVUT_DECLARE_FLAT_TEST(Bit_Array,Set_Get)
{
	struct Bit_Array ba;
	
	Bit_Array_init( &ba, 4241 );
	
	uint64_t key = rand()%4241;
	
	
	int i = 0;
	
	for( i = 0 ; i < 4241 ; i++ )
	{
		uint64_t val = rand()%2;
		
		Bit_Array_set( &ba, key, val );
		uint64_t ret = Bit_Array_get( &ba, key);
		SVUT_ASSERT_EQUAL( val, ret );
	}

	Bit_Array_release( &ba );
}


SVUT_DECLARE_FLAT_TEST(Bit_Array,Overflow)
{
	struct Bit_Array ba;
	
	Bit_Array_init( &ba, 10 );
	
	
	int i = 0;
	
	for( i = 10 ; i < 1200 ; i++ )
	{
		int val = rand()%2;
		int key = rand();
		
		Bit_Array_set( &ba, key, val );
		uint64_t ret = Bit_Array_get( &ba, key);
		SVUT_ASSERT_EQUAL( 3u, ret );
	}

	Bit_Array_release( &ba );
}



SVUT_DECLARE_FLAT_TEST(Bit_Array,Bit_Array_replicate)
{
	struct Bit_Array ba;
	struct Bit_Array bb;

	Bit_Array_init( &ba, 1024 );
	Bit_Array_init( &bb, 1024 );
	
	Bit_Array_replicate( &ba, &bb );
	
	int i = 0;
	
	for( i = 0 ; i < 1024 ; i++ )
	{
		int vala = rand()%2;
		int valb = rand()%2;
		
		Bit_Array_set( &ba, i, vala );
		Bit_Array_set( &bb, i, valb );
	}
	
	for( i = 0 ; i < 1024 ; i++ )
	{
		uint64_t reta = Bit_Array_get( &ba, i);
		uint64_t retb = Bit_Array_get( &ba, i);
		SVUT_ASSERT_EQUAL( reta, retb );
	}

	Bit_Array_release( &ba );
	Bit_Array_release( &bb );
}




