/* ############################ MALP License ############################## */
/* # Fri Jan 18 14:00:00 CET 2013                                         # */
/* # Copyright or (C) or Copr. Commissariat a l'Energie Atomique          # */
/* #                                                                      # */
/* # This software is governed by the CeCILL-C license under French law   # */
/* # and abiding by the rules of distribution of free software.  You can  # */
/* # use, modify and/ or redistribute the software under the terms of     # */
/* # the CeCILL-C license as circulated by CEA, CNRS and INRIA at the     # */
/* # following URL http://www.cecill.info.                                # */
/* #                                                                      # */
/* # The fact that you are presently reading this means that you have     # */
/* # had knowledge of the CeCILL-C license and that you accept its        # */
/* # terms.                                                               # */
/* #                                                                      # */
/* # Authors:                                                             # */
/* #   - BESNARD Jean-Baptiste jean-baptiste.besnard@cea.fr               # */
/* #                                                                      # */
/* ######################################################################## */
#include <svut.h>
#include <iostream>
#include <stdint.h>
#include <stdio.h>

#include <omp.h>

using namespace svUnitTest;

#include "Trie.h"


using namespace std;

SVUT_DECLARE_FLAT_TEST(Trie,Trie_init_release)
{
	struct Trie tr;
	
	Trie_init( &tr );

	SVUT_ASSERT_NOT_NULL( tr.root );

	Trie_release( &tr );

	SVUT_ASSERT_NULL( tr.root );
	
}

SVUT_DECLARE_FLAT_TEST(Trie,Trie_set_acquire)
{
	struct Trie tr;
	
	Trie_init( &tr );

	int magic_value = 16384;

	int ret = Trie_set( &tr, "magic_value", (void *)&magic_value, sizeof(int) );

	SVUT_ASSERT_EQUAL( 0, ret );

	struct Trie_node *node = Trie_get( &tr, "magic_value" );
	
	SVUT_ASSERT_NOT_NULL( node );

	int *pl = (int *)node->payload;
	
	SVUT_ASSERT_EQUAL( *pl , magic_value );

	magic_value = 99;
	
	SVUT_ASSERT_EQUAL( *pl , 16384 );
	
	Trie_relax_node( node );

	Trie_release( &tr );

	
}



SVUT_DECLARE_FLAT_TEST(Trie,Trie_set_delete)
{
	struct Trie tr;
	
	Trie_init( &tr );

	int magic_value = 16384;

	int ret = Trie_set( &tr, "magic_value", (void *)&magic_value, sizeof(int) );
	
	magic_value = 16383;
	Trie_set( &tr, "magic_valu", (void *)&magic_value, sizeof(int) );
	magic_value = 16382;
	Trie_set( &tr, "magic_val", (void *)&magic_value, sizeof(int) );

	SVUT_ASSERT_EQUAL( 0, ret );

	struct Trie_node *node = Trie_get( &tr, "magic_value" );
	SVUT_ASSERT_NOT_NULL( node );
	int *pl = (int *)node->payload;
	SVUT_ASSERT_EQUAL( *pl , 16384 );
	Trie_relax_node( node );


	Trie_delete( &tr, "magic_value" );
	node = Trie_get( &tr, "magic_value" );
	SVUT_ASSERT_NULL( node );

	
	node = Trie_get( &tr, "magic_valu" );
	SVUT_ASSERT_NOT_NULL( node );
	pl = (int *)node->payload;
	SVUT_ASSERT_EQUAL( *pl , 16383 );
	Trie_relax_node( node );
	
	node = Trie_get( &tr, "magic_val" );
	SVUT_ASSERT_NOT_NULL( node );
	pl = (int *)node->payload;
	SVUT_ASSERT_EQUAL( *pl , 16382 );
	Trie_relax_node( node );
	
	
	Trie_release( &tr );

	
}


SVUT_DECLARE_FLAT_TEST(Trie,Trie_set_get_delete_get_many)
{
	struct Trie tr;
	
	Trie_init( &tr );

	int i;
	char name[20];
	
	for( i = 0 ; i < 1e6 ; i++ )
	{
		if( !(i%100000) )
			printf("Setting elem %d\n", i );
		sprintf( name, "%d" , i );
		 Trie_set( &tr, name, (void *)&i, sizeof(int) );
	}
	
	for( i = 0 ; i < 1e6 ; i++ )
	{
		if( !(i%100000) )
			printf("Checking elem %d\n", i );
			
		sprintf( name, "%d" , i );
		
		struct Trie_node *node = Trie_get( &tr, name );
		SVUT_ASSERT_NOT_NULL( node );
		int *pl = (int *)node->payload;
		SVUT_ASSERT_EQUAL( *pl , i );
		Trie_relax_node( node );
		
		
	}
	
	for( i = 0 ; i < 1e6 ; i++ )
	{
		if( !(i%100000) )
			printf("Deleting elem %d\n", i );
			
		sprintf( name, "%d" , i );	
		Trie_delete( &tr, name );
	
	}
	
	for( i = 1e6 ; 0 <= i ; i-- )
	{
		if( !(i%100000) )
			printf("Checking elem %d is gone ...\n", i );
			
		sprintf( name, "%d" , i );
		
		struct Trie_node *node = Trie_get( &tr, name );
		SVUT_ASSERT_NULL( node );
		
	}
	
	Trie_release( &tr );

	
}



SVUT_DECLARE_FLAT_TEST(Trie,Trie_set_get_delete_get_many_omp)
{
	struct Trie tr;
	
	Trie_init( &tr );

	int i;

#pragma omp parallel
{	

	char name[20];
		
		#pragma omp for private(i)
	for( i = 0 ; i < 1000000 ; i++ )
	{
		if( !(i%100000) )
			printf("Setting elem %d\n", i );
		sprintf( name, "%d" , i );
		 Trie_set( &tr, name, (void *)&i, sizeof(int) );
	}
	
	
	#pragma omp for private(i)
	for( i = 0 ; i < 1000000 ; i++ )
	{
		if( !(i%100000) )
			printf("Checking elem %d\n", i );
			
		sprintf( name, "%d" , i );
		
		struct Trie_node *node = Trie_get( &tr, name );
		SVUT_ASSERT_NOT_NULL( node );
		int *pl = (int *)node->payload;
		SVUT_ASSERT_EQUAL( *pl , i );
		Trie_relax_node( node );
		
		
	}
	
	#pragma omp for private(i)
	for( i = 0 ; i < 1000000 ; i++ )
	{
		if( !(i%100000) )
			printf("Deleting elem %d\n", i );
			
		sprintf( name, "%d" , i );	
		Trie_delete( &tr, name );
	
	}
	
	#pragma omp for private(i)
	for( i = 1000000 ; 0 <= i ; i-- )
	{
		if( !(i%100000) )
			printf("Checking elem %d is gone ...\n", i );
			
		sprintf( name, "%d" , i );
		
		struct Trie_node *node = Trie_get( &tr, name );
		SVUT_ASSERT_NULL( node );
		
	}
	
}
	
	Trie_release( &tr );

	
}

