/* ############################ MALP License ############################## */
/* # Fri Jan 18 14:00:00 CET 2013                                         # */
/* # Copyright or (C) or Copr. Commissariat a l'Energie Atomique          # */
/* #                                                                      # */
/* # This software is governed by the CeCILL-C license under French law   # */
/* # and abiding by the rules of distribution of free software.  You can  # */
/* # use, modify and/ or redistribute the software under the terms of     # */
/* # the CeCILL-C license as circulated by CEA, CNRS and INRIA at the     # */
/* # following URL http://www.cecill.info.                                # */
/* #                                                                      # */
/* # The fact that you are presently reading this means that you have     # */
/* # had knowledge of the CeCILL-C license and that you accept its        # */
/* # terms.                                                               # */
/* #                                                                      # */
/* # Authors:                                                             # */
/* #   - BESNARD Jean-Baptiste jean-baptiste.besnard@cea.fr               # */
/* #                                                                      # */
/* ######################################################################## */
#include <svut.h>
#include <iostream>
#include <stdint.h>

using namespace svUnitTest;

#include "Key_Entry.h"
#include "Knowledge_System.h"

using namespace std;


SVUT_DECLARE_FLAT_TEST(Key_entry,Key_entry_init)
{
	struct Key_entry ke;
	Key_entry_init( &ke, 1234 );
	
	SVUT_ASSERT_EQUAL(1234u,ke.key);
	SVUT_ASSERT_EQUAL(0u,ke.target_ks.count);
	SVUT_ASSERT_EQUAL(0u,ke.lock);
	
	Key_entry_release( &ke );
	
}


SVUT_DECLARE_FLAT_TEST(Key_entry,Key_entry_release)
{
	struct Key_entry ke;
	Key_entry_init( &ke, 1234 );
	Key_entry_release( &ke );
	
	
	SVUT_ASSERT_EQUAL(0u,ke.key);
	SVUT_ASSERT_EQUAL(0u,ke.target_ks.count);
	SVUT_ASSERT_EQUAL(1u,ke.lock);
	
}

SVUT_DECLARE_FLAT_TEST(Key_entry,Key_entry_get_ks)
{
	struct Key_entry ke;
	struct Knowledge_system ks;

	Key_entry_init( &ke, 1234 );
	
	Key_entry_push_ks( &ke, &ks );
	
	struct Knowledge_system **ret = Key_entry_get_ks( &ke, &ks );
	SVUT_ASSERT_NOT_NULL( ret );
	SVUT_ASSERT_EQUAL(&ks,*ret);
	
	
	Key_entry_release( &ke );
}



SVUT_DECLARE_FLAT_TEST(Key_entry,Key_entry_push_ks)
{
	struct Key_entry ke;
	Key_entry_init( &ke, 1234 );
	
	struct Knowledge_system ks;
	
	Key_entry_push_ks( &ke, &ks );
	Key_entry_push_ks( &ke, &ks );
	Key_entry_push_ks( &ke, &ks );
	Key_entry_push_ks( &ke, &ks );
	Key_entry_push_ks( &ke, &ks );
	Key_entry_push_ks( &ke, &ks );
	
	SVUT_ASSERT_EQUAL(1u,ke.target_ks.count);
		
	/*Check content*/
	struct Knowledge_system **ret = Key_entry_get_ks( &ke, &ks );
	SVUT_ASSERT_NOT_NULL( ret );
	SVUT_ASSERT_EQUAL(&ks,*ret);
	
	Key_entry_release( &ke );
}


SVUT_DECLARE_FLAT_TEST(Key_entry,Key_entry_pop_ks)
{
	struct Key_entry ke;
	Key_entry_init( &ke, 1234 );
	
	struct Knowledge_system ks;
	
	Key_entry_push_ks( &ke, &ks );

	SVUT_ASSERT_EQUAL(1u,ke.target_ks.count);
	
	struct Knowledge_system **ret = Key_entry_get_ks( &ke, &ks );
	
	/* Check content */
	SVUT_ASSERT_NOT_NULL( ret );
	SVUT_ASSERT_EQUAL(&ks,*ret);
	
	Key_entry_pop_ks( &ke, &ks );

	ret = Key_entry_get_ks( &ke, &ks );
	
	/* Check that tis Ks is gone */
	SVUT_ASSERT_NULL( ret );
		
	Key_entry_release( &ke );
}


SVUT_DECLARE_FLAT_TEST(Key_entry,Key_entry_clear_ks)
{
	struct Key_entry ke;
	Key_entry_init( &ke, 1234 );
	
	struct Knowledge_system ks[5];
	
	Key_entry_push_ks( &ke, &ks[0] );
	SVUT_ASSERT_EQUAL(1u,ke.target_ks.count);
	
	Key_entry_push_ks( &ke, &ks[1] );
	SVUT_ASSERT_EQUAL(2u,ke.target_ks.count);
	
	Key_entry_push_ks( &ke, &ks[2] );
	SVUT_ASSERT_EQUAL(3u,ke.target_ks.count);
	
	Key_entry_push_ks( &ke, &ks[3] );
	SVUT_ASSERT_EQUAL(4u,ke.target_ks.count);
	
	Key_entry_push_ks( &ke, &ks[4] );
	SVUT_ASSERT_EQUAL(5u,ke.target_ks.count);
	
	Key_entry_clear_ks( &ke );
	
	SVUT_ASSERT_EQUAL(0u,ke.target_ks.count);
	
	Key_entry_release( &ke );
}


SVUT_DECLARE_FLAT_TEST(Key_entry,Key_entry_get_count)
{
	struct Key_entry ke;
	Key_entry_init( &ke, 1234 );
	
	struct Knowledge_system ks[5];
	
	Key_entry_push_ks( &ke, &ks[0] );
	Key_entry_push_ks( &ke, &ks[1] );
	Key_entry_push_ks( &ke, &ks[2] );
	Key_entry_push_ks( &ke, &ks[3] );
	Key_entry_push_ks( &ke, &ks[4] );

	SVUT_ASSERT_EQUAL(5u,ke.target_ks.count);
	
	int count = Key_entry_get_count( &ke );
	SVUT_ASSERT_EQUAL(5,count);
	
	Key_entry_pop_ks( &ke, &ks[0] );
	
	count = Key_entry_get_count( &ke );
	SVUT_ASSERT_EQUAL(4,count);
	
	Key_entry_pop_ks( &ke, &ks[1] );
	
	count = Key_entry_get_count( &ke );
	SVUT_ASSERT_EQUAL(3,count);	
	
	Key_entry_pop_ks( &ke, &ks[2] );
	
	count = Key_entry_get_count( &ke );
	SVUT_ASSERT_EQUAL(2,count);
	
	Key_entry_pop_ks( &ke, &ks[3] );
	
	count = Key_entry_get_count( &ke );
	SVUT_ASSERT_EQUAL(1,count);
	
	Key_entry_pop_ks( &ke, &ks[4] );
	
	count = Key_entry_get_count( &ke );
	SVUT_ASSERT_EQUAL(0,count);
	
	Key_entry_release( &ke );
}

int handler( struct Knowledge_system *ks, void *arg )
{
	int *counter = (int *)arg;
	(*counter)++;
	
	return 0;
}


int handler_until_3( struct Knowledge_system *ks, void *arg )
{
	int *counter = (int *)arg;
	
	if( 1 == (*counter) )
		return 1;
	
	(*counter)++;
	
	return 0;
}


SVUT_DECLARE_FLAT_TEST(Key_entry,Key_entry_walk)
{
	struct Key_entry ke;
	Key_entry_init( &ke, 1234 );
	
	struct Knowledge_system ks[5];
	
	Key_entry_push_ks( &ke, &ks[0] );
	Key_entry_push_ks( &ke, &ks[1] );
	Key_entry_push_ks( &ke, &ks[2] );
	Key_entry_push_ks( &ke, &ks[3] );
	Key_entry_push_ks( &ke, &ks[4] );

	SVUT_ASSERT_EQUAL(5u,ke.target_ks.count);

	int counter = 0;
	
	Key_entry_walk( &ke, handler, (void *)&counter );
	
	SVUT_ASSERT_EQUAL(5,counter);
	
	counter = 0;
	
	Key_entry_walk( &ke, handler_until_3, (void *)&counter );

	SVUT_ASSERT_EQUAL(1,counter);
	
	Key_entry_release( &ke );
}
