/* ############################ MALP License ############################## */
/* # Fri Jan 18 14:00:00 CET 2013                                         # */
/* # Copyright or (C) or Copr. Commissariat a l'Energie Atomique          # */
/* #                                                                      # */
/* # This software is governed by the CeCILL-C license under French law   # */
/* # and abiding by the rules of distribution of free software.  You can  # */
/* # use, modify and/ or redistribute the software under the terms of     # */
/* # the CeCILL-C license as circulated by CEA, CNRS and INRIA at the     # */
/* # following URL http://www.cecill.info.                                # */
/* #                                                                      # */
/* # The fact that you are presently reading this means that you have     # */
/* # had knowledge of the CeCILL-C license and that you accept its        # */
/* # terms.                                                               # */
/* #                                                                      # */
/* # Authors:                                                             # */
/* #   - BESNARD Jean-Baptiste jean-baptiste.besnard@cea.fr               # */
/* #                                                                      # */
/* ######################################################################## */
#include <svut.h>
#include <iostream>
#include <stdint.h>

using namespace svUnitTest;

#include "Key_Entry.h"
#include "Knowledge_System.h"
#include "Data_Entry.h"

using namespace std;


void process_ks(  struct MALP_blackboard *bb, struct Data_entry **de, uint32_t entry_count, void *arg )
{
	
	
}

void process_ks1(  struct MALP_blackboard *bb, struct Data_entry **de, uint32_t entry_count, void *arg )
{
	
	
}

void process_ks2(  struct MALP_blackboard *bb, struct Data_entry **de, uint32_t entry_count, void *arg )
{
	
	
}


SVUT_DECLARE_FLAT_TEST(Knowledge_system,Knowledge_system_init)
{
	struct Knowledge_system ks;
	MALP_Trace_crc64_init();
	
	char pl1[200];
	memset( pl1, 41, 200);

	char pl2[200];
	memset( pl2, 42, 200);

	struct Data_entry *dat1 = Data_entry_init( 456, pl1, 200 );
	struct Data_entry *dat2 = Data_entry_init( 12, pl2, 200 );


	uint64_t sens_list[2] =  { 456 , 12 };

	Knowledge_system_init( &ks, 12345, sens_list, 2 ,process_ks, NULL);

	SVUT_ASSERT_EQUAL( 12345, ks.type);
	SVUT_ASSERT_EQUAL( 456 , ks.sensivity_keys[0]);
	SVUT_ASSERT_EQUAL( 12 , ks.sensivity_keys[1]);
	SVUT_ASSERT_EQUAL( 2u, ks.sensivity_count);

	int i = 0;
	
	for( i = 0 ; i < 2 ; i++ )
	{
		SVUT_ASSERT_EQUAL( 0u, ks.sensivity_array[i]);
	}

	SVUT_ASSERT_EQUAL( process_ks, ks.process );
	
	Knowledge_system_release( &ks );
	
	Data_entry_release( dat1 );
	Data_entry_release( dat2 );
}

SVUT_DECLARE_FLAT_TEST(Knowledge_system,Knowledge_system_release)
{
	struct Knowledge_system ks;
	MALP_Trace_crc64_init();
	
	char pl1[200];
	memset( pl1, 41, 200);

	char pl2[200];
	memset( pl2, 42, 200);

	struct Data_entry *dat1 = Data_entry_init( 456, pl1, 200 );
	struct Data_entry *dat2 = Data_entry_init( 12,  pl2, 200 );

	uint64_t sens_list[2] =  { 456, 12 };
	Knowledge_system_init( &ks, 12345, sens_list, 2 ,process_ks, NULL);

	Knowledge_system_release( &ks );


	SVUT_ASSERT_EQUAL( 0, ks.type);
	SVUT_ASSERT_EQUAL( 1u, ks.lock);
	SVUT_ASSERT_EQUAL( 0u, ks.sensivity_count);
	SVUT_ASSERT_NULL( ks.sensivity_keys );
	SVUT_ASSERT_NULL( ks.sensivity_array );
	SVUT_ASSERT_NULL( ks.data_entries );
	SVUT_ASSERT_NULL( ks.process );

	Data_entry_release( dat1 );
	Data_entry_release( dat2 );
}


SVUT_DECLARE_FLAT_TEST(Knowledge_system,Knowledge_system_set_process)
{
	struct Knowledge_system ks;
	MALP_Trace_crc64_init();
	
	char pl1[200];
	memset( pl1, 41, 200);

	char pl2[200];
	memset( pl2, 42, 200);

	struct Data_entry *dat1 = Data_entry_init( 456, pl1, 200 );
	struct Data_entry *dat2 = Data_entry_init( 12, pl2, 200 );

	uint64_t sens_list[2] =  { 456, 12 };
	Knowledge_system_init( &ks, 12345, sens_list, 2 ,process_ks, NULL);
	
	
	Knowledge_system_set_process( &ks, process_ks1, NULL );
	SVUT_ASSERT_EQUAL( process_ks1, ks.process );

	Knowledge_system_set_process( &ks, process_ks2, NULL  );
	SVUT_ASSERT_EQUAL( process_ks2, ks.process );
	
	Knowledge_system_release( &ks );

	Data_entry_release( dat1 );
	Data_entry_release( dat2 );
}

SVUT_DECLARE_FLAT_TEST(Knowledge_system,Knowledge_system_submit_data_entry_similar)
{
	struct Knowledge_system ks;
	MALP_Trace_crc64_init();
	
	char pl1[200];
	memset( pl1, 41, 200);

	char pl2[200];
	memset( pl2, 42, 200);

	struct Data_entry *dat1 = Data_entry_init( 456, pl1, 200 );
	struct Data_entry *dat2 = Data_entry_init( 456, pl2, 200 );

	uint64_t sens_list[2] =  { 456 ,456 };
	Knowledge_system_init( &ks, 12345, sens_list, 2 ,process_ks, NULL);
	
	/* Set ref count for two triggers */
	Data_entry_inc_ref_count( dat1, 2 );
	Data_entry_inc_ref_count( dat2, 2 );
	
	/* Push dep 1 and check for store */
	struct Knowledge_system_job job;
	job = Knowledge_system_submit_data_entry( &ks, dat1 );
	SVUT_ASSERT_EQUAL( 1u, ks.sensivity_array[0] );
	SVUT_ASSERT_EQUAL( 1u, ks.data_entries[0].elem_count );
	SVUT_ASSERT_NULL( job.process );

	/* Push dep2 and chek for trigger */
	job = Knowledge_system_submit_data_entry( &ks, dat2 );
	SVUT_ASSERT_EQUAL( 0u, ks.sensivity_array[0] );
	SVUT_ASSERT_EQUAL( 0u, ks.data_entries[0].elem_count );
	SVUT_ASSERT_EQUAL( 0u, ks.sensivity_array[1] );
	SVUT_ASSERT_EQUAL( 0u, ks.data_entries[1].elem_count );
	SVUT_ASSERT_NOT_NULL( job.process );

	/* Check job */
	SVUT_ASSERT_EQUAL( 2, job.entries_count );
	SVUT_ASSERT_EQUAL( dat1, job.entries[0] );
	SVUT_ASSERT_EQUAL( dat2, job.entries[1] );
	SVUT_ASSERT_EQUAL( process_ks, job.process );	 
	
	Knowledge_system_job_release( &job );

	/* Push dep 2 and check for store */
	job = Knowledge_system_submit_data_entry( &ks, dat2 );
	SVUT_ASSERT_EQUAL( 1u, ks.sensivity_array[0] );
	SVUT_ASSERT_EQUAL( 1u, ks.data_entries[0].elem_count );
	SVUT_ASSERT_NULL( job.process );

	/* Push dep 1 and check for trigger */
	job = Knowledge_system_submit_data_entry( &ks, dat1 );
	SVUT_ASSERT_EQUAL( 0u, ks.sensivity_array[0] );
	SVUT_ASSERT_EQUAL( 0u, ks.data_entries[0].elem_count );
	SVUT_ASSERT_EQUAL( 0u, ks.sensivity_array[1] );
	SVUT_ASSERT_EQUAL( 0u, ks.data_entries[1].elem_count );
	SVUT_ASSERT_NOT_NULL( job.process );

	/* Check job */
	SVUT_ASSERT_EQUAL( 2, job.entries_count );
	SVUT_ASSERT_EQUAL( dat2, job.entries[0] );
	SVUT_ASSERT_EQUAL( dat1, job.entries[1] );
	SVUT_ASSERT_EQUAL( process_ks, job.process );
	
	Knowledge_system_job_release( &job );
	
	Knowledge_system_release( &ks );

	Data_entry_release( dat1 );
	Data_entry_release( dat2 );
}



SVUT_DECLARE_FLAT_TEST(Knowledge_system,Knowledge_system_submit_data_entry)
{
	struct Knowledge_system ks;
	MALP_Trace_crc64_init();
	
	char pl1[200];
	memset( pl1, 41, 200);

	char pl2[200];
	memset( pl2, 42, 200);

	struct Data_entry *dat1 = Data_entry_init( 456, pl1, 200 );
	struct Data_entry *dat2 = Data_entry_init( 12, pl2, 200 );

	uint64_t sens_list[2] =  { 456, 12 };
	Knowledge_system_init( &ks, 12345, sens_list, 2 ,process_ks, NULL);
	
	/* Set ref count for two triggers */
	Data_entry_inc_ref_count( dat1, 2 );
	Data_entry_inc_ref_count( dat2, 2 );
	
	/* Push dep 1 and check for store */
	struct Knowledge_system_job job;
	job = Knowledge_system_submit_data_entry( &ks, dat1 );
	SVUT_ASSERT_EQUAL( 1u, ks.sensivity_array[0] );
	SVUT_ASSERT_EQUAL( 1u, ks.data_entries[0].elem_count );
	SVUT_ASSERT_NULL( job.process );

	/* Push dep2 and chek for trigger */
	job = Knowledge_system_submit_data_entry( &ks, dat2 );
	SVUT_ASSERT_EQUAL( 0u, ks.sensivity_array[0] );
	SVUT_ASSERT_EQUAL( 0u, ks.data_entries[0].elem_count );
	SVUT_ASSERT_EQUAL( 0u, ks.sensivity_array[1] );
	SVUT_ASSERT_EQUAL( 0u, ks.data_entries[1].elem_count );
	SVUT_ASSERT_NOT_NULL( job.process );

	/* Check job */
	SVUT_ASSERT_EQUAL( 2, job.entries_count );
	SVUT_ASSERT_EQUAL( dat1, job.entries[0] );
	SVUT_ASSERT_EQUAL( dat2, job.entries[1] );
	SVUT_ASSERT_EQUAL( process_ks, job.process );	 
	
	Knowledge_system_job_release( &job );

	/* Push dep 2 and check for store */
	job = Knowledge_system_submit_data_entry( &ks, dat2 );
	SVUT_ASSERT_EQUAL( 1u, ks.sensivity_array[1] );
	SVUT_ASSERT_EQUAL( 1u, ks.data_entries[1].elem_count );
	SVUT_ASSERT_NULL( job.process );

	/* Push dep 1 and check for trigger */
	job = Knowledge_system_submit_data_entry( &ks, dat1 );
	SVUT_ASSERT_EQUAL( 0u, ks.sensivity_array[0] );
	SVUT_ASSERT_EQUAL( 0u, ks.data_entries[0].elem_count );
	SVUT_ASSERT_EQUAL( 0u, ks.sensivity_array[1] );
	SVUT_ASSERT_EQUAL( 0u, ks.data_entries[1].elem_count );
	SVUT_ASSERT_NOT_NULL( job.process );

	/* Check job */
	SVUT_ASSERT_EQUAL( 2, job.entries_count );
	SVUT_ASSERT_EQUAL( dat1, job.entries[0] );
	SVUT_ASSERT_EQUAL( dat2, job.entries[1] );
	SVUT_ASSERT_EQUAL( process_ks, job.process );
	
	Knowledge_system_job_release( &job );
	
	Knowledge_system_release( &ks );

	Data_entry_release( dat1 );
	Data_entry_release( dat2 );
}



