/* ############################ MALP License ############################## */
/* # Fri Jan 18 14:00:00 CET 2013                                         # */
/* # Copyright or (C) or Copr. Commissariat a l'Energie Atomique          # */
/* #                                                                      # */
/* # This software is governed by the CeCILL-C license under French law   # */
/* # and abiding by the rules of distribution of free software.  You can  # */
/* # use, modify and/ or redistribute the software under the terms of     # */
/* # the CeCILL-C license as circulated by CEA, CNRS and INRIA at the     # */
/* # following URL http://www.cecill.info.                                # */
/* #                                                                      # */
/* # The fact that you are presently reading this means that you have     # */
/* # had knowledge of the CeCILL-C license and that you accept its        # */
/* # terms.                                                               # */
/* #                                                                      # */
/* # Authors:                                                             # */
/* #   - BESNARD Jean-Baptiste jean-baptiste.besnard@cea.fr               # */
/* #                                                                      # */
/* ######################################################################## */
#include <svut.h>
#include <iostream>
#include <stdint.h>

using namespace svUnitTest;

#include "Key_Entry.h"
#include "Knowledge_System.h"
#include "Data_Entry.h"
#include "Pbb_Workers.h"
#include "Spinlock.h"
#include "MALP_Blackboard.h"

using namespace std;

uint64_t counter = 0;
MALP_Spinlock lock;

struct MALP_blackboard pbb_bb;

void worker_process_ks(  struct MALP_blackboard *bb, struct Data_entry **de, uint32_t entry_count, void *arg )
{
	MALP_Spinlock_lock( &lock );
	counter++;
	if( !( counter % 100000 ) )
		printf( "Tbb_Workers processed %lld jobs\n", (long long int)counter );
	MALP_Spinlock_unlock( &lock );
	
	int i = 0;
	int u = 0;
	
	/* Small payload */
	while( i < 1000000 )
	{
		u *= i + 5;
		i++;
	}

}


SVUT_DECLARE_FLAT_TEST(Pbb_workers,Pbb_workers_init)
{
	struct Pbb_workers wk;
	Pbb_workers_init( &wk, &pbb_bb, 16 );
	
	SVUT_ASSERT_EQUAL( 16u, wk.worker_count );
	SVUT_ASSERT_EQUAL( &pbb_bb, wk.bb );

	SVUT_ASSERT_NOT_NULL( wk.threads );
	SVUT_ASSERT_NOT_NULL( wk.fifos );
	
	uint64_t i = 0 ;
	for( i = 0 ; i < wk.worker_count ; i++ )
	{
		SVUT_ASSERT_EQUAL( 0u, wk.fifos[i].elem_count );
	}

	Pbb_workers_release( &wk );
}


SVUT_DECLARE_FLAT_TEST(Pbb_workers,Pbb_workers_release)
{
	struct Pbb_workers wk;
	Pbb_workers_init( &wk, &pbb_bb, 4 );
	Pbb_workers_release( &wk );

	
	SVUT_ASSERT_EQUAL( 0u, wk.worker_count );

	SVUT_ASSERT_NULL( wk.threads );
	SVUT_ASSERT_NULL( wk.fifos );
}


SVUT_DECLARE_FLAT_TEST(Pbb_workers,Pbb_workers_submit)
{
	struct Knowledge_system_job job;
	
	job.process = worker_process_ks;
	job.entries = NULL;
	job.entries_count = 0;
	job.arg = NULL;
	
	struct Pbb_workers wk;
	Pbb_workers_init( &wk, &pbb_bb, 4 );

	int exp = 0;
	while( exp < 5 )
	{
		uint64_t i = 0;
		uint64_t num = rand()%100;

		for( i = 0 ; i < num ;i ++ )
		{
			Pbb_workers_submit( &wk,&job );
		}
		
		Pbb_workers_wait( &wk );
		SVUT_ASSERT_EQUAL( num, counter );
		counter = 0;
		
		exp++;
	}
	
	Pbb_workers_wait( &wk );
	Pbb_workers_release( &wk );
}


