/* ############################ MALP License ############################## */
/* # Fri Jan 18 14:00:00 CET 2013                                         # */
/* # Copyright or (C) or Copr. Commissariat a l'Energie Atomique          # */
/* #                                                                      # */
/* # This software is governed by the CeCILL-C license under French law   # */
/* # and abiding by the rules of distribution of free software.  You can  # */
/* # use, modify and/ or redistribute the software under the terms of     # */
/* # the CeCILL-C license as circulated by CEA, CNRS and INRIA at the     # */
/* # following URL http://www.cecill.info.                                # */
/* #                                                                      # */
/* # The fact that you are presently reading this means that you have     # */
/* # had knowledge of the CeCILL-C license and that you accept its        # */
/* # terms.                                                               # */
/* #                                                                      # */
/* # Authors:                                                             # */
/* #   - BESNARD Jean-Baptiste jean-baptiste.besnard@cea.fr               # */
/* #                                                                      # */
/* ######################################################################## */
#include <svut.h>
#include <iostream>
#include <stdint.h>

using namespace svUnitTest;
#include "MALP_Blackboard.h"
#include "Hash_Table.h"

using namespace std;


#define EVT_SIZE 5242880llu
//52428800llu

struct event_set
{
	uint64_t values[EVT_SIZE];
};


struct accum
{
	uint64_t hits;
	uint64_t sum;
};



enum data_entries
{
	EVENT,
	DISP_EVENT,
	ACCUM,
	LAST_ACC,
	RED_ACC
};


enum ks
{
	KS_DISPATCH,
	KS_REDUCE,
	KS_ACCUM
};


void ks_dispatcher(  struct MALP_blackboard *bb, struct Data_entry **de, uint32_t entry_count, void *arg )
{
	if( !MALP_Blackboard_writable_data( de[0] ) )
	{
		printf("Dispatched event is not writable \n");
		abort();
	}

	int head = rand()%bb->cpu_count;
	de[0]->key = MALP_Blackboard_type_state_hash(DISP_EVENT, head );

	if( !MALP_Blackboard_submit_data_entry( bb, de[0] ) )
	{
		printf("Event has no sensivity\n");
	}
}


void ks_accumulator(  struct MALP_blackboard *bb, struct Data_entry **de, uint32_t entry_count, void *arg )
{
	struct event_set *es = (struct event_set *)de[0]->payload;
	struct accum *acc = (struct accum *)de[1]->payload;

	if( !MALP_Blackboard_writable_data( de[1] ) )
	{
		printf("Accumulator is not writable %d\n", MALP_Blackboard_writable_data( de[1] ) );
		abort();
	}

	//printf("In accumulator %p(%d, %d) event %p (%d,%d) size %d acc %p\n", de[1], de[1]->type, de[1]->state , de[0], de[0]->type, de[0]->state, de[0]->size, acc);

	if( de[0]->size == 0 )
	{
		/* empty event */
		de[1]->key = MALP_Blackboard_type_hash( LAST_ACC );
		MALP_Blackboard_submit_data_entry( bb, de[1] );
	}
	else
	{
		es = (struct event_set *)de[0]->payload;
		acc->hits+=EVT_SIZE;
		uint64_t i = 0;
		for( i = 0 ; i < EVT_SIZE ; i++ )
		{
			acc->sum += es->values[i];
		}

		MALP_Blackboard_submit_data_entry( bb, de[1] );
	}
}


void ks_reduce(  struct MALP_blackboard *bb, struct Data_entry **de, uint32_t entry_count, void *arg )
{
	printf("In reduce with %d entries\n", entry_count);
	
	struct accum *root = (struct accum *)de[0]->payload;

	uint64_t i = 0;
	
	for( i = 1; i < entry_count ; i++ )
	{
		struct accum *pivot = (struct accum *)de[i]->payload;
		root->hits += pivot->hits;
		root->sum += pivot->sum;
	}

	de[0]->key = RED_ACC;
	MALP_Blackboard_submit_data_entry( bb, de[0] );
}


void ks_print(  struct MALP_blackboard *bb, struct Data_entry **de, uint32_t entry_count, void *arg )
{
	struct accum *root = (struct accum *)de[0]->payload;
	
	printf("Hits : %lld Sum : %lld \n", (long long int)root->hits, (long long int)root->sum );

}



SVUT_DECLARE_FLAT_TEST(Reduce_Array,Reduce)
{
	struct MALP_blackboard bb;
	
	MALP_blackboard_init( &bb, 8, NULL);

	/* Spawn event dispatcher */
	uint64_t disp_sens[1] = { MALP_Blackboard_type_hash( EVENT ) };
	MALP_Blackboard_new_ks( &bb, KS_DISPATCH, disp_sens, 1, ks_dispatcher, NULL );

	uint64_t i = 0;
	int ks_id = 0;
	/* Spawn accumulators */
	uint64_t acc_sens[2];

	for( i = 0 ; i < bb.cpu_count ; i++ )
	{
		acc_sens[0] = MALP_Blackboard_type_state_hash( DISP_EVENT, i );
		acc_sens[1] = MALP_Blackboard_type_state_hash( ACCUM, i);

		MALP_Blackboard_new_ks( &bb, ++ks_id, acc_sens, 2, ks_accumulator, NULL );

		struct accum acc = { 0, 0 };
		MALP_Blackboard_submit_data( &bb, MALP_Blackboard_type_state_hash(ACCUM, i), (void *)&acc, sizeof( struct accum )  );
	}


	/* Spawn reduce array */
	uint64_t red_sens[3];

	for( i = 0 ; i < bb.cpu_count ; i++ )
	{
		int tot = 1;
		uint64_t cpu_id = i + 1;
		
		red_sens[0] = MALP_Blackboard_type_state_hash( LAST_ACC, i );
		
		if( 2 * cpu_id <= bb.cpu_count )
		{
			red_sens[1] = MALP_Blackboard_type_state_hash( RED_ACC, 2 * cpu_id - 1 );
			tot++;
		}

		if( 2 * cpu_id + 1<= bb.cpu_count )
		{
			red_sens[2] = MALP_Blackboard_type_state_hash( RED_ACC, 2 * cpu_id );
			tot++;
		}

		printf(" ID : %lld  LEFT : %lld  RIGHT : %lld  ==> TOT : %lld \n", (long long int)i, (long long int)(2 * cpu_id - 1), (long long int)(2 * cpu_id) , (long long int)tot );

		MALP_Blackboard_new_ks( &bb, 1024+i, red_sens, tot, ks_reduce, NULL );
	}
	
	/* Spawn printer */
	uint64_t print_sens[1];
	print_sens[0] = MALP_Blackboard_type_hash( RED_ACC );
	MALP_Blackboard_new_ks( &bb, ++ks_id, print_sens, 1, ks_print, NULL );
	
	struct event_set *es = (struct event_set *)malloc( sizeof( struct event_set ) );
	
	for( i = 0 ; i < EVT_SIZE ; i++ )
	{
		es->values[i] = 1;
	}
	
	uint64_t tot_size = 0;

	for( i = 0 ; i < 20 ; i++ )
	{
		MALP_Blackboard_submit_data( &bb, MALP_Blackboard_type_hash(EVENT), (void *)es, sizeof( struct event_set )  );
		tot_size += EVT_SIZE * 8;
		printf("Processed %g MB\n", (double)tot_size/(1024*1024));
	}

	
	MALP_Blackboard_wait_pending( &bb, bb.cpu_count );

	for( i = 0 ; i < bb.cpu_count ; i++ )
	{
		MALP_Blackboard_submit_data( &bb, MALP_Blackboard_type_state_hash(DISP_EVENT, i), NULL, 0  );
	}


	MALP_blackboard_wait( &bb );

	MALP_blackboard_release( &bb );
}
