#include <stdio.h>
#include <string.h>

#include "Shared_Cache.h"

#include "Utils.h"
#include "VMPI_stream.h"
#include "VMPI.h"
#include "Trace_Collector.h"
#include "Meta_Data_Handler.h"
#include "MALP_Config.h"

//#include "MALP_MPI_T_instr.h"

static void display_help()
{
	printf("malp_an usage :\n");
	printf("malp_an -w  #to write defaut configuration file\n");
	printf("malp_an  #to launch the analysis using the in-dir config:\n");
	printf("malp_an -c FILE  #to use the given config file\n");
}


static int setup_config( int argc, char **argv )
{
	TA_Conf_init_default();
	
	int rank = -1;
	PMPI_Comm_rank( VMPI_Get_partition_comm(), &rank );

	if( ! rank )
			TA_Conf_init_from_env();


	//Write config
	if ( argc == 2 ) {
		if ( !strcasecmp(argv[1], "-w") || !strcasecmp(argv[1], "--write") ) {
			if ( rank == 0 ) {
				printf("Writing config to ./malp_an.conf\n");
				TA_Conf_dump( "./malp_an.conf" );
			}
			return 0;
		} else if ( !strcasecmp(argv[1], "-h") || !strcasecmp(argv[1], "--help") ) {
			display_help();
			return 0;
		} else {
			if ( rank == 0 ) {
				printf("Unkwown option %s\n", argv[1]);
				display_help();
			}
			return 0;
		}
	}
	else if ( argc == 3 )
	{ //Force Config
		if ( !strcasecmp(argv[1], "-c") || !strcasecmp(argv[1], "--config") )
		{
			if ( rank == 0 )
			{
				if ( MALP_Trace_file_exists(argv[2]) )
				{
					TA_Conf_load( argv[2] );
				}
				else
				{
					printf("Could not locate file : %s \n", argv[2] );
					return 0;
				}
			}
		}
		else 
		{
			if ( rank == 0 ) {
				printf("Unkwown option %s\n", argv[1]);
				display_help();
			}
			return 0;
		}
	} 
	else if ( argc  == 1 ) 
	{ //Only root loads config then boadcasts it

		if ( rank == 0 )
		{
			
			if ( MALP_Trace_file_exists( "malp_an.conf" ))
			{
				TA_Conf_load( "./malp_an.conf" );
			} 

		}

	}
	return 1;
}


int analyzer_main( int argc, char **argv )
{
        int err, __mpi_t_provided;
	
	if( !setup_config( argc, argv ) )
	{
		return 1;
	}

	TA_Conf_dispatch();
	
	Timer_Init_scale(sleep);

	Shared_Cache_init();

#if HAVE_MPI_T
        malp_mpi_t_init_instrum( MPI_THREAD_SERIALIZED );
#endif /* HAVE_MPI_T */

	Trace_Collector_init();
	
	Trace_Collector_read_loop();
	
	Trace_Collector_release();

	PMPI_Barrier( VMPI_Get_partition_comm() );

	Shared_Cache_wait();

	if( Shared_Cache_is_in_trace_mode() )
	{
		Shared_Cache_to_file( "./MALP_trace.json" );
	}
	
	Shared_Cache_release();
	
	return 0;
}
