/* ############################ MALP License ############################## */
/* # Fri Jan 18 14:00:00 CET 2013                                         # */
/* # Copyright or (C) or Copr. Commissariat a l'Energie Atomique          # */
/* #                                                                      # */
/* # This software is governed by the CeCILL-C license under French law   # */
/* # and abiding by the rules of distribution of free software.  You can  # */
/* # use, modify and/ or redistribute the software under the terms of     # */
/* # the CeCILL-C license as circulated by CEA, CNRS and INRIA at the     # */
/* # following URL http://www.cecill.info.                                # */
/* #                                                                      # */
/* # The fact that you are presently reading this means that you have     # */
/* # had knowledge of the CeCILL-C license and that you accept its        # */
/* # terms.                                                               # */
/* #                                                                      # */
/* # Authors:                                                             # */
/* #   - BESNARD Jean-Baptiste jean-baptiste.besnard@cea.fr               # */
/* #                                                                      # */
/* ######################################################################## */
#ifndef Array_H
#define Array_H

#include <stdio.h>
#include <stdint.h>
#include <string.h>
#include <stdlib.h>
#include <mpi.h>

#include "CRC64.h"
#include "Red_HT.h"
#include "jsonCache.h"

/**
 * @ingroup Array_
 * @brief Define the structure of a Array
 *
 */
struct Array {
    int  __attribute__ ((aligned (64))) dimension;  /**< @brief ... */
    int  __attribute__ ((aligned (64))) Dx;         /**< @brief ... */
    int  __attribute__ ((aligned (64))) Dy;         /**< @brief ... */
    int  __attribute__ ((aligned (64))) Dz;         /**< @brief ... */
    int  __attribute__ ((aligned (64))) Dt;         /**< @brief ... */
    int  __attribute__ ((aligned (64))) is_sparse;  /**< @brief ... */

	/* Section for sparse Array */
    struct Red_Ht rht;                          /**< @brief Sparse Array (hashtable) */
	/*----------------------------*/

	/* Section for full Array */
    double  *datas;                           /**< @brief Full Array */
	/*----------------------------*/


	/* Misc. descriptors */
    int has_datas;                         /**< @brief Data presence flag */


    double min;       /**< @brief  */
    double max;       /**< @brief  */
    double moy;       /**< @brief  */
    double tmin;      /**< @brief  */
    double tmax;      /**< @brief  */

    uint8_t cmin;       /**< @brief  */
    uint8_t cmax;       /**< @brief  */
    uint8_t cmoy;       /**< @brief  */
    uint8_t ctmin;      /**< @brief  */
    uint8_t ctmax;      /**< @brief  */


};


/**
 * @ingroup Array_
 * @brief Flag for strong check of Arrays
 *
 */
#define STRONG_CHECK_ARRAY 1

/**
 * @ingroup Array_
 * @brief Flag for forcing CRC collision check of Arrays
 *
 */
#define FORCE_CRC_COLLISION_CHEK_ARRAY 1



/*-----------------------------------------------------------
Sparcity Handling
-----------------------------------------------------------*/
/**
 * @ingroup Array_
 * @brief Define a one dimension Array structure
 *
 */
struct Array_1D_point
{
	uint64_t key;
	int x;
	double value;
};


/**
 * @ingroup Array_
 * @brief Reduction of a one dimension point by adding pb's value to pa's value (pa and pb being Array_1D_point)
 * @param pa
 * @param pb
 */
void Array_1D_point_reduce(void *pa, void *pb);

/**
 *  @ingroup Array_
 * @brief Compare 2 dimensions to check if they are different or not
 * (with their key only by default, also with their dimensions if forced CRC control id activated).
 * @param pa
 * @param key
 * @param pc
 * @return (0: key is not equal to pa's key or in case of forced control, if pa and pc's dimensions are different; 1: otherwise)
 */
int Array_1D_point_test(void *pa, uint64_t key, void *pc);
/**
 *  @ingroup Array_
 * @brief Getter for the key of a 1 dimension Array
 * @param pa
 * @return pa's key
 */
uint64_t Array_1D_point_key(void *pa);


/**
 * @ingroup Array_
 * @brief Define a two dimensions Array structure
 *
 */
struct Array_2D_point
{
	uint64_t key;
	int x;
	int y;
	double value;
};

/**
 * @ingroup Array_
 * @brief Reduction of a two dimensions point by adding pb's value to pa's value (pa and pb being Array_2D_point)
 * @param pa
 * @param pb
 */
void Array_2D_point_reduce(void *pa, void *pb);

/**
 * @ingroup Array_
 * @brief Compare 2 Arrays dimensions to check if they are different or not
 * (with their key only by default, also with their dimensions if forced CRC control id activated).
 * @param pa
 * @param key
 * @param pc
 * @return (0: key is not equal to pa's key or in case of forced control, if pa and pc's dimensions are different; 1: otherwise)
 */
int Array_2D_point_test(void *pa, uint64_t key, void *pc);

/**
 * @ingroup Array_
 * @brief Getter for the key of a 2 dimensions Array
 * @param pa
 * @return pa's key
 */
uint64_t Array_2D_point_key(void *pa);


/**
 * @ingroup Array_
 * @brief Define a three dimensions Array structure
 *
 */
struct Array_3D_point
{
	uint64_t key;
	int x;
	int y;
	int z;
	double value;
};

/**
 * @ingroup Array_
 * @brief Reduction of a three dimensions point by adding pb's value to pa's value (pa and pb being Array_3D_point)
 * @param pa
 * @param pb
 */
void Array_3D_point_reduce(void *pa, void *pb);

/**
 *  @ingroup Array_
 * @brief Compare 2 Arrays dimensions to check if they are different or not
 * (with their key only by default, also with their dimensions if forced CRC control id activated).
 * @param pa
 * @param key
 * @param pc
 * @return (0: key is not equal to pa's key or in case of forced control, if pa and pc's dimensions are different; 1: otherwise)
 */
int Array_3D_point_test(void *pa, uint64_t key, void *pc);

/**
 *  @ingroup Array_
 * @brief Getter for the key of a 3 dimensions Array
 * @param pa
 * @return pa's key
 */
uint64_t Array_3D_point_key(void *pa);


/**
 * @ingroup Array_
 * @brief Define a four dimensions Array structure
 *
 */
struct Array_4D_point
{
	uint64_t key;
	int x;
	int y;
	int z;
	int t;
	double value;
};

/**
 * @ingroup Array_
 * @brief Reduction of a four dimensions point by adding pb's value to pa's value (pa and pb being Array_4D_point)
 * @param pa
 * @param pb
 */
void Array_4D_point_reduce(void *pa, void *pb);

/**
 *  @ingroup Array_
 * @brief Compare 2 Arrays dimensions to check if they are different or not
 * (with their key only by default, also with their dimensions if forced CRC control id activated).
 * @param pa
 * @param key
 * @param pc
 * @return (0: key is not equal to pa's key or in case of forced control, if pa and pc's dimensions are different; 1: otherwise)
 */
int Array_4D_point_test(void *pa, uint64_t key, void *pc);

/**
 *  @ingroup Array_
 * @brief Getter for the key of a 4 dimensions Array
 * @param pa
 * @return pa's key
 */
uint64_t Array_4D_point_key(void *pa);

/**
 * @ingroup Array_
 * @brief Number of elements in a Array
 */
#define Array_COLL_AVOID 64

/**
 * @ingroup Array_
 * @brief Array cell structure containing a one dimension Array list
 *
 */
struct Array_1D_point_cell
{
	struct Hash_Table_Internal ___;
	struct Array_1D_point points[Array_COLL_AVOID];
};

/**
 * @ingroup Array_
 * @brief Array cell structure containing a two dimensions Array list
 *
 */
struct Array_2D_point_cell
{
	struct Hash_Table_Internal ___;
	struct Array_2D_point points[Array_COLL_AVOID];
};

/**
 * @ingroup Array_
 * @brief Array cell structure containing a three dimensions Array list
 *
 */
struct Array_3D_point_cell
{
	struct Hash_Table_Internal ___;
	struct Array_3D_point points[Array_COLL_AVOID];
};

/**
 * @ingroup Array_
 * @brief Array cell structure containing a four dimensions Array list
 *
 */
struct Array_4D_point_cell
{
	struct Hash_Table_Internal ___;
	struct Array_4D_point points[Array_COLL_AVOID];
};


/*-----------------------------------------------------------
Interface
-----------------------------------------------------------*/
/**
 *  @ingroup Array_
 * @brief Initialise a 1 dimension Array
 * @param dts
 * @param Dx
 * @param is_sparse
 * @return Initialisation status (0: success, 1: error)
 */
int Array_init_1D( struct Array *dts, int Dx, int is_sparse );
/**
 *  @ingroup Array_
 * @brief Initialise a 2 dimensions Array
 * @param dts
 * @param Dx
 * @param Dy
 * @param is_sparse
 * @return Initialisation status (0: success, 1: error)
 */
int Array_init_2D( struct Array *dts, int Dx , int Dy , int is_sparse );
/**
 *  @ingroup Array_
 * @brief Initialise a 3 dimensions Array
 * @param dts
 * @param Dx
 * @param Dy
 * @param Dz
 * @param is_sparse
 * @return Initialisation status (0: success, 1: error)
 */
int Array_init_3D( struct Array *dts, int Dx , int Dy, int Dz, int is_sparse );

/**
 *  @ingroup Array_
 * @brief Initialise a 4 dimensions Array
 * @param dts
 * @param Dx
 * @param Dy
 * @param Dz
 * @param Dt
 * @param is_sparse
 * @return Initialisation status (0: success, 1: error)
 */
int Array_init_4D( struct Array *dts, int Dx , int Dy, int Dz, int Dt , int is_sparse );

/**
 *  @ingroup Array_
 * @brief Array_copy
 * @param dts
 * @param source
 * @return
 */
int Array_copy( struct Array *dts, struct Array *source );
/**
 *  @ingroup Array_
 * @brief Array_copy_datas
 * @param dts
 * @param source
 */
void Array_copy_datas(  struct Array *dts, struct Array *source );
/**
 *  @ingroup Array_
 * @brief Call Array_dim_agnostic_set for all the dimensions of a Array
 * @param dts
 */
void Array_zero_datas( struct Array *dts );

/**
 *  @ingroup Array_
 * @brief Copy the limit values of a source Array to another Array
 * @param dts
 * @param src
 */
void Array_copy_limits( struct Array *dts , struct Array *src );

/**
 *  @ingroup Array_
 * @brief Set all limit values of a Array to 0
 * @param dts
 */
void Array_zero_limits( struct Array *dts );


/**
 *  @ingroup Array_
 * @brief Array memory release
 * @param dts
 */
void Array_release( struct Array *dts );


/**
 *  @ingroup Array_
 * @brief Reduce a Array's data using a MPI operation
 * @param dts
 * @param op
 */
void Array_reduce( struct Array *dts, MPI_Op op );

/**
 *  @ingroup Array_
 * @brief Get a pointer for the new Array passed as a parameter, filled with the data corresponding to the coordinates passed as parameters.
 * @param dts
 * @param x
 * @param y
 * @param z
 * @param t
 * @return a pointer to the Array , or 0
 */
double Array_dim_agnostic_get( struct Array *dts , int x, int y, int z, int t );

/**
 *  @ingroup Array_
 * @brief Array_dim_agnostic_set
 * @param dts
 * @param value
 * @param x
 * @param y
 * @param z
 * @param t
 */
void Array_dim_agnostic_set( struct Array *dts , double value, int x, int y, int z, int t );


/**
 *  @ingroup Array_
 * @brief Array_count_not_null_2D
 * @param dts
 * @return
 */
uint64_t Array_count_not_null_2D( struct Array *dts  );

/**
 *  @ingroup Array_
 * @brief Array_get_get_max_offset_1D
 * @param dts
 * @return
 */
int Array_get_get_max_offset_1D( struct Array *dts );
/**
 *  @ingroup Array_
 * @brief Array_get_get_max_offset_2D_col
 * @param dts
 * @param col
 * @param set_zero
 * @return
 */
int Array_get_get_max_offset_2D_col( struct Array *dts, int col, int set_zero );
/**
 *  @ingroup Array_
 * @brief Array_get_get_max_offset_2D_line
 * @param dts
 * @param line
 * @param set_zero
 * @return
 */
int Array_get_get_max_offset_2D_line( struct Array *dts, int line , int set_zero);
/**
 *  @ingroup Array_
 * @brief Array_get_min_aggregate_2d
 * @param dts
 * @return
 */
double Array_get_min_aggregate_2d (struct Array * dts);
/**
 *  @ingroup Array_
 * @brief Array_get_max_aggregate_2d
 * @param dts
 * @return
 */
double Array_get_max_aggregate_2d (struct Array * dts);
/**
 *  @ingroup Array_
 * @brief Array_get_min_but_not_zero
 * @param dts
 * @return
 */
double Array_get_min_but_not_zero (struct Array * dts);
/**
 *  @ingroup Array_
 * @brief Array_get_min
 * @param dts
 * @return
 */
double Array_get_min (struct Array * dts);
/**
 *  @ingroup Array_
 * @brief Array_get_max
 * @param dts
 * @return
 */
double Array_get_max (struct Array * dts);
/**
 *  @ingroup Array_
 * @brief Array_get_moy
 * @param dts
 * @return
 */
double Array_get_moy (struct Array * dts);
/**
 *  @ingroup Array_
 * @brief Array_get_sum
 * @param dts
 * @return
 */
double Array_get_sum (struct Array * dts);


/**
 *  @ingroup Array_
 * @brief Array_normalize
 * @param dts
 * @param target_max
 */
void Array_normalize( struct Array * dts , double target_max );
/**
 *  @ingroup Array_
 * @brief Array_reset_bounds
 * @param dts
 */
void Array_reset_bounds( struct Array *dts );

/**
 *  @ingroup Array_
 * @brief Array_test
 */
void Array_test();


/**
 *
 * @brief Enum of Array kernel types.
 */
typedef enum {
    Array_GAUSSIAN_KERNEL,
    Array_GAUSSIAN_HORIZONTAL_KERNEL,
    Array_LAPLACIAN_OF_GAUSSIAN_KERNEL
} Array_Conv_kernel;


/**
 *  @ingroup Array_
 * @brief Array_Convolve
 * @param dest
 * @param dts
 * @param passes
 * @param k
 */
void Array_Convolve( struct Array *dest, struct Array *dts, int passes, Array_Conv_kernel k );


/*-----------------------------------------------------------
POINTERS OPs
-----------------------------------------------------------*/
/**
 *  @ingroup Array_
 * @brief Array_compute_key
 * @param x
 * @param y
 * @param z
 * @param t
 * @param dimension
 * @return
 */
static inline uint64_t Array_compute_key(int x, int y, int z, int t, int dimension )
{
	int acc[4] = { x, y ,z ,t };

	return MALP_Trace_crc64( (char *)acc , sizeof(int) * dimension );
}


/**
 *  @ingroup Array_
 * @brief Array_get_ptr_1D
 * @param dts
 * @param x
 * @return
 */
static inline double * Array_get_ptr_1D (struct Array * dts, int x)
{
#ifdef STRONG_CHECK_ARRAY
	if( dts->dimension != 1 ) {
		printf("Wrong dimension awaiting 1 got %d!\n", dts->dimension);
		abort();
	}

	if( (x < 0) || ( dts->Dx < x ) ) {
		printf("Wrong index x %d over %d !\n", x, dts->Dx );
		abort();
	}
#endif

	if(dts->is_sparse)
	{
		uint64_t key = Array_compute_key( x, 0, 0, 0, 1 );

		struct Array_1D_point test_c;
#if FORCE_CRC_COLLISION_CHEK_ARRAY
		test_c.x = x;
#endif
		struct Array_1D_point * point = (struct Array_1D_point *)
											Red_Ht_Table_get( &dts->rht, key , &test_c, Array_1D_point_test);

		if( point )
		{
			return &point->value;
		}
		else
		{
			return NULL;
		}

	}
	else
	{
		return &dts->datas[x];
	}
}

/**
 *  @ingroup Array_
 * @brief Array_get_ptr_2D
 * @param dts
 * @param x
 * @param y
 * @return
 */
static inline double * Array_get_ptr_2D (struct Array * dts, int x, int y)
{
#ifdef STRONG_CHECK_ARRAY
	if( dts->dimension != 2 ) {
		printf("Wrong dimension awaiting 2 !\n");
		abort();
	}

	if( (x < 0) || ( dts->Dx < x ) ) {
		printf("Wrong index x %d over %d !\n", x, dts->Dx );
		abort();
	}

	if( (y < 0) || ( dts->Dy < y ) ) {
		printf("Wrong index y %d over %d !\n", y, dts->Dy );
		abort();
	}
#endif

	if(dts->is_sparse)
	{
		uint64_t key = Array_compute_key( x, y, 0, 0, 2 );

		struct Array_2D_point test_c;

#if FORCE_CRC_COLLISION_CHEK_ARRAY
		test_c.x = x;
		test_c.y = y;
#endif
		struct Array_2D_point * point = (struct Array_2D_point *)
											Red_Ht_Table_get( &dts->rht, key , &test_c, Array_2D_point_test);

		if( point )
		{
			return &point->value;
		}
		else
		{
			return NULL;
		}

	}
	else
	{
		return &dts->datas[x + dts->Dx * y];
	}

}


/**
 * @ingroup Array_
 * @brief Array_get_ptr_3D
 * @param dts
 * @param x
 * @param y
 * @param z
 * @return
 */
static inline double * Array_get_ptr_3D (struct Array * dts, int x, int y, int z)
{
#ifdef STRONG_CHECK_ARRAY
	if( dts->dimension != 3 ) {
		printf("Wrong dimension awaiting 3 !\n");
		abort();
	}

	if( (x < 0) || ( dts->Dx < x ) ) {
		printf("Wrong index x %d over %d !\n", x, dts->Dx );
		abort();
	}

	if( (y < 0) || ( dts->Dy < y ) ) {
		printf("Wrong index y %d over %d !\n", y, dts->Dy );
		abort();
	}

	if( (z < 0) || ( dts->Dz < z ) ) {
		printf("Wrong index z %d over %d !\n", z, dts->Dz );
		abort();
	}
#endif

	if(dts->is_sparse)
	{
		uint64_t key = Array_compute_key( x, y, z, 0, 3 );

		struct Array_3D_point test_c;
#if FORCE_CRC_COLLISION_CHEK_ARRAY

		test_c.x = x;
		test_c.y = y;
		test_c.z = z;
#endif
		struct Array_3D_point * point = (struct Array_3D_point *)
											Red_Ht_Table_get( &dts->rht, key , &test_c, Array_3D_point_test);

		if( point )
		{
			return &point->value;
		}
		else
		{
			return NULL;
		}

	}
	else
	{
		return &dts->datas[x + dts->Dx * y + (dts->Dx*dts->Dy) * z ];
	}

}


/**
 * @ingroup Array_
 * @brief Array_get_ptr_4D
 * @param dts
 * @param x
 * @param y
 * @param z
 * @param t
 * @return
 */
static inline double * Array_get_ptr_4D (struct Array * dts, int x, int y, int z, int t)
{
#ifdef STRONG_CHECK_ARRAY
	if( dts->dimension != 4 ) {
		printf("Wrong dimension awaiting 4 !\n");
		abort();
	}

	if( (x < 0) || ( dts->Dx < x ) ) {
		printf("Wrong index x %d over %d !\n", x, dts->Dx );
		abort();
	}

	if( (y < 0) || ( dts->Dy < y ) ) {
		printf("Wrong index y %d over %d !\n", y, dts->Dy );
		abort();
	}

	if( (z < 0) || ( dts->Dz < z ) ) {
		printf("Wrong index z %d over %d !\n", z, dts->Dz );
		abort();
	}

	if( (t < 0) || ( dts->Dt < t ) ) {
		printf("Wrong index t %d over %d !\n", t, dts->Dt );
		abort();
	}
#endif

	if(dts->is_sparse)
	{
		uint64_t key = Array_compute_key( x, y, z, t, 4 );

		struct Array_4D_point test_c;
#if FORCE_CRC_COLLISION_CHEK_ARRAY
		test_c.x = x;
		test_c.y = y;
		test_c.z = z;
		test_c.t = t;
#endif
		struct Array_4D_point * point = (struct Array_4D_point *)
											Red_Ht_Table_get( &dts->rht, key , &test_c, Array_4D_point_test);

		if( point )
		{
			return &point->value;
		}
		else
		{
			return NULL;
		}

	}
	else
	{
		return &dts->datas[x + dts->Dx * y + (dts->Dx*dts->Dy) * z   + (dts->Dx*dts->Dy*dts->Dz) * t];
	}

}


/*-----------------------------------------------------------
MISCS
-----------------------------------------------------------*/
/**
 * @ingroup Array_
 * @brief Array_has_datas
 * @param dts
 * @return
 */
static inline double Array_has_datas( struct Array * dts )
{
	return dts->has_datas;
}

/*-----------------------------------------------------------
GETTERS
-----------------------------------------------------------*/
/**
 * @ingroup Array_
 * @brief Array_get_1D
 * @param dts
 * @param x
 * @return
 */
static inline double Array_get_1D (struct Array * dts, int x)
{
	double *ret = Array_get_ptr_1D (dts, x);

	if(ret)
		return *ret;
	else
		return 0.0;
}

/**
 * @ingroup Array_
 * @brief Array_get_2D
 * @param dts
 * @param x
 * @param y
 * @return
 */
static inline double Array_get_2D (struct Array * dts, int x, int y)
{
	double *ret = Array_get_ptr_2D (dts, x, y);

	if(ret)
		return *ret;
	else
		return 0.0;
}

/**
 * @ingroup Array_
 * @brief Array_get_3D
 * @param dts
 * @param x
 * @param y
 * @param z
 * @return
 */
static inline double Array_get_3D (struct Array * dts, int x, int y, int z)
{
	double *ret = Array_get_ptr_3D (dts, x, y, z);

	if(ret)
		return *ret;
	else
		return 0.0;
}

/**
 * @ingroup Array_
 * @brief Array_get_4D
 * @param dts
 * @param x
 * @param y
 * @param z
 * @param t
 * @return
 */
static inline double Array_get_4D (struct Array * dts, int x, int y, int z, int t)
{
	double *ret = Array_get_ptr_4D (dts, x, y, z, t);

	if(ret)
		return *ret;
	else
		return 0.0;
}

/*-----------------------------------------------------------
INC / DEC
-----------------------------------------------------------*/
/**
 * @ingroup Array_
 * @brief Array_push_1D_point
 * @param dts
 * @param x
 * @param val
 */
static inline void Array_push_1D_point(struct Array * dts, int x, double val)
{
	struct Array_1D_point p;

	p.key = Array_compute_key( x, 0, 0, 0, 1 );
	p.x = x;
	p.value = val;

	Red_Ht_Table_push( &dts->rht, &p , p.key );
}

/**
 * @ingroup Array_
 * @brief Array_push_2D_point
 * @param dts
 * @param x
 * @param y
 * @param val
 */
static inline void Array_push_2D_point(struct Array * dts, int x, int y, double val)
{
	struct Array_2D_point p;

	p.key = Array_compute_key( x, y, 0, 0, 2 );
	p.x = x;
	p.y = y;
	p.value = val;

	Red_Ht_Table_push( &dts->rht, &p , p.key );
}
/**
 * @ingroup Array_
 * @brief Array_push_3D_point
 * @param dts
 * @param x
 * @param y
 * @param z
 * @param val
 */
static inline void Array_push_3D_point(struct Array * dts, int x, int y , int z, double val)
{
	struct Array_3D_point p;

	p.key = Array_compute_key( x, y, z, 0, 3 );
	p.x = x;
	p.y = y;
	p.z = z;
	p.value = val;

	Red_Ht_Table_push( &dts->rht, &p , p.key );
}
/**
 * @ingroup Array_
 * @brief Array_push_4D_point
 * @param dts
 * @param x
 * @param y
 * @param z
 * @param t
 * @param val
 */
static inline void Array_push_4D_point(struct Array * dts, int x, int y, int z, int t, double val)
{
	struct Array_4D_point p;

	p.key = Array_compute_key( x, y, z, t, 4 );
	p.x = x;
	p.y = y;
	p.z = z;
	p.t = t;
	p.value = val;

	Red_Ht_Table_push( &dts->rht, &p , p.key );
}

/**
 * @ingroup Array_
 * @brief Array_inc_1D
 * @param dts
 * @param x
 */
static inline void Array_inc_1D (struct Array * dts, int x)
{
	dts->has_datas++;
	double *ret = Array_get_ptr_1D (dts, x);

	if( ret )
	{
		*ret = *ret + 1.0;
	}
	else
	{
		Array_push_1D_point( dts, x, 1);
	}
}

/**
 * @ingroup Array_
 * @brief Array_inc_2D
 * @param dts
 * @param x
 * @param y
 */
static inline void Array_inc_2D (struct Array * dts, int x, int y)
{
	dts->has_datas++;
	double *ret = Array_get_ptr_2D (dts, x, y);

	if( ret )
	{
		*ret = *ret + 1;
	}
	else
	{
		Array_push_2D_point( dts, x, y, 1);
	}
}

/**
 * @ingroup Array_
 * @brief Array_inc_3D
 * @param dts
 * @param x
 * @param y
 * @param z
 */
static inline void Array_inc_3D (struct Array * dts, int x, int y, int z)
{
	dts->has_datas++;
	double *ret = Array_get_ptr_3D (dts, x, y, z);

	if( ret )
	{
		*ret = *ret + 1;
	}
	else
	{
		Array_push_3D_point( dts, x, y, z, 1);
	}
}

/**
 * @ingroup Array_
 * @brief Array_inc_4D
 * @param dts
 * @param x
 * @param y
 * @param z
 * @param t
 */
static inline void Array_inc_4D (struct Array * dts, int x, int y, int z, int t)
{
	dts->has_datas++;
	double *ret = Array_get_ptr_4D (dts, x, y, z, t);

	if( ret )
	{
		*ret = *ret + 1;
	}
	else
	{
		Array_push_4D_point( dts, x, y, z, t, 1);
	}
}

/**
 * @ingroup Array_
 * @brief Array_inc_val_1D
 * @param dts
 * @param val
 * @param x
 */
static inline void Array_inc_val_1D (struct Array * dts, double val, int x)
{
	dts->has_datas++;
	double *ret = Array_get_ptr_1D (dts, x );

	if( ret )
	{
		*ret = *ret + val;
	}
	else
	{
		Array_push_1D_point( dts, x, val);
	}
}

/**
 * @ingroup Array_
 * @brief Array_inc_val_2D
 * @param dts
 * @param val
 * @param x
 * @param y
 */
static inline void Array_inc_val_2D (struct Array * dts, double val, int x, int y)
{
	dts->has_datas++;
	double *ret = Array_get_ptr_2D (dts, x, y );

	if( ret )
	{
		*ret = *ret + val;
	}
	else
	{
		Array_push_2D_point( dts, x, y, val);
	}
}

/**
 * @ingroup Array_
 * @brief Array_inc_val_3D
 * @param dts
 * @param val
 * @param x
 * @param y
 * @param z
 */
static inline void Array_inc_val_3D (struct Array * dts, double val, int x, int y, int z)
{
	dts->has_datas++;
	double *ret = Array_get_ptr_3D (dts, x, y, z );

	if( ret )
	{
		*ret = *ret + val;
	}
	else
	{
		Array_push_3D_point( dts, x, y, z, val);
	}
}

/**
 * @ingroup Array_
 * @brief Array_inc_val_4D
 * @param dts
 * @param val
 * @param x
 * @param y
 * @param z
 * @param t
 */
static inline void Array_inc_val_4D (struct Array * dts, double val, int x, int y, int z, int t)
{
	dts->has_datas++;
	double *ret = Array_get_ptr_4D (dts, x, y, z, t );

	if( ret )
	{
		*ret = *ret + val;
	}
	else
	{
		Array_push_4D_point( dts, x, y, z, t, val);
	}
}

/**
 * @ingroup Array_
 * @brief Array_dec_1D
 * @param dts
 * @param x
 */
static inline void Array_dec_1D (struct Array * dts, int x)
{
	dts->has_datas++;
	double *ret = Array_get_ptr_1D (dts, x);

	if( ret )
	{
		if( *ret )
			*ret = *ret - 1;
	}
}

/**
 * @ingroup Array_
 * @brief Array_dec_2D
 * @param dts
 * @param x
 * @param y
 */
static inline void Array_dec_2D (struct Array * dts, int x, int y)
{
	dts->has_datas++;
	double *ret = Array_get_ptr_2D (dts, x, y);

	if( ret )
	{
		if( *ret )
			*ret = *ret - 1;
	}
}

/**
 * @ingroup Array_
 * @brief Array_dec_3D
 * @param dts
 * @param x
 * @param y
 * @param z
 */
static inline void Array_dec_3D (struct Array * dts, int x, int y, int z)
{
	dts->has_datas++;
	double *ret = Array_get_ptr_3D (dts, x, y, z);

	if( ret )
	{
		if( *ret )
			*ret = *ret - 1;
	}
}

/**
 * @ingroup Array_
 * @brief Array_dec_4D
 * @param dts
 * @param x
 * @param y
 * @param z
 * @param t
 */
static inline void Array_dec_4D (struct Array * dts, int x, int y, int z, int t)
{
	dts->has_datas++;
	double *ret = Array_get_ptr_4D (dts, x, y, z, t);

	if( ret )
	{
		if( *ret )
			*ret = *ret - 1;
	}
}

/*-----------------------------------------------------------
SETTERS
-----------------------------------------------------------*/
/**
 * @ingroup Array_
 * @brief Array_set_1D
 * @param dts
 * @param val
 * @param x
 */
static inline void Array_set_1D (struct Array * dts, double val, int x)
{
	dts->has_datas++;
	double *ret = Array_get_ptr_1D (dts, x );

	if( ret )
	{
			*ret = val;
	}
	else
	{
		if( val )
			Array_push_1D_point( dts, x, val);
	}
}

/**
 * @ingroup Array_
 * @brief Array_set_2D
 * @param dts
 * @param val
 * @param x
 * @param y
 */
static inline void Array_set_2D (struct Array * dts, double val, int x, int y)
{
	dts->has_datas++;
	double *ret = Array_get_ptr_2D (dts, x, y );

	if( ret )
	{
			*ret = val;
	}
	else
	{
		if( val )
			Array_push_2D_point( dts, x, y, val);
	}
}

/**
 * @ingroup Array_
 * @brief Array_set_3D
 * @param dts
 * @param val
 * @param x
 * @param y
 * @param z
 */
static inline void Array_set_3D (struct Array * dts, double val, int x, int y, int z)
{
	dts->has_datas++;
	double *ret = Array_get_ptr_3D (dts, x, y, z );

	if( ret )
	{
			*ret = val;
	}
	else
	{
		if( val )
			Array_push_3D_point( dts, x, y, z, val);
	}
}

/**
 * @ingroup Array_
 * @brief Array_set_4D
 * @param dts
 * @param val
 * @param x
 * @param y
 * @param z
 * @param t
 */
static inline void Array_set_4D (struct Array * dts, double val, int x, int y, int z, int t)
{
	dts->has_datas++;
	double *ret = Array_get_ptr_4D (dts, x, y, z, t );

	if( ret )
	{
			*ret = val;
	}
	else
	{
		if( val )
			Array_push_4D_point( dts, x, y, z, t, val);
	}
}





#endif /* Array */



