#ifndef COLLECTOR_H
#define COLLECTOR_H

#include <stdint.h>
#include <stdlib.h>


#include "Analyzer_Common.h"
#include "Meta_Data_Handler.h"
#include "Event_Desc.h"
#include "Shared_Cache.h"
#include "Event_Dispatcher.h"

struct Collector
{
	uint64_t partition_id;
	char name[128];
	char analysis_cache_name[128];
	MALP_Trace_event_type type;
	json_t * json;
	
	int (*init_coll)( struct Collector * coll );
	int (*release_coll)( struct Collector * coll );
	
	void (*push_event)(struct MALP_Trace_Event *, void *);
	int (*reduce)( struct Collector * coll );
	int (*accumulate)( struct Collector * coll );
	
	void *internal_data;
};

int Collector_init( struct Collector *coll, MALP_Trace_event_type type,
					uint64_t partition_id,
					char *name,
					int (*init_coll)( struct Collector * coll ),
					int (*release_coll)( struct Collector * coll ),
					void (*push_event)(struct MALP_Trace_Event *, void *),
					int (*reduce)( struct Collector * coll ),
					int (*accumulate)( struct Collector * coll ) );


int Collector_partition_size( struct Collector *coll );


static inline void Collector_init_coll( void *coll )
{
	if( ((struct Collector *)coll)->init_coll )
		(((struct Collector *)coll)->init_coll)( (struct Collector *)coll );
}

static inline void Collector_release_coll( void *coll )
{
	if( ((struct Collector *)coll)->release_coll )
		(((struct Collector *)coll)->release_coll)( (struct Collector *)coll );
}


static inline void Collector_reduce( void *coll )
{
	if( ((struct Collector *)coll)->reduce )
		(((struct Collector *)coll)->reduce)( (struct Collector *)coll );
}

static inline void Collector_accumulate( void *coll )
{
	if( ((struct Collector *)coll)->accumulate )
	{
		(((struct Collector *)coll)->accumulate)( (struct Collector *)coll );
	}
}


void _Collector_release( void *coll );

void Collector_release( struct Collector *coll );

void _Collector_push_event(struct MALP_Trace_Event *evt, void *pcoll);


#endif
