#ifndef __MALP_MPI_T_PROFILE_H__
#define __MALP_MPI_T_PROFILE_H__

#if HAVE_MPI_T

#include <stdint.h>

#include "MPIT_Events.h"
#include "Event_Desc.h" 
#include "Collector.h"
#include "Shared_Cache.h"
#include "Hash_Table.h"

typedef MPI_T_VARS_COUNTERS_TYPE
{
   MALP_MPI_T_INT,
   MALP_MPI_T_DOUBLE,

}

struct MPI_T_Profile_acc
{
	/* Its a bit redundant but saves some 
	 * allocation complexity for a few KB */
	uint64_t associated_tid;
	uint64_t *hits;
	uint64_t *size;
};


#define MPI_T_PROF_COLL_AVOID 2
struct MPI_T_Profile_acc_cell
{
	struct Hash_Table_Internal _____;
	struct MPI_T_Profile_acc accs[MPI_T_PROF_COLL_AVOID];
};

struct MPI_T_Profile
{
        /* CVARS */
        int mpit_cvars_tot;
        int mpit_cvars_not_const;
        int* mpit_cvars_not_const_ids;
        /* PVARS */
        int mpit_pvars_tot;
	/* TOTAL */
        int mpit_vars_tot;
        /* STANDARD STRUCT */
	struct MPI_T_Profile_acc global_acc;
	struct Hash_Table per_stream_profiles;
};

static inline uint64_t*
__MPI_T_pack_not_const_value( struct MPI_T_Profile *prof, uint64_t array, uint64_t* packed_array )
{
   int i;
   const int not_const_cvars = prof->mpit_cvars_not_const;

   assert( pack_array );

   for( i = 0; i < not_const_cvars; i++ )
   {
      const int translate_idx = mpit_cvars_not_const_ids[i];
      assert( translate_idx < prof->mpit_cvars_tot )
      pack_array[i] = array[translate_idx];
   }
}

static inline uint64_t*
__MPI_T_unpack_not_const_value( struct MPI_T_Profile *prof, uint64_t array, uint64_t* packed_array )
{
   int i;
   const int not_const_cvars = prof->mpit_cvars_not_const;

   assert( pack_array );

   for( i = 0; i < not_const_cvars; i++ )
   {
      const int translate_idx = mpit_cvars_not_const_ids[i];
      assert( translate_idx < prof->mpit_cvars_tot )
      array[translate_idx] = pack_array[i];
   }
}

void MPI_T_Profile_init(struct Collector *coll, uint64_t partition_id);

#endif /* HAVE_MPI_T */

#endif /* __MALP_MPI_T_PROFILE_H__ */
