#include "config.h" 

#if HAVE_MPI_T

#include "Meta_Data_Handler.h"
#include "Shared_Cache.h"
#include "Timer.h"
#include "Collector.h"

#include "mpi.h"
#include "MPIT_Events.h"
#include "MALP_MPI_T_data.h"


static json_t*
gen_mpit_static_cvar_profile( void )
{
   uint64_t __val; 
   char* __str_buffer;
   MPI_T_cvar_handle handle;
   int err, i, __var_count, count;
   MALP_mpi_t_var_info_t  *__var_ids;
   MALP_mpi_t_cvar_info_t *__cvar_data;
   json_t *__var_obj, *__tmp_var_val, *__var_array;

   __var_ids = __malp_mpi_t_get_var_ids( MALP_MPI_T_TYPE_STATIC_CVAR );
   __var_count = __malp_mpi_t_get_num_vars( MALP_MPI_T_TYPE_STATIC_CVAR );  
   assert( !__var_count || ( __var_count && __var_ids ) );
   
   __var_array = json_array();

   for( i = 0; i < __var_count; i++ )
   {
      __var_obj = json_object();

      __cvar_data = &( __var_ids[i].var.cvar ); 
      err = PMPI_T_cvar_handle_alloc( __cvar_data->absolute_id, NULL, &( handle ), &count );
      assert( err == MPI_SUCCESS );
      
      if( __cvar_data->mpi_type == MPI_CHAR )
      {
         __str_buffer = (char*) malloc( sizeof( char ) * ( count + 1 ) );
         assert( __str_buffer );
         err = PMPI_T_cvar_read( handle, __str_buffer );
         if( err != MPI_SUCCESS )
         {
            __tmp_var_val = json_string( "Error on PMPI_T_cvar_read" );
         }
         else
         {
            __tmp_var_val = json_string( __str_buffer );
         }
         free( __str_buffer ); __str_buffer = NULL;
      }
      else
      {
         assert( count == 1 );
         err = PMPI_T_cvar_read( handle, &( __val ) ); 
         assert( err == MPI_SUCCESS );
         if( __cvar_data->enum_type == MPI_T_ENUM_NULL ) 
         {
            const is_double = ( __cvar_data->mpi_type == MPI_DOUBLE ); 
            __tmp_var_val = (is_double) ? json_real( __val ) : json_int( __val );
         }
         else /* Translate int value to enum value */
         {
            char var_enum_name[1024];
            int var_enum_length = 1024; 
            int var_enum_max_elt, j;

            err = PMPI_T_enum_get_info( __cvar_data->enum_type, &var_enum_max_elt, var_enum_name, &var_enum_length );
            assert( err == MPI_SUCCESS );
            assert( var_enum_length < 1024 );
            var_enum_name[var_enum_length] = '\0';

            __tmp_var_val = json_string( var_enum_name );
            json_object_set( __var_obj, "name_enum", __tmp_var_val );
            json_decref( __tmp_var_val ); __tmp_var_val = NULL;              

            for( j = 0; j < var_enum_max_elt; j++ )
            {
               int __real_val;
               int var_enum_length = 1024;
	       err = PMPI_T_enum_get_item( __cvar_data->enum_type, j, &__real_val, var_enum_name, &var_enum_length );
               assert( err == MPI_SUCCESS );
               if( __real_val == (int) __val ) break;
            } 

            // ENUM VAL NOT FOUND
            if( j == var_enum_max_elt )            
               var_enum_length = snprintf( var_enum_name, 1024, "UNDEF ENUM VALUE >> %d <<", __val ); 

            assert( var_enum_length < 1024 );
            var_enum_name[var_enum_length] = '\0';
            __tmp_var_val = json_string( var_enum_name );
         }
      }

      err = PMPI_T_cvar_handle_free( &(handle) );
      assert( err == MPI_SUCCESS );

      json_object_set( __var_obj, "value", __tmp_var_val );
      json_decref( __tmp_var_val ); __tmp_var_val = NULL;

      __str_buffer = __malp_mpi_t_get_var_name( MALP_MPI_T_TYPE_STATIC_CVAR, i );
      assert( __str_buffer );
      __tmp_var_val = json_string( __str_buffer );      
      free( __str_buffer ); __str_buffer = NULL;
      json_object_set( __var_obj, "name", __tmp_var_val );
      json_decref( __tmp_var_val ); __tmp_var_val = NULL;

      __str_buffer = MALP_mpi_t_translate_MPI_T_VERBOSITY( __cvar_data->verbosity );
      __tmp_var_val = json_string( __str_buffer );
      free( __str_buffer ); __str_buffer = NULL;
      json_object_set( __var_obj, "verbosity", __tmp_var_val );
      json_decref( __tmp_var_val ); __tmp_var_val = NULL;

      __str_buffer = MALP_mpi_t_translate_MPI_OBJ_BIND( __cvar_data->bind );
      __tmp_var_val = json_string( __str_buffer );
      free( __str_buffer ); __str_buffer = NULL; 
      json_object_set( __var_obj, "bind", __tmp_var_val ); 
      json_decref( __tmp_var_val ); __tmp_var_val = NULL;

      __str_buffer = MALP_mpi_t_translate_MPI_Datatype( __cvar_data->mpi_type );
      __tmp_var_val = json_string( __str_buffer );
      free( __str_buffer ); __str_buffer = NULL; 
      json_object_set( __var_obj, "datatype", __tmp_var_val ); 
      json_decref( __tmp_var_val ); __tmp_var_val = NULL;

      __str_buffer = MALP_mpi_t_translate_MPI_CVAR_SCOPE( __cvar_data->scope );
      __tmp_var_val = json_string( __str_buffer );
      free( __str_buffer ); __str_buffer = NULL; 
      json_object_set( __var_obj, "scope", __tmp_var_val ); 
      json_decref( __tmp_var_val ); __tmp_var_val = NULL;
      
      __str_buffer = __malp_mpi_t_get_var_desc( MALP_MPI_T_TYPE_STATIC_CVAR, i ); 
      MALP_mpi_t_remove_invalid_character_from_string( __str_buffer );
      assert( __str_buffer );
      __tmp_var_val = json_string( __str_buffer );
      free( __str_buffer ); __str_buffer = NULL; 
      json_object_set( __var_obj, "description", __tmp_var_val ); 
      json_decref( __tmp_var_val ); __tmp_var_val = NULL;

      json_array_push( __var_array, __var_obj );
      json_decref( __var_obj ); __var_obj = NULL;
   } 

   return __var_array;
}

static json_t*
gen_mpit_static_pvar_profile( void )
{
   uint64_t __val; 
   char* __str_buffer;
   MPI_T_pvar_handle handle;
   MPI_T_pvar_session session;
   int err, i, __var_count, count;
   MALP_mpi_t_var_info_t  *__var_ids;
   MALP_mpi_t_pvar_info_t *__pvar_data;
   json_t *__var_obj, *__tmp_var_val, *__var_array;

   __var_ids = __malp_mpi_t_get_var_ids( MALP_MPI_T_TYPE_STATIC_PVAR );
   __var_count = __malp_mpi_t_get_num_vars( MALP_MPI_T_TYPE_STATIC_PVAR );  
   assert( !__var_count || ( __var_count && __var_ids ) );
   
   __var_array = json_array();

   err = PMPI_T_pvar_session_create( &session );
   assert( err == MPI_SUCCESS );

   for( i = 0; i < __var_count; i++ )
   {
      __var_obj = json_object();

      __pvar_data = &( __var_ids[i].var.pvar ); 
      err = PMPI_T_pvar_handle_alloc( session, __pvar_data->absolute_id, NULL, &( handle ), &count );
      if( err != MPI_SUCCESS ) continue;
      
      err = PMPI_T_pvar_start( session, handle );
      assert( err == MPI_SUCCESS );

      if( __pvar_data->mpi_type == MPI_CHAR )
      {
         __str_buffer = (char*) malloc( sizeof( char ) * ( count + 1 ) );
         assert( __str_buffer );
         err = PMPI_T_pvar_read( session, handle, (void*) __str_buffer );
         if( err != MPI_SUCCESS )
         {
            __tmp_var_val = json_string( "Error on PMPI_T_pvar_read" );
         }
         else
         {
            __tmp_var_val = json_string( __str_buffer );
         }
         free( __str_buffer ); __str_buffer = NULL;
      }
      else
      {
         assert( count == 1 );
         err = PMPI_T_pvar_read( session, handle, &( __val ) ); 
         assert( err == MPI_SUCCESS );

         if( __pvar_data->enum_type == MPI_T_ENUM_NULL ) 
         {
            const is_double = ( __pvar_data->mpi_type == MPI_DOUBLE ); 
            __tmp_var_val = (is_double) ? json_real( __val ) : json_int( __val );
         }
         else /* Translate int value to enum value */
         {
            char var_enum_name[1024];
            int var_enum_length = 1024; 
            int var_enum_max_elt, j;

            err = PMPI_T_enum_get_info( __pvar_data->enum_type, &var_enum_max_elt, var_enum_name, &var_enum_length );
            assert( err == MPI_SUCCESS );
            assert( var_enum_length < 1024 );
            var_enum_name[var_enum_length] = '\0';

            __tmp_var_val = json_string( var_enum_name );
            json_object_set( __var_obj, "name_enum", __tmp_var_val );
            json_decref( __tmp_var_val ); __tmp_var_val = NULL;              

            for( j = 0; j <  var_enum_max_elt; j++ )
            {
               int __real_val;
               int var_enum_length = 1024;
               err = PMPI_T_enum_get_item( __pvar_data->enum_type, j, &__real_val, var_enum_name, &var_enum_length );
               assert( err == MPI_SUCCESS );
               if( __real_val == __val ) break;
            } 
            
            // ENUM VAL NOT FOUND
            if( j == var_enum_max_elt )            
               var_enum_length = snprintf( var_enum_name, 1024, "UNDEF ENUM VALUE >> %d <<", __val ); 

            assert( var_enum_length < 1024 ); 
            var_enum_name[var_enum_length] = '\0';
            __tmp_var_val = json_string( var_enum_name );
         }
      }

      err = PMPI_T_pvar_stop( session, handle ); 
      assert( err == MPI_SUCCESS );

      err = PMPI_T_pvar_handle_free( session, &(handle) );  
      assert( err == MPI_SUCCESS );

      json_object_set( __var_obj, "value", __tmp_var_val );
      json_decref( __tmp_var_val ); __tmp_var_val = NULL;

      __str_buffer = MALP_mpi_t_translate_MPI_OBJ_BIND( __pvar_data->bind );
      __tmp_var_val = json_string( __str_buffer );
      free( __str_buffer ); __str_buffer = NULL; 
      json_object_set( __var_obj, "bind", __tmp_var_val ); 
      json_decref( __tmp_var_val ); __tmp_var_val = NULL;

      __str_buffer = MALP_mpi_t_translate_MPI_Datatype( __pvar_data->mpi_type );
      __tmp_var_val = json_string( __str_buffer );
      free( __str_buffer ); __str_buffer = NULL; 
      json_object_set( __var_obj, "datatype", __tmp_var_val ); 
      json_decref( __tmp_var_val ); __tmp_var_val = NULL;

      __str_buffer = __malp_mpi_t_get_var_name( MALP_MPI_T_TYPE_STATIC_PVAR, i );
      __tmp_var_val = json_string( __str_buffer );
      free( __str_buffer ); __str_buffer = NULL;
      json_object_set( __var_obj, "name", __tmp_var_val );
      json_decref( __tmp_var_val ); __tmp_var_val = NULL;

      __str_buffer = MALP_mpi_t_translate_MPI_T_VERBOSITY( __pvar_data->verbosity );
      __tmp_var_val = json_string( __str_buffer );
      free( __str_buffer ); __str_buffer = NULL;
      json_object_set( __var_obj, "verbosity", __tmp_var_val );
      json_decref( __tmp_var_val ); __tmp_var_val = NULL;

      __str_buffer = MALP_mpi_t_translate_PVAR_VAR_CLASS( __pvar_data->var_class ); 
      __tmp_var_val = json_string( __str_buffer );
      free( __str_buffer ); __str_buffer = NULL;
      json_object_set( __var_obj, "class", __tmp_var_val );
      json_decref( __tmp_var_val ); __tmp_var_val = NULL;
      
      __tmp_var_val = json_bool( __pvar_data->readonly ); 
      json_array_push( __var_array, __var_obj );
      json_object_set( __var_obj, "readonly", __tmp_var_val );
      json_decref( __var_obj ); __var_obj = NULL;

      __tmp_var_val = json_bool( __pvar_data->atomic );
      json_array_push( __var_array, __var_obj );
      json_object_set( __var_obj, "atomic", __tmp_var_val );
      json_decref( __var_obj ); __var_obj = NULL;

      __tmp_var_val = json_bool( __pvar_data->continuous ); 
      json_array_push( __var_array, __var_obj );
      json_object_set( __var_obj, "continuous", __tmp_var_val );
      json_decref( __var_obj ); __var_obj = NULL;
   } 

   return __var_array;
}

json_t * gen_mpit_static_profile( void )
{
   json_t* profile;
   json_t* __var_profile; 

   profile = json_object();

   __var_profile = gen_mpit_static_cvar_profile();
   json_object_set( profile, "cvar_static", __var_profile );
   json_decref( __var_profile ); __var_profile = NULL;

   __var_profile = gen_mpit_static_pvar_profile();
   json_object_set( profile, "pvar_static", __var_profile );
   json_decref( __var_profile ); __var_profile = NULL;

   return profile;
}

int MPIT_Static_infos_Profile_init_coll( struct Collector * coll )
{
   return 0;
}


int  MPIT_Static_infos_Profile_release_coll( struct Collector * coll )
{
   return 0;
}

void MPIT_Static_infos_Profile_push_event(struct MALP_Trace_Event *event, void *pcoll)
{
   return;
}

int  MPIT_Static_infos_Profile_reduce( struct Collector * coll )
{
   return 0;
}

int MPIT_Static_infos_Profile_accumulate( struct Collector * coll )
{

   int rank;
   PMPI_Comm_rank( VMPI_Get_partition_comm(), &rank );

   if( rank ) return 0;

   json_t * profile = gen_mpit_static_profile();

   char uri[100];
   snprintf(uri, 100, "%s/profile", coll->analysis_cache_name ); 
   nodeSharedCache_set( &nsc, uri, profile ); 
   json_decref( profile ); 

   return 0;
}

void MPIT_Static_infos_Profile_init(struct Collector *coll, uint64_t partition_id)
{
	Collector_init( coll, MALP_EVENT_NONE,
			partition_id,
		  	"MPIT_Static_infos_Profile",
			MPIT_Static_infos_Profile_init_coll,
			MPIT_Static_infos_Profile_release_coll,
			MPIT_Static_infos_Profile_push_event,
			MPIT_Static_infos_Profile_reduce,
			MPIT_Static_infos_Profile_accumulate );
}
#endif /* HAVE_MPI_T */
