/* ############################ MALP License ############################## */
/* # Fri Jan 18 14:00:00 CET 2013                                         # */
/* # Copyright or (C) or Copr. Commissariat a l'Energie Atomique          # */
/* #                                                                      # */
/* # This software is governed by the CeCILL-C license under French law   # */
/* # and abiding by the rules of distribution of free software.  You can  # */
/* # use, modify and/ or redistribute the software under the terms of     # */
/* # the CeCILL-C license as circulated by CEA, CNRS and INRIA at the     # */
/* # following URL http://www.cecill.info.                                # */
/* #                                                                      # */
/* # The fact that you are presently reading this means that you have     # */
/* # had knowledge of the CeCILL-C license and that you accept its        # */
/* # terms.                                                               # */
/* #                                                                      # */
/* # Authors:                                                             # */
/* #   - BESNARD Jean-Baptiste jean-baptiste.besnard@cea.fr               # */
/* #                                                                      # */
/* ######################################################################## */
#include "Event_Dispatcher.h"


void Handler_Desc_init( struct Handler_Desc *disp, void (*handler)(), void *arg)
{
	disp->handler = handler;
	disp->arg = arg;
}


void Handler_Desc_release( struct Handler_Desc *disp )
{
	disp->handler = NULL;
	disp->arg = NULL;
}


void Event_Dispatcher_init( struct Event_Dispatcher *disp )
{
	int i = 0;

	for (i = 0; i < (MALP_EVENT_TYPE_COUNT + 1); i++) {
		Chained_List_init( &disp->handler_lists[i] , 42, sizeof( struct Handler_Desc));
	}

	disp->been_initialized = 0x47;
}




void Event_Dispatcher_release( struct Event_Dispatcher *disp )
{
	int i = 0;

	disp->been_initialized = 0;

	for (i = 0; i < (MALP_EVENT_TYPE_COUNT + 1); i++) {
		Chained_List_walkthrough(&disp->handler_lists[i] , (void (*)(void*))Handler_Desc_release);
		Chained_List_release(&disp->handler_lists[i]);
	}
}


void Event_Dispatcher_register( struct Event_Dispatcher *disp, MALP_Trace_event_type type, void (*handler)(struct MALP_Trace_Event *, void *), void *arg)
{
	if( disp->been_initialized != 0x47 ) {
		printf("Event dispatcher not initialized \n");
		abort();
	}

	struct Handler_Desc desc;

	Handler_Desc_init( &desc, handler, arg );


	Chained_List_push( &disp->handler_lists[(int)type], &desc);

}

int Event_Dispatcher_has_handler( struct Event_Dispatcher *disp, MALP_Trace_event_type type )
{
	return Chained_List_count(&disp->handler_lists[(int)type]) + Chained_List_count(&disp->handler_lists[MALP_EVENT_ANY]);
}


void __Event_Dispatcher_call(void *pelem, void *pevent)
{
	struct Handler_Desc *desc = (struct Handler_Desc *)pelem;
	struct MALP_Trace_Event *event = (struct MALP_Trace_Event *)pevent;

	(desc->handler)( event, desc->arg );
}

