#ifndef MPI_SECTION_TREE_H
#define MPI_SECTION_TREE_H

#include <stdint.h>
#include "Hash_Table.h"
#include "Spinlock.h"
#include "jsonCache.h"
#include "Timer.h"

struct MPI_Section_balance
{
	/* Content */
	uint64_t * ballance;
	uint64_t cell_count;

	/* Sampling Infos */
	uint64_t start_offset;
	uint64_t quantum;
};


int MPI_Section_balance_init( struct MPI_Section_balance * bal , uint64_t cells );
int MPI_Section_balance_release( struct MPI_Section_balance * bal );
int MPI_Section_balance_hit( struct MPI_Section_balance * bal , uint64_t value );
int MPI_Section_balance_walk( struct MPI_Section_balance * bal, 
				void (*call)( uint64_t off, uint64_t val, void * arg),
				void * arg );
json_t * MPI_Section_balance_render( struct MPI_Section_balance * bal );



struct MPI_Section_tree
{
	/* Pointer to Parent */
	struct MPI_Section_tree * parent;

	/* Protection Lock */
	MALP_Spinlock lock;
	
	/* Name */
	char label[64];
	uint64_t current_ts;
	
	/* Payload */
	void * generic_payload;
	uint64_t hits;
	uint64_t total_time;
	uint64_t exclusive_time;
	
	/* Ballancing */
	uint64_t cell_count;
	struct MPI_Section_balance bal;
	
	/* Pointer to children */
	unsigned int siblings_count;
	struct MPI_Section_tree ** siblings;
};

int MPI_Section_tree_init_static( struct MPI_Section_tree * tree, char * label, uint64_t cells, void * payload );
struct MPI_Section_tree * MPI_Section_tree_new( struct MPI_Section_tree * parent, char * label );
int MPI_Section_tree_release( struct MPI_Section_tree * tree );

struct MPI_Section_tree * MPI_Section_tree_enter( struct MPI_Section_tree * tree, char * section_label, uint64_t timestamp );
struct MPI_Section_tree * MPI_Section_tree_leave( struct MPI_Section_tree * tree, char * section_label, uint64_t timestamp );

void MPI_Section_tree_compute_stats( struct MPI_Section_tree * tree );
int MPI_Section_tree_render( struct MPI_Section_tree * tree, json_t * parent_array );


#endif /* MPI_SECTION_TREE_H */
