#ifndef SAMPLE_GRID_H
#define SAMPLE_GRID_H

#include <stdint.h>
#include <assert.h>

#include "jsonCache.h"

struct Sample_Grid
{
	double **grid;
	
	double min;
	double max;
	
	double qx;
	
	uint64_t grid_size_x, slot_count;
		
	int has_data;
	
};

typedef struct Sample_2D_Grid_s
{
   uint64_t dim_x;
   uint64_t dim_y; 
   struct Sample_Grid **grids;
} Sample_2D_Grid_t; 

void Sample_Grid_init( struct Sample_Grid *g , uint64_t time_slices, uint64_t slots );
void Sample_Grid_release( struct Sample_Grid *g );
void Sample_Grid_flatten( struct Sample_Grid *g, int multiplicity );

void Sample_Grid_reduce( struct Sample_Grid *g );

void Sample_Grid_dump_text( struct Sample_Grid *g, char *dest );
json_t * Sample_Grid_dump_JSON( struct Sample_Grid *g );

double Sample_Grid_get( struct Sample_Grid *g ,  unsigned int slot, double offset );
void Sample_Grid_accumulate( struct Sample_Grid *g ,  unsigned int slot, double offset, double value );
void Sample_Grid_scale( struct Sample_Grid *g ,double offset );

void Sample_Grid_get_min_max( struct Sample_Grid *g , double *min, double *max );

static inline double Sample_Grid_get_duration( struct Sample_Grid *g )
{
	return g->grid_size_x * g->qx;
}

static inline Sample_2D_Grid_t*
Sample_2D_Grid_alloc( const uint64_t dim_x, const uint64_t dim_y )
{
   int i;
   struct Sample_Grid** grids;
   Sample_2D_Grid_t* new_2D_sample;
   
   new_2D_sample = (Sample_2D_Grid_t*) malloc( sizeof( Sample_2D_Grid_t ) );
   assert( new_2D_sample ); 

   new_2D_sample->dim_x = dim_x;
   new_2D_sample->dim_y = dim_y;

   grids = (struct Sample_Grid**) malloc( sizeof( struct Sample_Grid* ) * dim_x ); 
   assert( grids );
   new_2D_sample->grids = grids; 

   for( i = 0; i < dim_x; i++ )
   {
      grids[i] = (struct Sample_Grid*) malloc( sizeof( struct Sample_Grid) * dim_y );
      assert( grids[i] );
   }

   return new_2D_sample;
}

static inline void
Sample_2D_Grid_init( Sample_2D_Grid_t* __2D_grids, const uint64_t time_slices, const uint64_t num_slots )
{
   int i,j;
   assert( __2D_grids && __2D_grids->grids );
   for( i = 0; i < __2D_grids->dim_x; i++ )
      for( j = 0; j < __2D_grids->dim_y; j++ )
         Sample_Grid_init( &( __2D_grids->grids[i][j] ), time_slices, num_slots);
}

static inline void
Sample_2D_Grid_fini( Sample_2D_Grid_t* __2D_grids )
{
   int i,j;
   assert( __2D_grids && __2D_grids->grids );
   for( i = 0; i < __2D_grids->dim_x; i++ )
      for( j = 0; j < __2D_grids->dim_y; j++ )
         Sample_Grid_release( &( __2D_grids->grids[i][j] ) );
} 

static inline void
Sample_2D_Grid_desalloc( Sample_2D_Grid_t* __2D_grids  )
{
   int i,j;
   assert( __2D_grids && __2D_grids->grids );

   for( i = 0; i < __2D_grids->dim_x; i++ )
      free( __2D_grids->grids[i] );
   free( __2D_grids->grids );
   free( __2D_grids );
}
#endif /* SAMPLE_GRID_H */
