/* ############################ MALP License ############################## */
/* # Fri Jan 18 14:00:00 CET 2013                                         # */
/* # Copyright or (C) or Copr. Commissariat a l'Energie Atomique          # */
/* #                                                                      # */
/* # This software is governed by the CeCILL-C license under French law   # */
/* # and abiding by the rules of distribution of free software.  You can  # */
/* # use, modify and/ or redistribute the software under the terms of     # */
/* # the CeCILL-C license as circulated by CEA, CNRS and INRIA at the     # */
/* # following URL http://www.cecill.info.                                # */
/* #                                                                      # */
/* # The fact that you are presently reading this means that you have     # */
/* # had knowledge of the CeCILL-C license and that you accept its        # */
/* # terms.                                                               # */
/* #                                                                      # */
/* # Authors:                                                             # */
/* #   - BESNARD Jean-Baptiste jean-baptiste.besnard@cea.fr               # */
/* #                                                                      # */
/* ######################################################################## */
#include "Trace_Collector.h"
#include "Meta_Data_Handler.h"
#include "VMPI.h"

struct Trace_Collector __trace_collector; 

void Trace_Collector_init()
{
	int rank, size;
	
	PMPI_Comm_rank( VMPI_Get_partition_comm(), &rank );
	PMPI_Comm_size( VMPI_Get_partition_comm(), &size );
	Duration_infos_init();
	Meta_Data_Handler_init(rank, size);

	/* Setup Stream */
	VMPI_Map map;
	VMPI_Map_clear( &map );
	
	int i = 0;
	for( i = 0 ; i < VMPI_Get_partition_count(); i++ )
	{
		if( i != VMPI_Get_partition_id() )
		{
			if( VMPI_Map_partitions( i, VMPI_MAP_ROUND_ROBIN, &map) )
			{
				printf("Failed to map Analyzer to Instrumentation\n");
				exit(1);
			}
		}
	}

	VMPI_Stream_init( &Trace_Collector()->trace_in, sizeof( struct Event_Pack ), VMPI_STREAM_BALANCE_ROUND_ROBIN );
	VMPI_Stream_open_map( &Trace_Collector()->trace_in, &map, "r" );
	
	if( VMPI_Stream_read_count( &Trace_Collector()->trace_in ) == 0 )
	{
		printf("Failed to bootstrap trace stream\n");
		exit(1);
	}
	

	Analyzer_BB_init( &Trace_Collector()->anbb, 2048 );

}

void Trace_Collector_release()
{
	VMPI_Stream_close( &Trace_Collector()->trace_in );

        fprintf(stderr, "VMPI_Stream_close ... done\n");
	Analyzer_BB_release( &Trace_Collector()->anbb );

	Meta_Data_Handler_release();
	
	Duration_infos_release();
}


int cnt = 0;


void Trace_Collector_read_loop()
{
	struct Event_Pack * pack = calloc( 1, sizeof(struct Event_Pack) );
	
	if( !pack )
	{
		perror("malloc");
		exit(1);
	}

	int ret;

	do
	{
		ret = VMPI_Stream_read( &Trace_Collector()->trace_in, (char *)pack, 1, VMPI_STREAM_NONBLOCK);

		if( ret == VMPI_EAGAIN )
		{
			pthread_yield();
			continue;
		}

		if( 0 < ret )
		{
			Analyzer_BB_push_pack( &Trace_Collector()->anbb, pack );
			
			cnt += pack->offset;
		}

		
	}while( ret != 0 );
	
        fprintf(stderr, "Quit VMPI_Stream_read ... done\n");
	/* Reduce total size for information purpose */

	long long int total_cnt = 0;
	PMPI_Reduce(&cnt, &total_cnt, 1, MPI_INT, MPI_SUM, 0, VMPI_Get_partition_comm());

	int rank;
	PMPI_Comm_rank(VMPI_Get_partition_comm(), &rank);

	if(!rank)
	{
		double size = (double)(total_cnt * sizeof( struct MALP_Trace_Event ) ) / (1024*1024);
		printf("\033[1;31m##################################################\033[0m\n");
		printf("\033[1;31mAnalyzer processed %lld events !! ( %g MB )\033[0m\n", total_cnt, size );
		printf("\033[1;31m##################################################\033[0m\n");
	}

	free( pack );

}
