/* ############################ MALP License ############################## */
/* # Fri Jan 18 14:00:00 CET 2013                                         # */
/* # Copyright or (C) or Copr. Commissariat a l'Energie Atomique          # */
/* #                                                                      # */
/* # This software is governed by the CeCILL-C license under French law   # */
/* # and abiding by the rules of distribution of free software.  You can  # */
/* # use, modify and/ or redistribute the software under the terms of     # */
/* # the CeCILL-C license as circulated by CEA, CNRS and INRIA at the     # */
/* # following URL http://www.cecill.info.                                # */
/* #                                                                      # */
/* # The fact that you are presently reading this means that you have     # */
/* # had knowledge of the CeCILL-C license and that you accept its        # */
/* # terms.                                                               # */
/* #                                                                      # */
/* # Authors:                                                             # */
/* #   - BESNARD Jean-Baptiste jean-baptiste.besnard@cea.fr               # */
/* #                                                                      # */
/* ######################################################################## */
#ifndef TRACE_COLLECTOR_H
#define TRACE_COLLECTOR_H


#ifdef __cplusplus
extern "C" {
#endif

#include <stdint.h>
#include "VMPI_stream.h"
#include "VMPI.h"
#include "Event_Desc.h"
#include "Spinlock.h"
#include "Analyzer_BB.h"

/**
 * @ingroup Trace_Collector_
 * @brief The trace collector structure used by the Analyzer component, containing a link to the data input stream (@link VMPI_Stream @endlink ) and to the analyzer BlackBoard (@ref Analyzer_BB_ ).
 */
struct Trace_Collector
{
    VMPI_Stream trace_in;    /**<  Input stream for collected profiling trace */
    struct Analyzer_BB anbb; /**< The Analyzer black board */
};

/**
 * @ingroup Trace_Collector_
 * @brief Extern declaration for the trace collector.
 */
extern struct Trace_Collector __trace_collector;

/**
 * @ingroup Trace_Collector_
 * @brief Static getter for the Trace_Collector structure instance (declared as extern).
 * @return a @link Trace_Collector @endlink structure pointer
 */
static inline struct Trace_Collector *Trace_Collector()
{
	return &__trace_collector;
}

/**
 * @ingroup Trace_Collector_
 * @brief Initialisation of the Trace_Collector
 */
void Trace_Collector_init();

/**
 * @ingroup Trace_Collector_
 * @brief Close the input data stream and free the BlackBoard.
 */
void Trace_Collector_release();
void Trace_Collector_read_loop();

/**
 * @ingroup Trace_Collector_
 * @brief Start the data input reading loop
 */
void Trace_Collector_read_loop();

#ifdef __cplusplus
}
#endif


#endif /*TRACE_COLLECTOR_H*/
