/* ############################ MALP License ############################## */
/* # Fri Jan 18 14:00:00 CET 2013                                         # */
/* # Copyright or (C) or Copr. Commissariat a l'Energie Atomique          # */
/* #                                                                      # */
/* # This software is governed by the CeCILL-C license under French law   # */
/* # and abiding by the rules of distribution of free software.  You can  # */
/* # use, modify and/ or redistribute the software under the terms of     # */
/* # the CeCILL-C license as circulated by CEA, CNRS and INRIA at the     # */
/* # following URL http://www.cecill.info.                                # */
/* #                                                                      # */
/* # The fact that you are presently reading this means that you have     # */
/* # had knowledge of the CeCILL-C license and that you accept its        # */
/* # terms.                                                               # */
/* #                                                                      # */
/* # Authors:                                                             # */
/* #   - BESNARD Jean-Baptiste jean-baptiste.besnard@cea.fr               # */
/* #                                                                      # */
/* ######################################################################## */
#include "Bit_Array.h"

#include <string.h>

void Bit_Array_init( struct Bit_Array *ba, uint64_t size )
{
	ba->real_size = size;
	ba->size = (size>>3) + 1;

	ba->array = malloc( ba->size * sizeof( uint8_t ));

	if( !ba->array ) {
		printf("Failed to allocate a Bit_Array \n");
		return;
	}

	int i = 0;
	for( i = 0 ; i < ba->size ; i++ )
		ba->array[i] = 0;

}

void Bit_Array_release( struct Bit_Array *ba )
{
	int i = 0;
	for( i = 0 ; i < ba->size ; i++ )
		ba->array[i] = 0;

	ba->size = 0;
	ba->real_size = 0;

	free( ba->array );
	ba->array = NULL;
}


void Bit_Array_replicate( struct Bit_Array *dest, struct Bit_Array *src )
{
	if( dest->size != src->size )
	{
		printf("Trying to copy to a different size Bit_Array\n");
		abort();
	}
	
	memcpy( dest->array, src->array , dest->size );
}



void Bit_Array_dump( struct Bit_Array *ba )
{

	int i = 0;
	unsigned int size = ba->size * 8;

	printf("Array size is %d \n", size);

	for( i = 0 ; i < size; i++ ) {
		printf("[%d]%d ", i,  Bit_Array_get( ba, i) );
	}


	printf("\n");
}



