/* ############################ MALP License ############################## */
/* # Fri Jan 18 14:00:00 CET 2013                                         # */
/* # Copyright or (C) or Copr. Commissariat a l'Energie Atomique          # */
/* #                                                                      # */
/* # This software is governed by the CeCILL-C license under French law   # */
/* # and abiding by the rules of distribution of free software.  You can  # */
/* # use, modify and/ or redistribute the software under the terms of     # */
/* # the CeCILL-C license as circulated by CEA, CNRS and INRIA at the     # */
/* # following URL http://www.cecill.info.                                # */
/* #                                                                      # */
/* # The fact that you are presently reading this means that you have     # */
/* # had knowledge of the CeCILL-C license and that you accept its        # */
/* # terms.                                                               # */
/* #                                                                      # */
/* # Authors:                                                             # */
/* #   - BESNARD Jean-Baptiste jean-baptiste.besnard@cea.fr               # */
/* #                                                                      # */
/* ######################################################################## */

#include "Bloom_Filter.h"



void Bloom_Filter_init( struct Bloom_Filter *bf , uint64_t size, size_t sizeof_payload )
{
    Bit_Array_init( &bf->array, size );
    bf->lock = 0;
    bf->size = size;
    bf->sizeof_payload = sizeof_payload;
}

void Bloom_Filter_release( struct Bloom_Filter *bf )
{
    Bit_Array_release( &bf->array );
    bf->lock = 0;
    bf->size = 0;
    bf->sizeof_payload = 0;
}



void Bloom_Filter_replicate( struct Bloom_Filter *dest , struct Bloom_Filter *src )
{
    MALP_Spinlock_lock( &dest->lock );
    MALP_Spinlock_lock( &src->lock );
    Bit_Array_replicate( &dest->array, &src->array );
    MALP_Spinlock_unlock( &src->lock );
    MALP_Spinlock_unlock( &dest->lock );
}
