/* ############################ MALP License ############################## */
/* # Fri Jan 18 14:00:00 CET 2013                                         # */
/* # Copyright or (C) or Copr. Commissariat a l'Energie Atomique          # */
/* #                                                                      # */
/* # This software is governed by the CeCILL-C license under French law   # */
/* # and abiding by the rules of distribution of free software.  You can  # */
/* # use, modify and/ or redistribute the software under the terms of     # */
/* # the CeCILL-C license as circulated by CEA, CNRS and INRIA at the     # */
/* # following URL http://www.cecill.info.                                # */
/* #                                                                      # */
/* # The fact that you are presently reading this means that you have     # */
/* # had knowledge of the CeCILL-C license and that you accept its        # */
/* # terms.                                                               # */
/* #                                                                      # */
/* # Authors:                                                             # */
/* #   - BESNARD Jean-Baptiste jean-baptiste.besnard@cea.fr               # */
/* #                                                                      # */
/* ######################################################################## */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>


#ifndef MALP_CONFIG_H
#define MALP_CONFIG_H

/**
 * @ingroup Config_
 * @brief Enumeration of the configuration parameter keys for MALP
 */
typedef enum {
    //GLOBAL CONFIG
    TA_CONF_GLOBAL_HEADER,
    TA_CONF_THREAD_PER_TASK,


    //VERBOSITY
	TA_CONF_VERBOSITY_HEADER,
	TA_CONF_DISPLAY_BANNER,
    TA_CONF_DISPLAY_BB_LOAD,
	TA_CONF_VERBOSE_OUTPUT,

    //INSTRUMENTATION
    TA_CONF_INSTRUMENT_HEADER,
	TA_CONF_INSTRUMENT_MPI,
	TA_CONF_INSTRUMENT_POSIX,
	TA_CONF_INSTRUMENT_MPI_OTF,
	TA_CONF_INSTRUMENT_POSIX_OTF,
	TA_CONF_INSTRUMENT_POSIX_PROFILE,
	TA_CONF_INSTRUMENT_POSIX_PROFILE_DUMP,

	// SAMPLE ANALYSIS
    
    TA_CONF_SAMPLE_ANALYSIS_HEADER,
    TA_CONF_SAMPLE_ANALYSIS_ENABLED,
	
    // OTF2 ANALYSIS

    TA_CONF_OTF2_ANALYSIS_HEADER,
    TA_CONF_OTF2_ANALYSIS_ENABLED,
    TA_CONF_OTF2_TRACE_PATH,
    TA_CONF_OTF2_TRACE_NAME,

    //END
    TA_CONF_KEY_COUNT

} TA_Conf_key_t;


static const char * const TA_Conf_key_lablel[TA_CONF_KEY_COUNT] = {
	//GLOBAL CONFIG
	"TA_CONF_GLOBAL_HEADER",
	"TA_CONF_THREAD_PER_TASK",


    //VERBOSITY
	"TA_CONF_VERBOSITY_HEADER",
	"TA_CONF_DISPLAY_BANNER",
    "TA_CONF_DISPLAY_BB_LOAD",
	"TA_CONF_VERBOSE_OUTPUT",

    //INSTRUMENTATION
    "TA_CONF_INSTRUMENT_HEADER",
	"TA_CONF_INSTRUMENT_MPI",
	"TA_CONF_INSTRUMENT_POSIX",
	"TA_CONF_INSTRUMENT_MPI_OTF",
	"TA_CONF_INSTRUMENT_POSIX_OTF",
	"TA_CONF_INSTRUMENT_POSIX_PROFILE",
	"TA_CONF_INSTRUMENT_POSIX_PROFILE_DUMP",

	// SAMPLE ANALYSIS
	
	"TA_CONF_SAMPLE_ANALYSIS_HEADER",
    "TA_CONF_SAMPLE_ANALYSIS_ENABLED",

    // OTF2 ANALYSIS

    "TA_CONF_OTF2_ANALYSIS_HEADER",
    "TA_CONF_OTF2_ANALYSIS_ENABLED",
    "TA_CONF_OTF2_TRACE_PATH",
    "TA_CONF_OTF2_TRACE_NAME"
	
};

/**
 * @ingroup Config_
 * @brief Possible types for the @ref Analyzer_ configuration parameter values
 */
typedef enum {
    TA_CONF_NONE,   /**< @brief No defined type */
    TA_CONF_INT,    /**< @brief Integer */
    TA_CONF_BOOL,   /**< @brief Boolean */
    TA_CONF_DOUBLE, /**< @brief Double */
    TA_CONF_STRING  /**< @brief String */
} TA_Conf_type;


/**
 * @ingroup Config_
 * @brief Define a configuration entry structure, which contain a descritpion buffer, the entry type and variables and buffers for all possible type of an entry value.
 */
struct TA_Conf_entry {
    char desc[500];     /**< @brief Descriptor of a configuration parameter */
    TA_Conf_type type;  /**< @brief Type of a configuration parameter value */

	//Embeds all types for better efficiency
    char c_value[1000]; /**< @brief char* buffer for the configuration parameter value */
    float d_value;      /**< @brief float variable for the configuration parameter value */
    int i_value;        /**< @brief integer variable for the configuration parameter value */

};

/**
 * @ingroup Config_
 * @brief Initialisation of the @ref Analyzer_ configuration parameter's type, description and value
 * @param key Configuration parameter key
 * @param desc Configuration parameter description.
 * @param type Configuration parameter value's type
 * @param value new value for the configuration parameter
 * @return the modification status (0: key not found or update failure, 1: success )
 */
void TA_Conf_entry_init( TA_Conf_key_t key, char *desc, TA_Conf_type type, char *value );

/**
 * @ingroup Config_
 * @brief Update the value for a configuration parameter after control of the value's content and type
 *
 *(see @ref TA_Conf_update for control on the key name)
 * @param entry Key of the configuration parameter to modify
 * @param value new value for the configuration parameter (should be a numerical value)
 * @return the modification status (0: value of wrong type, 1: success )
 */
int TA_Conf_entry_update( struct TA_Conf_entry *entry, char *value );

/**
 * @ingroup Config_
 * @brief Update the value for a configuration option with control of the value name.
 * @param key_name Key of the Analyzer configuration parameter to modify
 * @param val new value for the corresponding configuration parameter  (should be a numerical value)
 * @return modification status (0: key not found or update failure, 1: success )
 */
int TA_Conf_update( char *key_name, char *val );

/**
 * @ingroup Config_
 * @brief Initialisation of the @ref Trace_Collector with default options.
 */
void TA_Conf_init_default();

/**
 * @ingroup Config_
 * @brief Initialisation of config from environment
 */
void TA_Conf_init_from_env();

/**
 * @ingroup Config_
 * @brief Start broadcasting for the analyzer
 *
 * Start the exchange between the communication channel (MPI_COMM_WORLD) and the analyzer list of parameters ( @ref TA_Conf_entries - list of @ref TA_Conf_entry ).
 */
void TA_Conf_dispatch();

/**
 * @ingroup Config_
 * @brief Save the Analyzer_ configuration in the destination file in argument
 * @param dest Configuration file destination
 */
void TA_Conf_dump( char *dest );

/**
 * @ingroup Config_
 * @brief Load and use a file to set the Analyzer configuration
 * @param source Configuration file source
 */
void TA_Conf_load( char *source );

#ifdef MPC_PRIVATIZED
    extern __process struct TA_Conf_entry TA_Conf_entries[TA_CONF_KEY_COUNT];
#else
    extern struct TA_Conf_entry TA_Conf_entries[TA_CONF_KEY_COUNT];
#endif

/**
 * @ingroup Config_
 * @brief Get a configuration parameter value of type int
 *
 * Get a configuration parameter value of type int if there's one corresponding to a parameter key, nothing otherwise
 * @param key
 * @return the value of the parameter if it's an integer
 */
static inline int TA_Conf_get_key_int( TA_Conf_key_t key )
{
	if( ( key < TA_CONF_KEY_COUNT )  ) {
		if( TA_Conf_entries[key].type == TA_CONF_INT ) {
			return TA_Conf_entries[key].i_value;
		} else {
			printf("Bad config type\n");
			abort();
		}
	} else {
		printf("Bad config key\n");
		abort();
	}
}

/**
 * @ingroup Config_
 * @brief Get a configuration parameter value of type bool
 *
 * Get a configuration parameter value of type bool corresponding to a parameter key, nothing otherwise
 * @param key
 * @return the value of the parameter if it's a boolean value
 */
static inline int TA_Conf_get_key_bool( TA_Conf_key_t key )
{
	if( ( key < TA_CONF_KEY_COUNT )  )
	{
		if( TA_Conf_entries[key].type == TA_CONF_BOOL )
		{
			return TA_Conf_entries[key].i_value & 0x1;
		} else {
			printf("Bad config type for key %s\n", TA_Conf_key_lablel[key]);
			abort();
		}
	} else {
		printf("Bad config key\n");
		abort();
	}
}

/**
 * @ingroup Config_
 * @brief Get a configuration parameter value of type double
 *
 * Get a configuration parameter value of type double corresponding to a parameter key, nothing otherwise
 * @param key
 * @return the value of the parameter if it's a double
 */
static inline double TA_Conf_get_key_double( TA_Conf_key_t key )
{
	if( ( key < TA_CONF_KEY_COUNT )  ) {
		if( TA_Conf_entries[key].type == TA_CONF_DOUBLE ) {
			return TA_Conf_entries[key].d_value;
		} else {
			printf("Bad config type for key %s\n", TA_Conf_key_lablel[key]);
			abort();
		}
	} else {
		printf("Bad config key\n");
		abort();
	}
}

/**
 * @ingroup Config_
 * @brief Get a configuration parameter value of type string
 *
 * Get a configuration parameter value of type string corresponding to a parameter key, nothing otherwise
 * @param key
 * @return the value of the parameter if it's a string
 */
static inline char * TA_Conf_get_key_string( TA_Conf_key_t key )
{
	if( ( key < TA_CONF_KEY_COUNT )  ) {
		if( TA_Conf_entries[key].type == TA_CONF_STRING ) {
			return TA_Conf_entries[key].c_value;
		} else {
			printf("Bad config type for key %s\n", TA_Conf_key_lablel[key]);
			abort();
		}
	} else {
		printf("Bad config key\n");
		abort();
	}
}


#define TA_Conf_dispatch()	do{int ___ret = PMPI_Bcast( (char*)TA_Conf_entries,\
							sizeof(struct TA_Conf_entry) * TA_CONF_KEY_COUNT,\
							MPI_CHAR, 0, VMPI_Get_partition_comm());\
							if( ___ret != MPI_SUCCESS ) {\
								printf("Failled to dispatch configuration \n");\
								abort();\
							}}while(0)




#endif
