
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include "mpi.h"
#include "json_cache.h"
#include "MALP_MPI_T_Enum.h"

MALP_MPI_T_enum_type_t*
MALP_MPI_T_Extract_Enum( MPI_T_enum enumtype )
{
   int i, err;
   MALP_MPI_T_enum_type_t* new_enum;
   char *enum_name[MALP_MPI_T_ENUM_MAX_SIZE];
   int enum_name_len = MALP_MPI_T_ENUM_MAX_SIZE;

   new_enum = ( MALP_MPI_T_enum_type_t* ) malloc( sizeof(MALP_MPI_T_enum_type_t) );
   assert( new_enum );
   memset( new_enum, 0, sizeof(MALP_MPI_T_enum_type_t) );

   err = MPI_T_enum_get_info( enumtype, &(new_enum->num_vals), enum_name, &enum_name_len ); 
   assert( err == MPI_SUCCESS );
 
   assert( enum_name_len < MALP_MPI_T_ENUM_MAX_SIZE );
   enum_name[enum_name_len] = '\0'; 
   new_enum->name = strdup( enum_name ); 

   new_enum->val_array = (uint64_t*) malloc( new_enum->num_vals * sizeof( uint64_t ) ); 
   assert( new_enum->val_array );

   new_enum->val_names = (char**) malloc( new_enum->num_vals * sizeof( char* ) );
   assert( new_enum->val_names );

   for( i = 0; i < new_enum->num_vals; i++ )
   {
      enum_name_len = MALP_MPI_T_ENUM_MAX_SIZE;
      err = MPI_T_enum_get_item( enumtype, i, &(new_enum->val_array[i]), enum_name, &enum_name_len );
      assert( err == MPI_SUCCESS );

      assert( enum_name_len < MALP_MPI_T_ENUM_MAX_SIZE );
      enum_name[enum_name_len] = '\0';
      new_enum->val_names[i] = strdup( enum_name ); 
   }
   return new_enum;
} 

json_t*
MALP_MPI_T_Enum_JSON_dump( MPI_T_enum enumtype )
{
   int i;
   MALP_MPI_T_enum_type_t* malp_enum;
   json_t *__tmp_int_array, *__tmp_str_array,  *__tmp_int, *__tmp_obj, *__tmp_str;

   malp_enum = MALP_MPI_T_Extract_Enum( enumtype ); 
   assert( malp_enum );

   __tmp_obj = json_object();
   assert( __tmp_obj );

   __tmp_str = json_str( malp_enum->name );
   json_object_set( __tmp_obj, "name", __tmp_str ); 
   json_decref( __tmp_str ); __tmp_str = NULL;
   
   __tmp_int = json_int( malp_enum->num_vals );
   json_object_set( __tmp_obj, "number", __tmp_int );
   json_decref( __tmp_int ); __tmp_int = NULL;

   __tmp_int_array = json_array(); 
   assert( __tmp_int_array ); 

   __tmp_str_array = json_array();
   assert( __tmp_int_array );

   for( i = 0; i < malp_enum->num_vals; i++ )
   {
      __tmp_int = json_int( malp_enum->val_array[i] );
      __tmp_str = json_str( malp_enum->val_names[i] );

     json_array_push( __tmp_int_array, __tmp_int );
     json_array_push( __tmp_str_array, __tmp_str );

     json_decref( __tmp_int ); __tmp_int = NULL;
     json_decref( __tmp_str ); __tmp_str = NULL;
   }

   json_object_set( __tmp_obj, "enum_values", __tmp_int_array );
   json_object_set( __tmp_obj, "enum_names", __tmp_str_array );

   json_decref( __tmp_int_array ); __tmp_int_array = NULL;
   json_decref( __tmp_str_array ); __tmp_str_array = NULL;

   return __tmp_obj;
}

char*
MALP_MPI_T_Enum_JSON_dump( MPI_T_enum enumtype )
{
   char* printable_enumtype;
   json_t* enumtype_json_obj;

   enumtype_json_obj = MALP_MPI_T_Enum_JSON_dump( enumtype ); 
   assert( enumtype_json_obj );
  
   printable_enumtype = json_dump( enumtype_json_obj, JSON_COMPACT ); 
   assert( printable_enumtype );

   return printable_enumtype; 
}
