#include "config.h"

#if HAVE_MPI_T

#include <stdio.h>
#include <stdint.h>
#include <assert.h>
#include <stdlib.h>
#include <string.h>
#include <regex.h>

#include "mpi.h"
#include "VMPI.h"
#include "MPIT_Events.h"

static MALP_mpi_t_var_info_t *__poll_cvar_virt_ids = NULL;
static MALP_mpi_t_var_info_t *__poll_pvar_virt_ids = NULL;
static MALP_mpi_t_var_info_t *__static_cvar_virt_ids = NULL;
static MALP_mpi_t_var_info_t *__static_pvar_virt_ids = NULL;

static uint64_t __malp_mpi_t_poll_freq = 1;

static int __poll_pvar_virt_num = 0;
static int __poll_cvar_virt_num = 0;
static int __static_cvar_virt_num = 0;
static int __static_pvar_virt_num = 0;


MALP_mpi_t_var_info_t *
__malp_mpi_t_get_var_id( MALP_mpi_t_var_type_t type, const int virt_idx )
{  
   MALP_mpi_t_var_info_t *ret;

   switch( type )
   {
      case MALP_MPI_T_TYPE_STATIC_CVAR:
         assert( virt_idx < __static_cvar_virt_num );
         ret = &( __static_cvar_virt_ids[virt_idx] );
         break; 
      case MALP_MPI_T_TYPE_STATIC_PVAR:
         assert( virt_idx < __static_pvar_virt_num );
         ret = &( __static_pvar_virt_ids[virt_idx] );
         break; 
      case MALP_MPI_T_TYPE_POLL_CVAR:
         assert( virt_idx < __poll_cvar_virt_num );
         ret = &( __poll_cvar_virt_ids[virt_idx] );
         break; 
      case MALP_MPI_T_TYPE_POLL_PVAR:
         assert( virt_idx < __poll_pvar_virt_num );
         ret = &( __poll_pvar_virt_ids[virt_idx] );
         break; 
      default:
         ret = NULL;
         fprintf(stderr, "error: undef MALP_mpi_t_var_type_t variable\n");
   }
   return ret;
}

char* __malp_mpi_t_get_cvar_name( MALP_mpi_t_var_type_t type, const int virt_idx )
{
   int err, namelen, desclen;
   char* var_name;
   char var_desc[1024]; 
   MALP_mpi_t_var_info_t *gen_mpi_t_infos;
   MALP_mpi_t_cvar_info_t *var_data;

   gen_mpi_t_infos = __malp_mpi_t_get_var_id( type, virt_idx );
   assert( gen_mpi_t_infos );
   var_data = &( gen_mpi_t_infos->var.cvar );

   const int name_max_len = 1024;
   namelen = name_max_len;
   var_name = (char*) malloc( namelen * sizeof( char ) ); 
   assert( var_name );
   memset( var_name, 0, namelen * sizeof( char ) );

   desclen = 1024;
   err = PMPI_T_cvar_get_info( var_data->absolute_id, var_name, &namelen,
			      &( var_data->verbosity ), &( var_data->mpi_type ), 
			      &( var_data->enum_type ), var_desc, &desclen,
			      &( var_data->bind ), &( var_data->scope ) );
   assert( err == MPI_SUCCESS );
   assert( namelen < name_max_len );
   var_name[namelen] = '\0';
   return var_name;
}

char* __malp_mpi_t_get_cvar_desc( MALP_mpi_t_var_type_t type, const int virt_idx )
{
   int err, namelen, desclen;
   char* var_desc;
   char var_name[1024]; 

   MALP_mpi_t_var_info_t *gen_mpi_t_infos;
   MALP_mpi_t_cvar_info_t *var_data;

   gen_mpi_t_infos = __malp_mpi_t_get_var_id( type, virt_idx );
   assert( gen_mpi_t_infos );
   var_data = &( gen_mpi_t_infos->var.cvar );

   const int desc_max_len = 4096;
   desclen = desc_max_len;
   var_desc = (char*) malloc( desclen * sizeof( char ) ); 
   assert( var_desc );
   memset( var_desc, 0, desclen * sizeof( char ) );

   namelen = 1024;
   err = PMPI_T_cvar_get_info( var_data->absolute_id, var_name, &namelen,
			      &( var_data->verbosity ), &( var_data->mpi_type ), 
			      &( var_data->enum_type ), var_desc, &desclen,
			      &( var_data->bind ), &( var_data->scope ) );
   assert( err == MPI_SUCCESS );
   assert( desclen < desc_max_len );
   var_desc[desclen] = '\0';
   return var_desc;
}

char* __malp_mpi_t_get_pvar_name( MALP_mpi_t_var_type_t type, const int virt_idx )
{
   int err, namelen, desclen;
   char* var_name;
   char var_desc[1024]; 
   MALP_mpi_t_var_info_t *gen_mpi_t_infos;
   MALP_mpi_t_pvar_info_t *var_data;

   gen_mpi_t_infos = __malp_mpi_t_get_var_id( type, virt_idx );
   assert( gen_mpi_t_infos );
   var_data = &( gen_mpi_t_infos->var.pvar );

   const int name_max_len = 1024;
   namelen = name_max_len;
   var_name = (char*) malloc( namelen * sizeof( char ) ); 
   assert( var_name );
   memset( var_name, 0, namelen * sizeof( char ) );

   desclen = 1024;
   err = PMPI_T_pvar_get_info( var_data->absolute_id, var_name, &namelen, 
			      &( var_data->verbosity ), &( var_data->var_class ),
			      &( var_data->mpi_type ), &( var_data->enum_type ), 
                              var_desc, &desclen,
			      &( var_data->bind ), &( var_data->readonly ),
                              &( var_data->continuous ), &( var_data->atomic ) );
   assert( err == MPI_SUCCESS );
   assert( namelen < name_max_len );
   var_name[namelen] = '\0';
   return var_name;
}

char* __malp_mpi_t_get_pvar_desc( MALP_mpi_t_var_type_t type, const int virt_idx )
{
   int err, namelen, desclen;
   char* var_desc;
   char var_name[1024]; 
   MALP_mpi_t_var_info_t *gen_mpi_t_infos;
   MALP_mpi_t_pvar_info_t *var_data;

   gen_mpi_t_infos = __malp_mpi_t_get_var_id( type, virt_idx );
   assert( gen_mpi_t_infos );
   var_data = &( gen_mpi_t_infos->var.pvar );

   const int desc_max_len = 1024;
   desclen = desc_max_len;
   var_desc = (char*) malloc( namelen * sizeof( char ) ); 
   assert( var_desc );
   memset( var_desc, 0, desclen * sizeof( char ) );

   err = PMPI_T_pvar_get_info( var_data->absolute_id, var_name, &namelen, 
			      &( var_data->verbosity ), &( var_data->var_class ),
			      &( var_data->mpi_type ), &( var_data->enum_type ), 
                              var_desc, &desclen,
			      &( var_data->bind ), &( var_data->readonly ),
                              &( var_data->continuous ), &( var_data->atomic ) );
   assert( err == MPI_SUCCESS );
   assert( desclen < desc_max_len );
   var_desc[desclen] = '\0';
   return var_desc;
}


char* __malp_mpi_t_get_var_name( MALP_mpi_t_var_type_t type, const int virt_idx )
{
   char* var_name;
   switch( type )
   {
      case MALP_MPI_T_TYPE_POLL_CVAR:
      case MALP_MPI_T_TYPE_STATIC_CVAR:
         var_name = __malp_mpi_t_get_cvar_name( type, virt_idx );
         break; 
      case MALP_MPI_T_TYPE_POLL_PVAR:
      case MALP_MPI_T_TYPE_STATIC_PVAR:
         var_name = __malp_mpi_t_get_pvar_name( type, virt_idx );
         break; 
      default:
         var_name = NULL;
         fprintf(stderr, "error: undef MALP_mpi_t_var_type_t variable\n");
   } 
   return var_name;
}

char* __malp_mpi_t_get_var_desc( MALP_mpi_t_var_type_t type, const int virt_idx )
{
   char* var_desc;
   switch( type )
   {
      case MALP_MPI_T_TYPE_POLL_CVAR:
      case MALP_MPI_T_TYPE_STATIC_CVAR:
         var_desc = __malp_mpi_t_get_cvar_desc( type, virt_idx );
         break; 
      case MALP_MPI_T_TYPE_POLL_PVAR:
      case MALP_MPI_T_TYPE_STATIC_PVAR:
         var_desc = __malp_mpi_t_get_pvar_desc( type, virt_idx );
         break; 
      default:
         var_desc = NULL;
         fprintf(stderr, "error: undef MALP_mpi_t_var_type_t variable\n");
   } 
   return var_desc;
}

MALP_mpi_t_var_info_t *
__malp_mpi_t_get_var_ids( MALP_mpi_t_var_type_t type )
{
   MALP_mpi_t_var_info_t *ret;
   switch( type )
   {
      case MALP_MPI_T_TYPE_STATIC_CVAR:
         ret = __static_cvar_virt_ids;
         break; 
      case MALP_MPI_T_TYPE_STATIC_PVAR:
         ret = __static_pvar_virt_ids;
         break; 
      case MALP_MPI_T_TYPE_POLL_CVAR:
         ret = __poll_cvar_virt_ids;
         break; 
      case MALP_MPI_T_TYPE_POLL_PVAR:
         ret = __poll_pvar_virt_ids;
         break; 
      default:
         ret = NULL;
         fprintf(stderr, "error: undef MALP_mpi_t_var_type_t variable\n");
   }
   return ret;
}

int
__malp_mpi_t_get_num_vars( MALP_mpi_t_var_type_t type )
{
   int ret;
   switch( type )
   {
      case MALP_MPI_T_TYPE_STATIC_CVAR:
         ret = __static_cvar_virt_num;
         break; 
      case MALP_MPI_T_TYPE_STATIC_PVAR:
         ret = __static_pvar_virt_num;
         break; 
      case MALP_MPI_T_TYPE_POLL_CVAR:
         ret = __poll_cvar_virt_num;
         break; 
      case MALP_MPI_T_TYPE_POLL_PVAR:
         ret = __poll_pvar_virt_num;
         break; 
      default:
         ret = 0;
         fprintf(stderr, "error: undef MALP_mpi_t_var_type_t variable\n");
   }
 
   return ret;
}


int
__malp_mpi_t_register_var_id( MALP_mpi_t_var_type_t type, MALP_mpi_t_var_info_t* new_entry )
{
   int ret;
   char* name;
   size_t next_size;
   MALP_mpi_t_var_info_t *__tmp_realloc;

   switch( type )
   {
      case MALP_MPI_T_TYPE_STATIC_CVAR:
         next_size = (__static_cvar_virt_num + 1 ) * sizeof( MALP_mpi_t_var_info_t ); 
         __tmp_realloc = realloc( __static_cvar_virt_ids, next_size );
         assert( __tmp_realloc );
         __static_cvar_virt_ids = __tmp_realloc;
         memcpy( &(__static_cvar_virt_ids[__static_cvar_virt_num]), new_entry, sizeof( MALP_mpi_t_var_info_t ) );
         __static_cvar_virt_num++;
#ifdef __MALP_MPI_T_REGISTER_DEBUG_PRINT
         name = __malp_mpi_t_get_var_name( MALP_MPI_T_TYPE_STATIC_CVAR, __static_cvar_virt_num - 1 );
         fprintf(stderr, "[STATIC CVARS] register : %s\n", name );
         free( name ); name = NULL;
#endif /* __MALP_MPI_T_REGISTER_DEBUG_PRINT */
         ret = 0;
         break; 
      case MALP_MPI_T_TYPE_STATIC_PVAR:
         next_size = (__static_pvar_virt_num + 1 ) * sizeof( MALP_mpi_t_var_info_t ); 
         __tmp_realloc = realloc( __static_pvar_virt_ids, next_size );
         assert( __tmp_realloc );
         __static_pvar_virt_ids = __tmp_realloc;
         memcpy( &(__static_pvar_virt_ids[__static_pvar_virt_num]), new_entry, sizeof( MALP_mpi_t_var_info_t ) );
         __static_pvar_virt_num++;
#ifdef __MALP_MPI_T_REGISTER_DEBUG_PRINT
         name = __malp_mpi_t_get_var_name( MALP_MPI_T_TYPE_STATIC_CVAR, __static_cvar_virt_num - 1 );
         fprintf(stderr, "[STATIC PVARS] register : %s\n", name );
         free( name ); name = NULL;
#endif /* __MALP_MPI_T_REGISTER_DEBUG_PRINT */
         ret = 0;
         break; 
      case MALP_MPI_T_TYPE_POLL_CVAR:
         next_size = (__poll_cvar_virt_num + 1 ) * sizeof( MALP_mpi_t_var_info_t ); 
         __tmp_realloc = realloc( __poll_cvar_virt_ids, next_size );
         assert( __tmp_realloc );
         __poll_cvar_virt_ids = __tmp_realloc;
         memcpy( &(__poll_cvar_virt_ids[__poll_cvar_virt_num]), new_entry, sizeof( MALP_mpi_t_var_info_t ) );
         __poll_cvar_virt_num++;
#ifdef __MALP_MPI_T_REGISTER_DEBUG_PRINT
         name = __malp_mpi_t_get_var_name( MALP_MPI_T_TYPE_POLL_CVAR, __poll_cvar_virt_num - 1 );
         fprintf(stderr, "[POLL CVARS] register : %s\n", name );
         free( name ); name = NULL;
#endif /* __MALP_MPI_T_REGISTER_DEBUG_PRINT */
         ret = 0;
         break; 
      case MALP_MPI_T_TYPE_POLL_PVAR:
         next_size = (__poll_pvar_virt_num + 1 ) * sizeof( MALP_mpi_t_var_info_t ); 
         __tmp_realloc = realloc( __poll_pvar_virt_ids, next_size );
         assert( __tmp_realloc );
         __poll_pvar_virt_ids = __tmp_realloc;
         memcpy( &(__poll_pvar_virt_ids[__poll_pvar_virt_num]), new_entry, sizeof( MALP_mpi_t_var_info_t ) );
         __poll_pvar_virt_num++;
#if __MALP_MPI_T_REGISTER_DEBUG_PRINT
         name = __malp_mpi_t_get_var_name( MALP_MPI_T_TYPE_POLL_PVAR, __poll_pvar_virt_num - 1 );
         fprintf(stderr, "[POLL PVARS] register : %s\n", name );
         free( name ); name = NULL;
#endif /* __MALP_MPI_T_REGISTER_DEBUG_PRINT */
         ret = 0;
         break; 
      default:
         ret = 1;
         fprintf(stderr, "error: undef MALP_mpi_t_var_type_t variable\n");
   }
   return ret;
   
}   


char ** reg_splitter( char * desc, char * data , int * ll )
{
    char * dat = strdup( data );

    char ** ret = malloc( 128 * sizeof(char *) );

    if( !ret )
    {
        perror("malloc");
        *ll = 0;
        return NULL;
    }

    int cnt = 0;

    ret[cnt] = dat;

    char * next = NULL;

    while( next = strchr(ret[cnt], ':') )
    {
        if( *(next + 1) == '\0' )
            break;

        *(next) = '\0';

        cnt++;

        ret[cnt] = next + 1;
    }


    if( 0 )
    {
        int i;

       fprintf(stderr, "== %s FILTERS ==\n", desc);
   
       for (i = 0; i <= cnt; ++i) {
           fprintf(stderr, "%d : %s\n", i, ret[i]);
       }
       
       fprintf(stderr, "================\n", desc);
   
    }

    *ll = cnt + 1;

    return ret;
}

char** malp_mpi_t_get_cvars_list( int* list_len )
{
    char * data = getenv("MALP_MPIT_CVAR_FILTER");

    if( !data )
    {
        char ** ret = malloc( sizeof(char *) );
        *list_len = 1;
        ret[0] = strdup(".*");
        return ret;
    }

   return reg_splitter( "CVAR" , data, list_len );
}

char** malp_mpi_t_get_pvars_list( int* list_len )
{
    char * data = getenv("MALP_MPIT_PVAR_FILTER");

    if( !data )
    {
        char ** ret = malloc( sizeof(char *) );
        *list_len = 1;
        ret[0] = strdup("*");
        return ret;
    }

   return reg_splitter( "PVAR" , data, list_len);
}

static int
__malp_mpi_t_get_cvars_static_infos( MALP_mpi_t_var_info_t* new_entry )
{
   int err, rank;
   MALP_mpi_t_cvar_info_t* var_data = NULL;
   
   assert( new_entry );

   var_data = &( new_entry->var.cvar );
   PMPI_Comm_rank( VMPI_Get_partition_comm(), &rank );

   /* Trigger Only MPI_CHAR CVARS OR CONST CVARS */
   if( var_data->scope != MPI_T_SCOPE_CONSTANT &&
       var_data->mpi_type != MPI_CHAR  )
       return 0;
         
   /* Intrum can't poll these events */
   if( !VMPI_partition_partition_is_analyzer() )
      return 1;

   /* Analyzer only rank 0 poll these events */
   if( rank ) return 1; 
   
   /* Push event in Static CVARS list */
   err = __malp_mpi_t_register_var_id( MALP_MPI_T_TYPE_STATIC_CVAR, new_entry );
   assert( !err );

   return 1; 
}


static inline MALP_mpi_t_var_info_t* 
__malp_mpi_t_get_cvars_infos( const int global_idx, regex_t * filter_exp )
{
   int err, desclen, namelen, verbosity;
   char desc[1024], name[1024];

   MALP_mpi_t_var_info_t* new_entry = NULL;
   MALP_mpi_t_cvar_info_t* var_data = NULL; 

   assert( global_idx >= 0 );

   new_entry = (MALP_mpi_t_var_info_t*) malloc( sizeof( MALP_mpi_t_var_info_t ) );
   assert( new_entry );
   
   new_entry->type = MALP_MPI_T_TYPE_POLL_CVAR; 
   var_data = &( new_entry->var.cvar ); 

   desclen = 1024; namelen = 1024; 
   err = PMPI_T_cvar_get_info( global_idx, name, &namelen, 
			      &( var_data->verbosity ), &( var_data->mpi_type ), 
			      &( var_data->enum_type ), desc, &desclen,
			      &( var_data->bind ), &( var_data->scope ) );
   var_data->absolute_id = global_idx;

   if( err == MPI_T_ERR_INVALID_INDEX )
   {
#ifdef MALP_MPI_T_VERBOSE_SKIPPING_SUPPORT
       fprintf(stderr, "warning: CVARS #%d invalid index ( %s, err: %s )\n", global_idx );
#endif /* MALP_MPI_T_VERBOSE_SKIPPING_SUPPORT */
       free( new_entry );
       return NULL;
   }

   if( var_data->bind != MPI_T_BIND_NO_OBJECT)
   { 
#ifdef MALP_MPI_T_VERBOSE_SKIPPING_SUPPORT
      fprintf(stderr, "warning: MALP doesn't support bind MPI_T vars. ( %s ignored)\n", name );
#endif /* MALP_MPI_T_VERBOSE_SKIPPING_SUPPORT */
      free( new_entry );
      return NULL;
   }

   assert( err == MPI_SUCCESS );

   /* Only Analyzer poll these PVARS */
   if( __malp_mpi_t_get_cvars_static_infos( new_entry ) )
   {
      free( new_entry );
      return NULL;
   }

   var_data->absolute_id = global_idx;
  
   if( filter_exp )  /* Apply filter on cvar name */
   {
      int match = regexec( filter_exp, name, 0, NULL, 0);

      if( match == REG_NOMATCH )
      {
          //fprintf(stderr, "NOMATCH %s\n", name);
          free( new_entry ); new_entry = NULL; 
      } 
      else
      {
        //fprintf(stderr, "MATCH %s\n", name);
      }
   }

   return new_entry; 
} 

static int
__malp_mpi_t_get_pvars_static_infos( MALP_mpi_t_var_info_t* new_entry )
{
   int err, rank;
   MALP_mpi_t_pvar_info_t* var_data = NULL;
   
   assert( new_entry );
 
   var_data = &( new_entry->var.pvar );
   PMPI_Comm_rank( VMPI_Get_partition_comm(), &rank );
         
   /* Trigger Only MPI_CHAR PVARS */
   if( var_data->mpi_type != MPI_CHAR  )
       return 0;

   /* Intrum can't poll these events */
   if( !VMPI_partition_partition_is_analyzer() )
      return 1;

   /* Analyzer only rank 0 poll these events */
   if( rank ) return 1; 
   
   /* Push event in Static CVARS list */
   err = __malp_mpi_t_register_var_id( MALP_MPI_T_TYPE_STATIC_PVAR, new_entry );
   assert( !err );
   return 1; 
}

static MALP_mpi_t_var_info_t* 
__malp_mpi_t_get_pvars_infos( const int global_idx, regex_t *filter_exp  )
{
   int err, desclen, namelen, verbosity;
   char desc[1024], name[1024];

   MALP_mpi_t_var_info_t* new_entry = NULL;
   MALP_mpi_t_pvar_info_t* var_data = NULL; 
   
   assert( global_idx >= 0 );

   new_entry = (MALP_mpi_t_var_info_t*) malloc( sizeof( MALP_mpi_t_var_info_t ) );
   assert( new_entry );

   new_entry->type = MALP_MPI_T_TYPE_POLL_PVAR; 
   var_data = &( new_entry->var.pvar ); 
  
   desclen = 1024; namelen = 1024; 
   err = PMPI_T_pvar_get_info( global_idx, name, &namelen, 
			      &( var_data->verbosity ), &( var_data->var_class ),
			      &( var_data->mpi_type ), &(var_data->enum_type), 
                              desc, &desclen,
			      &( var_data->bind ), &( var_data->readonly ),
                              &( var_data->continuous ), &( var_data->atomic ) );
   var_data->absolute_id = global_idx;

   if( err == MPI_T_ERR_INVALID_INDEX )
   {
#ifdef MALP_MPI_T_VERBOSE_SKIPPING_SUPPORT
       fprintf(stderr, "warning: PVARS #%d invalid index ( %s, err: %s )\n", global_idx );
#endif /* MALP_MPI_T_VERBOSE_SKIPPING_SUPPORT */
       free( new_entry );
       return NULL;
   }

   assert( err == MPI_SUCCESS );

   if( (var_data->bind != MPI_T_BIND_NO_OBJECT) && (var_data->bind != MPI_T_BIND_MPI_COMM) )
   { 
      //fprintf(stderr, "warning: MALP doesn't support bind MPI_T vars outside of BIND_COMM. ( %s ignored)\n", name );
      free( new_entry );
      return NULL;
   }

   /* Only Analyzer poll these PVARS */
   if( __malp_mpi_t_get_pvars_static_infos( new_entry ) )
   {
      free( new_entry );
      return NULL;
   }

   var_data->absolute_id = global_idx;
  
   if( filter_exp )  /* Apply filter on cvar name */
   {
      int match = regexec( filter_exp, name, 0, NULL, 0);

      if( match == REG_NOMATCH )
      {
          //fprintf(stderr, "NOMATCH %s\n", name);
          free( new_entry ); 
          return NULL;
      }
      else
      {
        //fprintf(stderr, "MATCH %s\n", name);
      }
   }

   return new_entry; 
} 

static inline char*
__malp_mpi_t_get_filter_from_list_name( char** filter_list, const int index, size_t* name_len, int* have_star )
{
   int __have_star;
   size_t __var_name_len, __star_pos_in_name;
   char* __var_name_val, *__have_start_in_name;

   /* Sanity check */
   assert( filter_list && name_len && index >= 0 );

   __var_name_val = filter_list[index];
   assert( __var_name_val );

   __var_name_len = strlen( __var_name_val );
   __have_start_in_name =  strchr( __var_name_val, '*' );

   __have_star = 0; 
   if( __have_start_in_name )
   {
      __var_name_len = (size_t) ( __have_start_in_name - (size_t) __var_name_val );
      __have_star = 1;
   } 

   *have_star = __have_star;
   *name_len = __var_name_len;
   return __var_name_val;
}

/* Should be call in MPI_Init MALP_fonction */ 
void malp_mpi_t_init_instrum( int __mpi_t_required )
{
   char* __filter_value;
   char** __cvars_filter_name_list, **__pvars_filter_name_list;

   int __mpi_t_provided;
   int i, j, err;
   int __cvars_filter_tot_num, __pvars_filter_tot_num;
   int __cvars_mpi_tot_num, __pvars_mpi_tot_num;

   MALP_mpi_t_var_type_t type;
   MALP_mpi_t_cvar_info_t *var_data;
   MALP_mpi_t_var_info_t  *new_entry, *buf_enry;

   err = PMPI_T_init_thread( __mpi_t_required, &__mpi_t_provided );   
   if( err != MPI_SUCCESS )
   {
       fprintf(stderr, "warning: PMPI_T_init_thread failed (disable MPI_T support)\n");
       return;
   }

   /* Get filter rules */
   __cvars_filter_name_list = malp_mpi_t_get_cvars_list( &__cvars_filter_tot_num );
   err = PMPI_T_cvar_get_num( &__cvars_mpi_tot_num );
   assert( err == MPI_SUCCESS );

   __pvars_filter_name_list = malp_mpi_t_get_pvars_list( &__pvars_filter_tot_num );
   err = PMPI_T_pvar_get_num( &__pvars_mpi_tot_num );
   assert( err == MPI_SUCCESS );
  
   //fprintf(stderr, "CVT %d PVT %d CVC %d PVT %d\n", __cvars_filter_tot_num, __pvars_filter_tot_num, __cvars_mpi_tot_num, __pvars_mpi_tot_num);

   for( i = 0; i < __cvars_filter_tot_num; i++ )
   {
      regex_t reg;

      int ret = regcomp( &reg, __cvars_filter_name_list[i], 0 );
      
      if( ret < 0 )
      {
        perror("regcomp");
        break;
      }

      for( j = 0; j < __cvars_mpi_tot_num; j++ ) 
      {
         new_entry = __malp_mpi_t_get_cvars_infos( j, &reg );
         
         if( new_entry )
         {
            err = __malp_mpi_t_register_var_id( MALP_MPI_T_TYPE_POLL_CVAR, new_entry );
            assert( !err );
            free( new_entry ); new_entry = NULL;
         }
      }
        
      regfree(&reg);
   }

   for( i = 0 ; i < __pvars_filter_tot_num; i++ )
   {
      regex_t reg;

      int ret = regcomp( &reg, __cvars_filter_name_list[i], 0 );
      
      if( ret < 0 )
      {
        perror("regcomp");
        break;
      }


      for( j = 0; j < __pvars_mpi_tot_num; j++ ) 
      {
         new_entry = __malp_mpi_t_get_pvars_infos( j, &reg );
         
         if( new_entry )
         {
            err = __malp_mpi_t_register_var_id( MALP_MPI_T_TYPE_POLL_PVAR, new_entry ); 
            assert( !err );
            free( new_entry ); new_entry = NULL;
         }

      }
      
      regfree(&reg);
   }
} 

#endif /* HAVE_MPI_T */
