/* ############################ MALP License ############################## */
/* # Fri Jan 18 14:00:00 CET 2013                                         # */
/* # Copyright or (C) or Copr. Commissariat a l'Energie Atomique          # */
/* #                                                                      # */
/* # This software is governed by the CeCILL-C license under French law   # */
/* # and abiding by the rules of distribution of free software.  You can  # */
/* # use, modify and/ or redistribute the software under the terms of     # */
/* # the CeCILL-C license as circulated by CEA, CNRS and INRIA at the     # */
/* # following URL http://www.cecill.info.                                # */
/* #                                                                      # */
/* # The fact that you are presently reading this means that you have     # */
/* # had knowledge of the CeCILL-C license and that you accept its        # */
/* # terms.                                                               # */
/* #                                                                      # */
/* # Authors:                                                             # */
/* #   - BESNARD Jean-Baptiste jean-baptiste.besnard@cea.fr               # */
/* #                                                                      # */
/* ######################################################################## */
#include "map_plugin.h"


#include <dirent.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <string.h>
#include <libgen.h>
#include <dlfcn.h>


#include "map_plugin_api.h"


/** Util */

static inline int fast_strcmp( const char *haystack, const char *needle )
{
	if ( !haystack || !needle )
		return 1;

	while ( (*haystack != '\0') && (*needle !='\0') )
	{
		if ( *haystack != *needle )
			return 1;

		haystack++;
		needle++;
	}

	if ( *haystack != *needle )
		return 1;

	return 0;
}


static int __is_root_process = 0;

void MAP_Plugin_set_as_root()
{
	__is_root_process = 1;
}



/** MAP_Metric_group */

int MAP_Metric_group_init( struct MAP_Metric_group * group, xmlNode * desc, struct MAP_Plugin * plugin )
{
	xmlNode * tmp = desc;
	
	/*Enter the element */
	tmp = tmp->children;

	int metric_count = 0;
	
	/* First count metrics and retrieve other params in the meantime ;=) */
	while( tmp )
	{
		if (tmp->type == XML_ELEMENT_NODE)
		{
			if( !fast_strcmp(tmp->name, "metric" ) )
				metric_count++;
			
			if( !fast_strcmp( tmp->name, "displayName" ) )
			{
				if( tmp->children )
							if( tmp->children->content )
								group->name = strdup( tmp->children->content );
			}
			
			if( !fast_strcmp( tmp->name, "description" ) )
			{
				if( tmp->children )
							if( tmp->children->content )
								group->description = strdup( tmp->children->content );
			}
		}
		
		tmp = tmp->next;
	}
	
	
	if( !group->name )
	{
		fprintf(stderr, "Error: no group name provided\n");
		return 1;
	}
	
	if( metric_count == 0 )
	{
		fprintf(stderr, "Error: no metrics in group\n");
		return 1;
	}
	
	
	/* Now allocate metrics */
	group->group_metrics = calloc( metric_count, sizeof( struct MAP_Metric * ) );
	
	if( !group->group_metrics)
	{
		perror("calloc");
		return 1;
	}
	
	/* And now point to metrics */
	
	tmp = desc->children;
	int metric_offset = 0;
	
	while( tmp )
	{
		if( !fast_strcmp(tmp->name, "metric" ) )
		{
			xmlAttr * metric_attr = tmp->properties;

			char * id = NULL;

			while( metric_attr )
			{
				if( !fast_strcmp( metric_attr->name, "ref" ) )
				{
					if( metric_attr->children )
					{
						if( metric_attr->children->content )
							id = strdup( metric_attr->children->content );
					}
				}
				
				metric_attr = metric_attr->next;
			}
			
			if(! id )
			{
				fprintf(stderr, "Error: no 'ref' attribute on metric inside group\n");
				return 1;
			}
			
			/* Now retrieve a pointer to the metric */
			struct MAP_Metric * metric = MAP_Plugin_get_metric_by_name( plugin, id );

			
			if( !metric )
			{
				fprintf(stderr, "Error: no metric '%s' found inside plugin '%s' (%s)'\n", id, plugin->source_id, plugin->path_to_source_so);
				free( id );
				return 1;
			}
		
			group->group_metrics[ metric_offset ] = metric;
			
			free( id );
			metric_offset++;
		}
		
		tmp = tmp->next;
	}
	
	
	group->group_metric_count = metric_offset;
	
	
	return 0;
}

int MAP_Metric_group_release( struct MAP_Metric_group * group )
{
	if( !group )
		return 1;
		
	free( group->id );
	group->id = NULL;
	
	free( group->name );
	group->name = NULL;
		
	free( group->description );
	group->description = NULL;
	
	free( group->group_metrics );
	group->group_metrics = NULL;
	
	group->group_metric_count = 0;
	
	return 0;
}



/** MAP_Metric */

int MAP_Metric_init( struct MAP_Metric * metric, xmlNode * desc, void * libhandle )
{
	xmlNode * tmp = desc;
	
	
	xmlAttr * metric_attr = tmp->properties;

	while( metric_attr )
	{
		if( !fast_strcmp( metric_attr->name, "id" ) )
		{
			if( metric_attr->children )
			{
				if( metric_attr->children->content )
					metric->id = strdup( metric_attr->children->content );
			}
		}
		
		metric_attr = metric_attr->next;
	}	
	
	
	
	/* Enter the Metric object */
	tmp = tmp->children;
	

    while( tmp )
    {
		if( tmp->type == XML_ELEMENT_NODE)
		{
		

			if( !fast_strcmp( tmp->name, "derivative") )
			{
				if( tmp->children )
					if( tmp->children->content )
						metric->derivative = atoi( tmp->children->content );
				//printf("Der : %u\n", metric->derivative );
			}
		
			if( !fast_strcmp( tmp->name, "dataType") )
			{
				if( tmp->children )
				{
					if( tmp->children->content )
					{
						if( !strcasecmp( tmp->children->content, "uint64_t") )
						{
							metric->data_type = MAP_METRIC_UINT64;
						}
						else if( !strcasecmp( tmp->children->content, "double") )
						{
							metric->data_type = MAP_METRIC_DOUBLE;
						}
						else
						{
							fprintf(stderr, "Error: no such metric %s\n", tmp->children->content);
							return 1;
						}
					}
				}
				
				//printf("MET : %d\n", metric->data_type );
			}
		
			if( !fast_strcmp( tmp->name, "domain") )
			{
				if( tmp->children )
				{
					if( tmp->children->content )
					{
						if( !strcasecmp( tmp->children->content, "time") )
						{
							metric->domain= MAP_DOMAIN_TIME;
						}
						else
						{
							fprintf(stderr, "Error: no such domain %s\n", tmp->children->content);
							return 1;
						}
					}
				}
				
				//printf("DOM : %d\n", metric->domain );
			}
		
			if( !fast_strcmp( tmp->name, "source") )
			{
				xmlAttr * source_attr = tmp->properties;

				while( source_attr )
				{
					if( !fast_strcmp( source_attr->name, "ref" ) )
					{
						if( source_attr->children )
						{
							if( source_attr->children->content )
								metric->data_collector_ref = strdup( source_attr->children->content );
						}
					}
					
					if( !fast_strcmp( source_attr->name, "functionName" ) )
					{
						if( source_attr->children )
						{
							if( source_attr->children->content )
							{
								/* Now try to load the function from the library */
								void * func = dlsym( libhandle, source_attr->children->content  );
								
								if( !func )
								{
									fprintf(stderr, "Error: could not load the function %s inside the plugin library\n", source_attr->children->content);
									return 1;
								}
								
								metric->data_collector = func;
							}
						}
					}
					
					if( !fast_strcmp( source_attr->name, "divideBySampleTime" ) )
					{
						if( source_attr->children )
						{
							if( source_attr->children->content )
							{
								if( !strcasecmp(  source_attr->children->content, "true" ) )
								{
									metric->data_collector_divide_by_sample = 1;
								}
								else
								{
									metric->data_collector_divide_by_sample = 0;
								}
							}
						}
						
						//printf("DAT : %p\n", metric->data_collector );
						//printf("DAR : %s\n", metric->data_collector_ref );
						//printf("DAD : %d\n", metric->data_collector_divide_by_sample );
					}
					
					source_attr = source_attr->next;
				}	
	
			}
			
			
			if( !fast_strcmp( tmp->name, "display") )
			{
				xmlNode * tmp2 = tmp;
			
				tmp2 = tmp2->children;
			
				while( tmp2 )
				{
			
					if( !fast_strcmp( tmp2->name, "displayName") )
					{
						if( tmp2->children )
							if( tmp2->children->content )
								metric->display_name = strdup( tmp2->children->content );
						//printf("DISP : %s\n", metric->display_name );
					}
					
					if( !fast_strcmp( tmp2->name, "description") )
					{
						if( tmp2->children )
							if( tmp2->children->content )
								metric->description = strdup( tmp2->children->content );
						//printf("DESC : %s\n", metric->description );
					}
					
					if( !fast_strcmp( tmp2->name, "type") )
					{
						if( tmp2->children )
							if( tmp2->children->content )
								metric->type = strdup( tmp2->children->content );
						//printf("TYPE : %s\n", metric->type );
					}
					
					if( !fast_strcmp( tmp2->name, "colour") )
					{
						if( tmp2->children )
							if( tmp2->children->content )
								metric->color = strdup( tmp2->children->content );
						//printf("COL : %s\n", metric->color );	
					}
				
					tmp2 = tmp2->next;
				}
			
			}
			
		}
		
		
		if( !tmp->next )
			tmp = tmp->children;
		else
			tmp = tmp->next;
	}
	
	
	/* Now check what was loaded */
	
	if( !metric->data_collector )
	{
		fprintf(stderr, "Error: no source function provided\n");
		return 1;
	}
	
	if( metric->data_type == MAP_METRIC_NONE )
	{
		fprintf(stderr, "Error: no data type provided\n");
		return 1;
	}
	
	if( metric->domain == MAP_DOMAIN_NONE )
	{
		fprintf(stderr, "Error: no domain provided\n");
		return 1;
	}
	
	if( metric->display_name == NULL )
	{
		fprintf(stderr, "Error: no display name provided\n");
		return 1;
	}
	
	if( metric->type == NULL )
	{
		fprintf(stderr, "Error: no display type provided\n");
		return 1;
	}
	
	metric->error_message = NULL;
	metric->return_code = 0;


	/* previous_ts < 0 <==> No previous */
	metric->previous_ts = -1.0;
	
	/* No payload */
	metric->payload = NULL;

	return 0;
}

int MAP_Metric_release( struct MAP_Metric * metric )
{
	if( !metric )
		return 1;
	
	free( metric->id );
	metric->id = NULL;
	
	free( metric->data_collector_ref );
	metric->data_collector_ref = NULL;
		
	free( metric->display_name );
	metric->display_name = NULL;
		
	free( metric->description );
	metric->description = NULL;
		
	free( metric->type );
	metric->type = NULL;
	
	free( metric->color );
	metric->color = NULL;
	
	memset( metric, 0, sizeof( struct MAP_Metric ) );

	return 0;
}



/** MAP_Plugin */


/**
 * @brief Checks if a file exists
 * @param path the path to the file to check
 * @return 1 if the file exists, 0 otherwise
 */
static inline int file_exists( const char *path )
{
	struct stat statb;

	if (lstat(path, &statb) == -1) {
		return 0;
	}

	return 1;
}




int MAP_Plugin_load( struct MAP_Plugin * plugin, char * xml_file )
{
	if( !plugin )
		return 1;
	
	int ret = 0;
	/* First try to load the XML */
	xmlDocPtr xml_doc;

	xml_doc = xmlReadFile(xml_file, NULL, 0);

	if(xml_doc == NULL)
	{
		fprintf(stderr, "Error: could not parse %s\n", xml_file);
		return 1;
	}

	/* Now  extract and validate data */
    xmlNode *root_element = xmlDocGetRootElement(xml_doc);
    
    if( !root_element)
    {
		fprintf(stderr, "Error: Could not find a root element\n");
		ret = 1;
		goto MAP_Plugin_load_end;	
	}
    
    
    if( fast_strcmp( root_element->name, "metricdefinitions") )
    {
		fprintf(stderr, "Error: Root element should be of 'metricdefinitions' type, found '%s'\n", root_element->name);
		ret = 1;
		goto MAP_Plugin_load_end;	
	}
	
	
	/* Count sub_elements */
	
	int metric_count = 0;
	int metric_group_count = 0;
	
	xmlNode * tmp = root_element;
	
	//print_element_names( tmp, 0 );
	
	xmlNode * source_info_node = NULL;
    
    /* Skip first root entry */
    tmp = tmp->children;

    while( tmp )
    {
		if( tmp->type == XML_ELEMENT_NODE)
		{
		
			if( !fast_strcmp( tmp->name, "metric" ) )
				metric_count++;
			
			if( !fast_strcmp( tmp->name, "metricGroup" ) )
				metric_group_count++;
			
			if( !fast_strcmp( tmp->name, "source" ) )
				source_info_node = tmp;
			
			if( !fast_strcmp( tmp->name, "metricdefinitions") )
			{
				fprintf(stderr, "Error: found two 'metricdefinitions' nodes in the XML file\n");
				ret = 1;
				goto MAP_Plugin_load_end;
			}
			
		}
		
		
		if( !tmp->next )
			tmp = tmp->children;
		else
			tmp = tmp->next;
	}
    
    /* First extract the source infos */
    if( source_info_node == NULL )
    {
			fprintf(stderr, "Error: Plugin does not provide any source information\n");
			ret = 1;
			goto MAP_Plugin_load_end;
	}

	/* Find the source id */

	xmlAttr * prop = source_info_node->properties;
	
	plugin->source_id = NULL;
	
	while( prop )
	{
		if( !fast_strcmp( prop->name, "id" ) )
		{
			if( prop->children )
			{
				if( prop->children->content )
					plugin->source_id = strdup( prop->children->content );
			}
		}
		
		prop = prop->next;
	}	
	
	xmlNode * shared_lib = source_info_node->children;
	
	/* Go the the first child node */
	while( shared_lib )
	{

		if( shared_lib->type == XML_ELEMENT_NODE)
			break;
		
		shared_lib = shared_lib->next;
	}
	
	if(!shared_lib )
	{
		fprintf(stderr, "Error: Source element is empty\n");
		ret = 1;
		goto MAP_Plugin_load_end;
	}
	
	if( fast_strcmp( shared_lib->name , "sharedLibrary" ) )
	{
		fprintf(stderr, "Error: Source is not a shared lib (had %s)\n",shared_lib->name);
		ret = 1;
		goto MAP_Plugin_load_end;
	}
	else
	{
		if( shared_lib->children )
		{
			char * path_to_so = strdup(shared_lib->children->content);
			
			if( !path_to_so )
			{
				fprintf(stderr, "Error: could not retrieve path to .so\n");
				ret = 1;
				goto MAP_Plugin_load_end;
			}
			
			/* Now determine if absolute or relative */
			if( !file_exists( path_to_so ) )
			{
				/* Assume that the file is in the prefix */
				char * prefix = dirname( xml_file );
				
				int len = strlen( prefix ) + strlen( path_to_so ) + 20;
				
				char * tmp = malloc( len * sizeof( char ) );
				
				if( !tmp )
				{
					perror("malloc");
					ret = 1;
					goto MAP_Plugin_load_end;	
				}

				snprintf( tmp, len, "%s/%s", prefix, path_to_so );
					
				free( path_to_so );
				path_to_so = tmp;
				
				if( !file_exists( path_to_so ) )
				{
					fprintf(stderr, "Error: could not locate .so file in prefix (%s)\n", path_to_so);
					ret = 1;
					goto MAP_Plugin_load_end;
				}
				
			}

			/* We have the SO path ! */
			plugin->path_to_source_so = path_to_so;
		}
		else
		{
			fprintf(stderr, "Error: sharedLibrary tag is empty\n");
			ret = 1;
			goto MAP_Plugin_load_end;
		}
		
	}
	
	/* It's now time to open the dynamic library */
	void * handler = dlopen( plugin->path_to_source_so, RTLD_LAZY);
	
	if( !handler )
	{
		fprintf(stderr, "Error: failed to load the target .so (%s)\n",plugin->path_to_source_so);
		ret = 1;
		goto MAP_Plugin_load_end;
	}
	
	plugin->libhandle = handler;
	
	/* And now load the library interface */
	int (*allinea_plugin_initialize) (plugin_id_t , void*);
	int (*allinea_plugin_cleanup) (plugin_id_t , void* );
	int (*mymetric_getIntValue) (metric_id_t , struct timespec *, uint64_t *);
	int (*mymetric_getDoubleValue) (metric_id_t , struct timespec *, double *);
	
	
	allinea_plugin_initialize = (int (*) (plugin_id_t , void*)) dlsym(handler, "allinea_plugin_initialize");
	
	if( !allinea_plugin_initialize )
	{
		fprintf(stderr, "Error: failed to load the allinea_plugin_initialize function in %s\n",plugin->path_to_source_so);
		ret = 1;
		goto MAP_Plugin_load_end;	
	}
	
	allinea_plugin_cleanup = (int (*) (plugin_id_t , void* )) dlsym(handler, "allinea_plugin_cleanup");

	if( !allinea_plugin_cleanup )
	{
		fprintf(stderr, "Error: failed to load the allinea_plugin_cleanup function in %s\n",plugin->path_to_source_so);
		ret = 1;
		goto MAP_Plugin_load_end;	
	}
	
	/* These two seems to be optionnal */
	
	mymetric_getIntValue = (int (*) (metric_id_t , struct timespec *, uint64_t *)) dlsym(handler, "mymetric_getIntValue");
	mymetric_getDoubleValue = (int (*) (metric_id_t , struct timespec *, double *)) dlsym(handler, "mymetric_getDoubleValue");

	/* Save in plugin */

	plugin->allinea_plugin_initialize = allinea_plugin_initialize;
	plugin->allinea_plugin_cleanup = allinea_plugin_cleanup;
	plugin->mymetric_getIntValue = mymetric_getIntValue;
	plugin->mymetric_getDoubleValue = mymetric_getDoubleValue;
	
    /* Now First allocate sub_elements */
    plugin->metrics = calloc( metric_count, sizeof( struct MAP_Metric) );
    
    if( !plugin->metrics )
    {
		perror("calloc");
		ret = 1;
		goto MAP_Plugin_load_end;	
	}
    
    
    plugin->metric_groups = calloc( metric_group_count, sizeof( struct MAP_Metric_group) );
    
    if( !plugin->metric_groups )
    {
		perror("calloc");
		ret = 1;
		goto MAP_Plugin_load_end;	
	}
    

	/* Walk all element to extract metrics */
	tmp = root_element;
    
    /* Skip first root entry */
    tmp = tmp->children;
    
    int current_metric_offset = 0;
    int current_metric_group_offset = 0;
    

    while( tmp )
    {
		if( tmp->type == XML_ELEMENT_NODE)
		{
		
			if( !fast_strcmp( tmp->name, "metric" ) )
			{
				if( MAP_Metric_init( plugin->metrics + current_metric_offset, tmp, plugin->libhandle ) )
				{
					fprintf(stderr, "Error: failed to load a metric\n");
					return 1;
				}
				else
				{
					/* Metric loaded */
					current_metric_offset++;
				}
				
			}
		}
				
		tmp = tmp->next;
	}

	if( current_metric_offset == 0 )
	{
		fprintf(stderr, "Error: no metric loaded\n");
		ret = 1;
		goto MAP_Plugin_load_end;	
	}
	
	
	/* Save actual count inside the plugin */
	plugin->metric_count = current_metric_offset;

	/* Walk all element to extract groups */
	tmp = root_element;
    
    /* Skip first root entry */
    tmp = tmp->children;
    
    while( tmp )
    {
		if( tmp->type == XML_ELEMENT_NODE)
		{
		
			if( !fast_strcmp( tmp->name, "metricGroup" ) )
			{
				if(MAP_Metric_group_init( plugin->metric_groups + current_metric_group_offset, tmp, plugin ))
				{
					fprintf(stderr, "Error: failed to load a group\n");
					return 1;
				}
				else
				{
					/* Metric group loaded */
					current_metric_group_offset++;
				}
			}
		}
		
		tmp = tmp->next;
	}

	plugin->metric_group_count = current_metric_group_offset;
    
    //print_element_names( root_element , 0);
  	plugin->error_message = NULL;
	plugin->return_code = 0;  
    
    	
	/* Now call the initializer */
	(plugin->allinea_plugin_initialize)( (plugin_id_t) plugin, NULL );
    
    MAP_Plugin_check_error( plugin );
    
MAP_Plugin_load_end:
    xmlFreeDoc(xml_doc);
	
	return ret;
}



struct MAP_Metric * MAP_Plugin_get_metric_by_name( struct MAP_Plugin * plugin, char * name )
{
	if(!plugin)
		return NULL;
	
	int i;
	
	for( i = 0 ; i < plugin->metric_count ; i++ )
	{
		if( !fast_strcmp( name,  plugin->metrics[i].id) )
			return &plugin->metrics[i];
	}
	
	return NULL;
}


int MAP_Plugin_release( struct MAP_Plugin * plugin )
{
	if( !plugin )
		return 1;

    	
	/* Now call the cleanup */
	(plugin->allinea_plugin_cleanup)( (plugin_id_t) plugin, NULL );
	
	MAP_Plugin_check_error( plugin );
    

	free( plugin->source_id );
	plugin->source_id = NULL;

	free( plugin->path_to_source_so );
	plugin->path_to_source_so = NULL;
	
	free( plugin->path_to_source_so );
	plugin->path_to_source_so = NULL;
	
	int i;
	
	for( i = 0 ; i < plugin->metric_count ; i++ )
	{
		MAP_Metric_release( &plugin->metrics[i] );
	}
	
	plugin->metric_count = 0;
	free( plugin->metrics );
	plugin->metrics = NULL;
	
	
	for( i = 0 ; i < plugin->metric_group_count ; i++ )
	{
		MAP_Metric_group_release( &plugin->metric_groups[i] );
	}
	
	plugin->metric_group_count = 0;
	free( plugin->metric_groups );
	plugin->metric_groups = NULL;
	
	return 0;
}





/** MAP_Plugin_array */

int __MAP_Plugin_array_load_selector( const struct dirent * entry )
{
	const char * n = entry->d_name;
	
	int len = strlen( n );
	
	if( len < 4 )
		return 0;
		
	if( n[ len - 1 ] == 'l'
	&&  n[ len - 2 ] == 'm'
	&&  n[ len - 3 ] == 'x'
	&&  n[ len - 4 ] == '.' )
		return 1;
	
	return 0;
}




int MAP_Plugin_array_load( struct MAP_Plugin_array * array, char * plugin_prefix )
{
	if( ! array )
		return 1;
	
	
	/* Save prefix */
	array->plugin_prefix = strdup( plugin_prefix );
	
	/* LIST xml files */
	struct dirent ** xml_files = NULL;
	
	int xml_file_count = scandir( plugin_prefix, &xml_files, __MAP_Plugin_array_load_selector, alphasort );
	
	/* Error checking */
	
	if( xml_file_count < 0 )
	{
		perror("Could not list plugins (scandir)");
		return 1;
	}
	
	if( xml_file_count == 0 )
	{
		fprintf(stderr, "No plugin were located in this prefix (%s)\n", plugin_prefix );
		return 1;
	}
	
	/* Allocate plugin storage */
	
	array->plugins = calloc( xml_file_count, sizeof(struct MAP_Plugin) );
	
	if( !array->plugins )
	{
		perror("calloc");
		return 1;
	}
	
	
	/* Load plugins */
	
	int i;
	int plugin_offset = 0;
	char full_path_to_xml[800];
	
	int plugin_count = xml_file_count;
	
	
	for( i = 0 ; i < xml_file_count; i++ )
	{
		snprintf(full_path_to_xml, 800, "%s/%s", plugin_prefix, xml_files[i]->d_name );
		
		if( __is_root_process )
			printf("Loading plugin '%s' ...\n", xml_files[i]->d_name );
		
		if( MAP_Plugin_load( array->plugins + plugin_offset, full_path_to_xml) )
		{
			/* Plugin load failed for any reason, skip this plugin */
			
			/* Eventually we will lose this plugin */
			plugin_count--;
		}
		else
		{
			/* Plugin loaded correctly */
			plugin_offset++;
		}
		
	}
	
	/* Now save the number of plugins actually loaded */
	array->plugin_count = plugin_count;
	
	/* Is there any plugin ? */
	if( ! array->plugin_count )
	{
		fprintf( stderr, "Error : failed to load any plugin in %s\n", plugin_prefix );
		return 1;
	}

	
	return 0;
}




int MAP_Plugin_array_release( struct MAP_Plugin_array * array )
{
	if( !array )
		return 1;
		
	int i;
	
	for( i = 0 ; i < array->plugin_count ; i++ )
	{
		if( MAP_Plugin_release( &array->plugins[i] ) )
		{
			fprintf( stderr, "Error : releasing plugin %s\n", array->plugins[i].source_id );
			return 1;
		}
	}
	
	
	array->plugin_count = 0;
	free( array->plugins );
	array->plugins = NULL;
	
	free( array->plugin_prefix );
	array->plugin_prefix = NULL;
	
	return 0;
}


int MAP_Plugin_array_trigger( struct MAP_Plugin_array * array , void (*handler)( struct MAP_Plugin * plugin,
																  struct MAP_Metric * metric,
																  int plugin_id,
																  int metric_id,
																  struct timespec * timestamp,
																  MAP_Metric_type type, 
																  double dvalue, 
																  uint64_t ulivalue,
																  void * arg),
																  void * arg )
{
	if( !array || !handler)
	{
		fprintf(stderr, "Error: Missing argument\n");
		return 1;
	}
	
	int i,j;
	
	for( i = 0 ; i < array->plugin_count ; i++ )
	{
		
		struct MAP_Plugin * plugin = &array->plugins[i];
		
		for( j = 0 ; j < plugin->metric_count ; j++ )
		{
			struct MAP_Metric * metric = &plugin->metrics[j];
			
			struct timespec ts;

			uint64_t ulivalue = 0;
			double dvalue = 0.0;
			int ret = 0;
			
			void * metric_handler = metric->data_collector;
			
			int (*double_function)(metric_id_t , struct timespec * , double *) = (int (*)(metric_id_t , struct timespec *, double *) )metric_handler;
			int (*uint64t_function)(metric_id_t , struct timespec *, uint64_t *) = (int (*)(metric_id_t , struct timespec *, uint64_t *))metric_handler ;
			
			
			/* Get TS */
			ts = allinea_get_current_time();
			
			/* Do we need to normalize ? */
			int please_normalize = 0;
			double normalization_interval = 0.0;
			
			if( metric->data_collector_divide_by_sample )
			{
				/* Do we have a previous sample ? */
				if( 0 < metric->previous_ts )
				{
					please_normalize = 1;
					normalization_interval = ts_convert_to_double( &ts ) - metric->previous_ts;
					/* Now that we know DELTA store current as previous */
					metric->previous_ts =  ts_convert_to_double( &ts );
				}
				else
				{
					/* FIRST POINT Save current sample */
					metric->previous_ts = ts_convert_to_double( &ts );
										
					/* Skip this point to process next sample
					 * on a known time interval */
					continue;
				}
				
			}
			
			switch( metric->data_type )
			{
				case MAP_METRIC_DOUBLE:
					ret = (double_function)((metric_id_t)metric, &ts, &dvalue);
					
					//fprintf( stderr, "%d = %d = %g // %g\n", i, j, dvalue , normalization_interval);
					
					if( please_normalize )
						dvalue /= normalization_interval;
					
				break;
				case MAP_METRIC_UINT64:
					ret = (uint64t_function)((metric_id_t)metric, &ts, &ulivalue);
					
					if( please_normalize )
						ulivalue /= normalization_interval;
				break;
				
				default:
					fprintf(stderr, "Error: No such metric type %d\n", metric->data_type);
					return 1;
					
			}
			
			MAP_Metric_check_error( metric );
			MAP_Plugin_check_error( plugin );
	
			/* Forward the point to the handler */
			(handler)( plugin, metric, i, j,  &ts, metric->data_type, dvalue, ulivalue, arg);
		}
		
	}
	
}
