/* ############################ MALP License ############################## */
/* # Fri Jan 18 14:00:00 CET 2013                                         # */
/* # Copyright or (C) or Copr. Commissariat a l'Energie Atomique          # */
/* #                                                                      # */
/* # This software is governed by the CeCILL-C license under French law   # */
/* # and abiding by the rules of distribution of free software.  You can  # */
/* # use, modify and/ or redistribute the software under the terms of     # */
/* # the CeCILL-C license as circulated by CEA, CNRS and INRIA at the     # */
/* # following URL http://www.cecill.info.                                # */
/* #                                                                      # */
/* # The fact that you are presently reading this means that you have     # */
/* # had knowledge of the CeCILL-C license and that you accept its        # */
/* # terms.                                                               # */
/* #                                                                      # */
/* # Authors:                                                             # */
/* #   - BESNARD Jean-Baptiste jean-baptiste.besnard@cea.fr               # */
/* #                                                                      # */
/* ######################################################################## */
#ifndef MALP_MAP_PLUGIN_API
#define MALP_MAP_PLUGIN_API


#ifdef __cplusplus
extern "C" {
#endif

#include <time.h>
#include <stdint.h>
#include <stdarg.h>
#include <stdlib.h>
#include "allinea_metric_plugin_types.h"

/** ERROR reporting API */

/* Plugin Level */
void allinea_set_plugin_error_message(plugin_id_t plugin_id, int error_code, const char *error_message);
void allinea_set_plugin_error_messagef(plugin_id_t plugin_id, int error_code, const char *error_message, ...);

/* Metric Level */
void allinea_set_metric_error_message(metric_id_t metric_id, int error_code, const char *error_message);
void allinea_set_metric_error_messagef(metric_id_t metric_id, int error_code, const char *error_message, ...);


/** Machine discovery API */

/* Including HT */
int allinea_get_logical_core_count();

/* Logical Cores */
int allinea_get_physical_core_count();

/** Safe Syscall API */

/* Print */
void allinea_safe_printf(const char *format, ...);
void allinea_safe_fprintf(int fd, const char *format, ...);
void allinea_safe_vfprintf(int fd, const char *format, va_list ap);

/* Open Close */
int allinea_safe_open (const char *file, int oflags, ...);
int allinea_safe_close(int fd);

/* IO operations */
ssize_t allinea_safe_read(int fd, void *buf, size_t count);
ssize_t allinea_safe_write(int fd, const void *buf, size_t count);
ssize_t allinea_safe_read_all(int fd, void *buf, size_t count);
ssize_t allinea_safe_read_line(int fd, void *buf, size_t count);

/* Timers */
struct timespec allinea_get_current_time();


static inline double ts_convert_to_double( struct timespec * ts )
{
	double sec = (double)ts->tv_sec;
	double nsec = (double)ts->tv_nsec;
	
	return sec + nsec * 1e-9;
}


static inline double ts_caculate_aminusb( struct timespec * tsa,  struct timespec * tsb )
{
	double a = ts_convert_to_double( tsa );
	double b = ts_convert_to_double( tsb );
	
	return a - b;
}


#ifdef __cplusplus
}
#endif


#endif /* MALP_MAP_PLUGIN_API */
