#ifndef NSC_H
#define NSC_H

#include "jsonCache.h"


struct Context
{
	char *passwd;
	char *addr;
	char *s_port;
	int port;
};


int Context_init( struct Context *ctx );
void Context_release( struct Context *ctx );

enum
{
	NSC_SET=0,
	NSC_DELETE=1,
	NSC_BCAST=2,
	NSC_REDUCE=3,
	NSC_CLOSE=4
};



struct nodeSharedCache
{
	struct Context ctx;

	int been_waited;

	uint32_t max_key_len;
	uint32_t max_buffer_size;

	pthread_t th;
	int socket;
	pthread_mutex_t socket_w_lock;
	int connected;
	
	int listening_socket;
	int *clients;
	int expected_clients;
	int connected_clients;
	pthread_mutex_t *client_socket_w_lock;
	pthread_t *client_threads;

	struct jsonCache cache;
};


void nodeSharedCache_set_handler( char *key, json_t *elem, void *ctx );
void nodeSharedCache_delete_handler( char *key, void *ctx );

void nodeSharedCache_commit( struct nodeSharedCache *nsc, char *key );
void nodeSharedCache_delete( struct nodeSharedCache *nsc, char *key );

char * nodeSharedCache_get_json( struct nodeSharedCache *nsc, char *key );
json_t * nodeSharedCache_get( struct nodeSharedCache *nsc, char *key );

void nodeSharedCache_set( struct nodeSharedCache *nsc, char *key, json_t *json );
void nodeSharedCache_set_json( struct nodeSharedCache *nsc, char *key, char *json );

void nodeSharedCache_reduce_json( struct nodeSharedCache *nsc, char *key, char *json );
void nodeSharedCache_reduce( struct nodeSharedCache *nsc, char *key, json_t *elem );

int nodeSharedCache_init( struct nodeSharedCache *nsc );
int nodeSharedCache_proxy_init( struct nodeSharedCache *nsc, char *destination_server, int expected_clients );
int nodeSharedCache_proxy_accept_clients( struct nodeSharedCache *nsc );
void nodeSharedCache_wait( struct nodeSharedCache *nsc);
void nodeSharedCache_release( struct nodeSharedCache *nsc);


#endif /* NSC_H */
