#ifndef __MALP_MPI_T_EVENTS_H__ 
#define __MALP_MPI_T_EVENTS_H__



#ifdef __cplusplus
extern "C"
{
#endif

#include "config.h"

#if HAVE_MPI_T

#include <string.h>
#include <stdint.h>
#include <stdio.h>

#include "mpi.h"

typedef union MALP_MPI_T_Datatype_value_u
{
   int val_int;
   unsigned int val_unsigned_int; 
   unsigned long val_unsigned_long; 
   unsigned long long val_unsigned_long_long; 
   uint64_t val_mpi_count; /* ?! */
   char val_char;
   double val_double;
   uint64_t val_uint64_t;
} MALP_MPI_T_Datatype_value_t;

typedef enum 
{  
   MALP_MPI_T_TYPE_UNDEF 	= 0,
   MALP_MPI_T_TYPE_STATIC_CVAR 	= 1, 
   MALP_MPI_T_TYPE_STATIC_PVAR 	= 2, 
   MALP_MPI_T_TYPE_POLL_CVAR 	= 3, 
   MALP_MPI_T_TYPE_POLL_PVAR 	= 4,
   MALP_MPI_T_TYPE_COUNT 	= 5 
} MALP_mpi_t_var_type_t;

typedef enum {
MALP_MPI_DOUBLE,
MALP_MPI_UINT64,
MALP_MPI_COUNT,
MALP_MPI_UNSIGNED_LONG,
MALP_MPI_UNSIGNED_LONG_LONG,
MALP_MPI_UNSIGNED_INT,
MALP_MPI_INT
}MALP_MPIT_types;


typedef struct MALP_MPI_T_event_s
{
   int logical_var_idx;
   MALP_mpi_t_var_type_t type;
   MALP_MPIT_types mpi_type;
   MALP_MPI_T_Datatype_value_t value;
} MALP_MPI_T_event_t;

typedef struct MALP_mpi_t_cvar_info_s
{  
    int bind;
    int scope;
    int verbosity;
    int absolute_id;
    int count;
    MPI_T_enum enum_type;
    MPI_Datatype mpi_type;
} MALP_mpi_t_cvar_info_t;




static inline MALP_MPIT_types MALP_mpi_t_type_encode(  MPI_Datatype type )
{

  if( type  ==  MPI_DOUBLE)
  {
    return MALP_MPI_DOUBLE;
  }
/*   if( type ==  MPI_UINT64)
  { 
    return 1;
  } */
  else if( type  ==  MPI_COUNT)
  {
    return MALP_MPI_COUNT;
  }
  else if( type  ==  MPI_UNSIGNED_LONG)
  {
    return MALP_MPI_UNSIGNED_LONG;
  }
  else if( type  ==  MPI_UNSIGNED_LONG_LONG)
  {
    return MALP_MPI_UNSIGNED_LONG_LONG;
    
  }
/*   else if( type ==  MPI_UNSIGNED_INT)
  {
    return 1;
    
  } */
  else if( type  ==  MPI_INT)
  {
    return MALP_MPI_INT;
  }

  return 0;
}


static inline int MALP_mpi_t_is_numeric( MALP_MPI_T_event_t * var )
{

  if( var->mpi_type  ==  MALP_MPI_DOUBLE)
  {
    return 1;
  }
/*   if( type ==  MALP_MPI_UINT64)
  { 
    return 1;
  } */
  else if( var->mpi_type  ==  MALP_MPI_COUNT)
  {
    return 1;
  }
  else if( var->mpi_type  ==  MALP_MPI_UNSIGNED_LONG)
  {
    return 1;
  }
  else if( var->mpi_type  ==  MALP_MPI_UNSIGNED_LONG_LONG)
  {
    return 1;
    
  }
/*   else if( type ==  MALP_MPI_UNSIGNED_INT)
  {
    return 1;
    
  } */
  else if( var->mpi_type  ==  MALP_MPI_INT)
  {
    return 1;
  }

  return 0;
}



static inline double MALP_mpi_t_to_double( MALP_MPI_T_event_t * var )
{
  double ret = 0;
  

  if( !MALP_mpi_t_is_numeric(var) )
    return ret;


  if( var->mpi_type ==  MALP_MPI_DOUBLE)
  {
    ret = var->value.val_double;
  }
/*   if( type ==  MALP_MPI_UINT64)
  { 
    ret = var->value.val_uint64_t;
  } */
  else if( var->mpi_type  ==  MALP_MPI_COUNT)
  {
    ret = var->value.val_mpi_count;
  }
  else if( var->mpi_type  ==  MALP_MPI_UNSIGNED_LONG)
  {
    ret = var->value.val_unsigned_long;
  }
  else if( var->mpi_type  ==  MALP_MPI_UNSIGNED_LONG_LONG)
  {
    ret = var->value.val_unsigned_long_long;
    
  }
/*   else if( type ==  MALP_MPI_UNSIGNED_INT)
  {
    ret = var->value.val_unsigned_int;
    
  } */
  else if( var->mpi_type  ==  MALP_MPI_INT)
  {
    ret = var->value.val_int;
  }

  return ret;
}



typedef struct MALP_mpi_t_pvar_info_s
{
   int bind;
   int scope;
   int verbosity;
   int var_class;
   int absolute_id;
   int readonly; 
   int continuous;
   int atomic;
   int count;
   MPI_T_enum enum_type;
   MPI_Datatype mpi_type;
} MALP_mpi_t_pvar_info_t;

typedef union MALP_mpi_t_generic_var_s
{
   MALP_mpi_t_cvar_info_t cvar; 
   MALP_mpi_t_pvar_info_t pvar;
} MALP_mpi_t_generic_var_t;

typedef struct MALP_mpi_t_var_info_s
{ 
   MALP_mpi_t_var_type_t type;
   MALP_mpi_t_generic_var_t var;
} MALP_mpi_t_var_info_t;

static inline void
MALP_mpi_t_remove_invalid_character_from_string( char* string )
{
  int i;
  const int string_len = strlen( string );
  for( i = 0; i < string_len; i++ )
     if( string[i] == '\"' ) string[i] = ' '; 
}

static inline char* 
MALP_mpi_t_translate_PVAR_VAR_CLASS( const int var_class )
{
   char* ret = NULL;

   switch( var_class )
   {
      case MPI_T_PVAR_CLASS_STATE:
         ret = strdup( "MPI_T_PVAR_CLASS_STATE");
         break;
      case MPI_T_PVAR_CLASS_LEVEL:
         ret = strdup( "MPI_T_PVAR_CLASS_LEVEL");
         break;
      case MPI_T_PVAR_CLASS_SIZE:
         ret = strdup( "MPI_T_PVAR_CLASS_SIZE");
         break;
      case MPI_T_PVAR_CLASS_PERCENTAGE:
         ret = strdup( "MPI_T_PVAR_CLASS_PERCENTAGE");
         break;
      case MPI_T_PVAR_CLASS_HIGHWATERMARK:
         ret = strdup( "MPI_T_PVAR_CLASS_HIGHWATERMARK");
         break;
      case MPI_T_PVAR_CLASS_LOWWATERMARK:
         ret = strdup( "MPI_T_PVAR_CLASS_LOWWATERMARK");
         break;
      case MPI_T_PVAR_CLASS_COUNTER:
         ret = strdup( "MPI_T_PVAR_CLASS_COUNTER");
         break;
      case MPI_T_PVAR_CLASS_AGGREGATE:
         ret = strdup( "MPI_T_PVAR_CLASS_AGGREGATE");
         break;
      case MPI_T_PVAR_CLASS_TIMER:
         ret = strdup( "MPI_T_PVAR_CLASS_TIMER");
         break;
      case MPI_T_PVAR_CLASS_GENERIC:
         ret = strdup( "MPI_T_PVAR_CLASS_GENERIC");
         break;
      default:
        ret = strdup( "MPI_T_PVAR_CLASS_UNDEF");
   }
   return ret;
} 

static inline char*
MALP_mpi_t_translate_MPI_T_VERBOSITY( const int verbosity )
{
   char* ret;

   switch( verbosity )
   {
      case MPI_T_VERBOSITY_USER_BASIC:
         //ret = strdup( "MPI_T_VERBOSITY_USER_BASIC" );
         ret = strdup( "USER_BASIC" );
         break;
      case MPI_T_VERBOSITY_USER_DETAIL:
         //ret = strdup( "MPI_T_VERBOSITY_USER_DETAIL" );
         ret = strdup( "USER_DETAIL" );
         break;
      case MPI_T_VERBOSITY_USER_ALL:
         //ret = strdup( "MPI_T_VERBOSITY_USER_ALL" );
         ret = strdup( "USER_ALL" );
         break;
      case MPI_T_VERBOSITY_TUNER_BASIC:
         //ret = strdup( "MPI_T_VERBOSITY_TUNER_BASIC" );
         ret = strdup( "TUNER_BASIC" );
         break;
      case MPI_T_VERBOSITY_TUNER_DETAIL:
         //ret = strdup( "MPI_T_VERBOSITY_TUNER_DETAIL" );
         ret = strdup( "TUNER_DETAIL" );
         break;
      case MPI_T_VERBOSITY_TUNER_ALL:
         //ret = strdup( "MPI_T_VERBOSITY_TUNER_ALL" );
         ret = strdup( "TUNER_ALL" );
         break;
      case MPI_T_VERBOSITY_MPIDEV_BASIC:
         //ret = strdup( "MPI_T_VERBOSITY_MPIDEV_BASIC" );
         ret = strdup( "MPIDEV_BASIC" );
         break;
      case MPI_T_VERBOSITY_MPIDEV_DETAIL:
         //ret = strdup( "MPI_T_VERBOSITY_MPIDEV_DETAIL" );
         ret = strdup( "MPIDEV_DETAIL" );
         break;
      case MPI_T_VERBOSITY_MPIDEV_ALL:
         //ret = strdup( "MPI_T_VERBOSITY_MPIDEV_ALL" );
         ret = strdup( "MPIDEV_ALL" );
         break;
      default:
        //ret = strdup( "MPI_T_VERBOSITY_UNDEF" );
        ret = strdup( "UNDEF" );
   }
   return ret;
}

static inline char*
MALP_mpi_t_translate_MPI_Datatype( const MPI_Datatype type )
{
   if( type == MPI_INT )
      return strdup( "MPI_INT" );

   if( type == MPI_UNSIGNED )
      return strdup( "MPI_UNSIGNED" );

   if( type == MPI_UNSIGNED_LONG )
      return strdup( "MPI_UNSIGNED_LONG" );

   if( type == MPI_UNSIGNED_LONG_LONG )
      return strdup( "MPI_UNSIGNED_LONG_LONG" );

   if( type == MPI_COUNT )
      return strdup( "MPI_COUNT" );

   if( type == MPI_CHAR )
      return strdup( "MPI_CHAR" );

   if( type == MPI_DOUBLE )
      return strdup( "MPI_DOUBLE" );

   /* ERROR - Defualt case */
   return strdup( "MPI_DATATYPE_UNDEF" );
}

static inline char*
MALP_mpi_t_translate_MPI_CVAR_SCOPE( const int scope )
{
   char* ret;

   switch( scope )
   {
      case MPI_T_SCOPE_CONSTANT:
         //ret = strdup( "MPI_T_SCOPE_CONSTANT" );
         ret = strdup( "CONSTANT" );
         break;
      case MPI_T_SCOPE_READONLY:
         //ret = strdup( "MPI_T_SCOPE_READONLY" );
         ret = strdup( "READONLY" );
         break;
      case MPI_T_SCOPE_LOCAL:
         //ret = strdup( "MPI_T_SCOPE_LOCAL" );
         ret = strdup( "LOCAL" );
         break;
      case MPI_T_SCOPE_GROUP:
         //ret = strdup( "MPI_T_SCOPE_GROUP" );
         ret = strdup( "GROUP" );
         break;
      case MPI_T_SCOPE_GROUP_EQ:
         //ret = strdup( "MPI_T_SCOPE_GROUP_EQ" );
         ret = strdup( "GROUP_EQ" );
         break;
      case MPI_T_SCOPE_ALL:
         //ret = strdup( "MPI_T_SCOPE_ALL" );
         ret = strdup( "ALL" );
         break;
      case MPI_T_SCOPE_ALL_EQ:
         //ret = strdup( "MPI_T_SCOPE_ALL_EQ" );
         ret = strdup( "ALL_EQ" );
         break;
      default:
         //ret = strdup( "MPI_T_SCOPE_ALL_UNDEF" );
         ret = strdup( "UNDEF" );
   }
   return ret;
}

static inline char*
MALP_mpi_t_translate_MPI_OBJ_BIND( const int bind )
{
   char* ret = NULL;

   switch( bind )
   {
      case MPI_T_BIND_NO_OBJECT:
         //ret = strdup( "MPI_T_BIND_NO_OBJECT" );
         ret = strdup( "NO_OBJECT" );
         break;
      case MPI_T_BIND_MPI_COMM:
         //ret = strdup( "MPI_T_BIND_MPI_COMM" );
         ret = strdup( "MPI_COMM" );
         break;
      case MPI_T_BIND_MPI_DATATYPE:
         //ret = strdup( "MPI_T_BIND_MPI_DATATYPE" );
         ret = strdup( "MPI_DATATYPE" );
         break;
      case MPI_T_BIND_MPI_ERRHANDLER:
         //ret = strdup( "MPI_T_BIND_MPI_ERRHANDLER" );
         ret = strdup( "MPI_ERRHANDLER" );
         break;
      case MPI_T_BIND_MPI_FILE:
         //ret = strdup( "MPI_T_BIND_MPI_FILE" );
         ret = strdup( "MPI_FILE" );
         break;
      case MPI_T_BIND_MPI_GROUP:
         //ret = strdup( "MPI_T_BIND_MPI_GROUP" );
         ret = strdup( "MPI_GROUP" );
         break;
      case MPI_T_BIND_MPI_OP:
         //ret = strdup( "MPI_T_BIND_MPI_OP" );
         ret = strdup( "MPI_OP" );
         break;
      case MPI_T_BIND_MPI_REQUEST:
         //ret = strdup( "MPI_T_BIND_MPI_REQUEST" );
         ret = strdup( "MPI_REQUEST" );
         break;
      case MPI_T_BIND_MPI_WIN:
         //ret = strdup( "MPI_T_BIND_MPI_WIN" );
         ret = strdup( "MPI_WIN" );
         break;
      case MPI_T_BIND_MPI_MESSAGE:
         //ret = strdup( "MPI_T_BIND_MPI_MESSAGE" );
         ret = strdup( "MPI_MESSAGE" );
         break;
      case MPI_T_BIND_MPI_INFO:
         //ret = strdup( "MPI_T_BIND_MPI_INFO" );
         ret = strdup( "MPI_INFO" );
         break;
      default:
         //ret = strdup( "MPI_T_BIND_MPI_UNDEF" );
         ret = strdup( "UNDEF" );
   }  
   return ret;
}

#endif /* HAVE_MPI_T */

#ifdef __cplusplus
}
#endif

#endif /* __MALP_MPI_T_EVENTS_H__ */
