/* ############################ MALP License ############################## */
/* # Fri Jan 18 14:00:00 CET 2013                                         # */
/* # Copyright or (C) or Copr. Commissariat a l'Energie Atomique          # */
/* #                                                                      # */
/* # This software is governed by the CeCILL-C license under French law   # */
/* # and abiding by the rules of distribution of free software.  You can  # */
/* # use, modify and/ or redistribute the software under the terms of     # */
/* # the CeCILL-C license as circulated by CEA, CNRS and INRIA at the     # */
/* # following URL http://www.cecill.info.                                # */
/* #                                                                      # */
/* # The fact that you are presently reading this means that you have     # */
/* # had knowledge of the CeCILL-C license and that you accept its        # */
/* # terms.                                                               # */
/* #                                                                      # */
/* # Authors:                                                             # */
/* #   - BESNARD Jean-Baptiste jbbesnard@paratools.fr                     # */
/* #                                                                      # */
/* ######################################################################## */
#ifndef OMPT_EVENTS_H
#define OMPT_EVENTS_H

#ifdef __cplusplus
extern "C"
{
#endif

#include "config.h"

/* Anticipate case where no OMPT */
#define T_OMPT_COUNT 0

#if HAVE_OMPT_H

#include "ompt.h"
#include "MALP_atomics.h"

#define MALP_OMPT_MAX_TASK_NESTED_LEVEL 32
#define MALP_OMPT_MAX_PARALLEL_NESTED_LEVEL 12

typedef enum MALP_ompt_scope_u
{
   MALP_OMPT_SCOPE_UNDEF 	= 0,
   MALP_OMPT_SCOPE_BEGIN 	= 1,
   MALP_OMPT_SCOPE_END 		= 2,
   MALP_OMPT_SCOPE_COUNT 	= 3
} MALP_ompt_scope_t;

typedef enum MALP_ompt_lock_type_u
{
   MALP_OMPT_LOCK_TYPE_UNDEF            = 0,
   MALP_OMPT_LOCK_TYPE        		= 1,
   MALP_OMPT_LOCK_TYPE_LOCK        	= 2,
   MALP_OMPT_LOCK_TYPE_NEST_LOCK        = 3,
   MALP_OMPT_LOCK_TYPE_CRITICAL         = 4,
   MALP_OMPT_LOCK_TYPE_ATOMIC           = 5,
   MALP_OMPT_LOCK_TYPE_ORDERED          = 6,
   MALP_OMPT_LOCK_TYPE_COUNT            = 7
} MALP_ompt_lock_type_t;

typedef enum MALP_ompt_generic_event_u
{
   MALP_OMPT_EVENT_UNDEF 		= 0,

   /* LOCK EVENT */
   MALP_OMPT_LOCK_EVENT_INIT		= 1,
   MALP_OMPT_LOCK_EVENT_NEST		= 2,
   MALP_OMPT_LOCK_EVENT_SET		= 3,
   MALP_OMPT_LOCK_EVENT_WAIT		= 4,
   MALP_OMPT_LOCK_EVENT_UNSET		= 5,
   MALP_OMPT_LOCK_EVENT_FINI  		= 6,

   /* PARALLEL EVENT */
   MALP_OMPT_PARALLEL_EVENT		= 7,
   MALP_OMPT_THREAD_EVENT		= 8,

   /* TASK EVENT */
   MALP_OMPT_TASK_EVENT_CREATE   	= 9,
   MALP_OMPT_TASK_EVENT_SCHEDULE 	= 10,

   MALP_OMPT_WORK_EVENT_LOOP 		= 11,
   MALP_OMPT_EVENT_MASTER 		= 12,

   MALP_OMPT_EVENT_COUNT                = 13
} MALP_ompt_generic_event_t;

#undef T_OMPT_COUNT
#define T_OMPT_COUNT MALP_OMPT_EVENT_COUNT

static char* OMPT_Event_names[T_OMPT_COUNT] = { "MALP_OMPT_EVENT_UNDEF", 
						"MALP_OMPT_LOCK_EVENT_INIT",
						"MALP_OMPT_LOCK_EVENT_NEST",
						"MALP_OMPT_LOCK_EVENT_SET",
						"MALP_OMPT_LOCK_EVENT_WAIT",
						"MALP_OMPT_LOCK_EVENT_UNSET",
						"MALP_OMPT_LOCK_EVENT_FINI",
                                                "MALP_OMPT_PARALLEL_EVENT",
						"MALP_OMPT_THREAD_EVENT",
						"MALP_OMPT_TASK_EVENT_CREATE",
						"MALP_OMPT_TASK_EVENT_SCHEDULE",
						"MALP_OMPT_WORK_EVENT_LOOP", 
						"MALP_OMPT_EVENT_MASTER" 
						};

/* OMPT runtime entry points */
typedef struct MALP_ompt_rt_entries_s 
{
   ompt_enumerate_states_t      lookup_states_fn;
   ompt_enumerate_mutex_impls_t lookup_mutex_impls_fn;
   ompt_set_callback_t          set_callback_fn;
   ompt_get_callback_t          get_callback_fn;
   ompt_get_thread_data_t       get_thread_data_fn;
   ompt_get_task_info_t         get_task_info_fn;
   ompt_get_parallel_info_t     get_parallel_info_fn;
} MALP_ompt_rt_entries_t;

/* Stacking work construct */
typedef struct MALP_ompt_work_timer_data_s
{
   int start_timer;
   struct MALP_ompt_work_timer_data_s* prev;
} MALP_ompt_work_timer_data_t;

typedef struct MALP_ompt_task_data_s
{
   int global_id;
   uint64_t start_time;
   MALP_ompt_work_timer_data_t* work_timer;
} MALP_ompt_task_data_t;


typedef struct MALP_ompt_parallel_data_s
{
   int global_id;
   uint64_t start_time; 
   MALP_ompt_work_timer_data_t* work_timer;
} MALP_ompt_parallel_data_t;

typedef struct MALP_ompt_thread_data_s
{  
   int global_id;
   uint64_t start_time; 
} MALP_ompt_thread_data_t;

typedef struct MALP_ompt_lock_data_s
{
} MALP_ompt_lock_data_t;

typedef struct MALP_ompt_lock_event_s
{
   MALP_ompt_lock_type_t type;  
   unsigned int hint;
   unsigned int impl;
   ompt_wait_id_t wait_id;
//   void *codeptr_ra;
} MALP_ompt_lock_event_t;

typedef struct MALP_ompt_call_context_s
{
   int thread_id;
   uint64_t nested_level;
   int task_ids[MALP_OMPT_MAX_TASK_NESTED_LEVEL];
   int parallel_ids[MALP_OMPT_MAX_PARALLEL_NESTED_LEVEL];
} MALP_ompt_context_call_t;

typedef struct MALP_ompt_lock_elt_s
{
   ompt_wait_id_t wait_id;
   MALP_ompt_generic_event_t event;
   struct MALP_ompt_lock_elt_s* prev;
} MALP_ompt_lock_elt_t;

typedef struct MALP_ompt_task_event_s
{
   
} MALP_ompt_task_event_t;

typedef struct MALP_ompt_parallel_event_s
{

} MALP_ompt_parallel_event_t;

typedef struct MALP_ompt_thread_event_s
{
  
} MALP_ompt_thread_event_t; 

typedef union MALP_ompt_specific_event_data_u
{
   MALP_ompt_task_event_t task;
   MALP_ompt_lock_event_t lock;
   MALP_ompt_thread_event_t thread;
   MALP_ompt_parallel_event_t parallel;
} MALP_ompt_specific_event_data_t;

typedef struct MALP_OMPT_event_s
{
   MALP_ompt_scope_t scope;
   MALP_ompt_context_call_t ctx;
   MALP_ompt_generic_event_t type;
   MALP_ompt_specific_event_data_t data;
}  MALP_OMPT_event_t;

#endif /* HAVE_OMPT_H */

#ifdef __cplusplus
}
#endif

#endif /* OMPT_EVENTS_H */
