#include "config.h"

#if HAVE_MPI_T
#include "mpi.h"

#include <pthread.h>
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <string.h>
#include <stdint.h>

#include "Timer.h"
#include "Event_Desc.h"
#include "Event_Pack.h"
#include "MPIT_Events.h"
#include "MALP_MPI_T_data.h"
#include "MALP_MPI_T_instr.h"
#include "Stream_MGR.h"

static uint64_t __malp_mpi_t_poll_freq = 1;
static pthread_t __malp_mpi_t_thread_data;
static volatile int __malp_mpi_t_thread_active;

void *__malp_mpi_t_polling_thread_start(void *data)
{
#ifdef MALP_MPI_T_STRING_SUPPORT
    char *__string_buffer;
#endif /* MALP_MPI_T_STRING_SUPPORT */
    int i, err, count, frequency;
    uint64_t __uint64_t_singleton;
    struct MALP_Trace_Event event;
    MALP_mpi_t_cvar_info_t *cvar_data;
    MALP_mpi_t_pvar_info_t *pvar_data;

    MPI_T_pvar_session session;
    MPI_T_cvar_handle *handle_cvar_list;
    MPI_T_pvar_handle *handle_pvar_list;

    /* Provided by getenv("MPI_T_POLLING_FREQ") see main.c */
    frequency = *(int *)data;

    const int __num_cvars = __malp_mpi_t_get_num_vars(MALP_MPI_T_TYPE_POLL_CVAR);
    const int __num_pvars = __malp_mpi_t_get_num_vars(MALP_MPI_T_TYPE_POLL_PVAR);

    MALP_mpi_t_var_info_t *__cvar_ids = __malp_mpi_t_get_var_ids(MALP_MPI_T_TYPE_POLL_CVAR);
    assert(!__num_cvars || (__num_cvars && __cvar_ids));

    MALP_mpi_t_var_info_t *__pvar_ids = __malp_mpi_t_get_var_ids(MALP_MPI_T_TYPE_POLL_PVAR);
    assert(!__num_pvars || (__num_pvars && __pvar_ids));

    handle_cvar_list = malloc(sizeof(MPI_T_cvar_handle) * __num_cvars);
    assert(handle_cvar_list);

    err = PMPI_T_pvar_session_create(&session);
    assert(err == MPI_SUCCESS);

    handle_pvar_list = malloc(sizeof(MPI_T_pvar_handle) * __num_pvars);
    assert(handle_pvar_list);

    int max_string_val = 1;

    /* Register all MPI_Handle */
    for (i = 0; i < __num_cvars; i++)
    {
        cvar_data = &(__cvar_ids[i].var.cvar);
        const int __var = cvar_data->absolute_id;

        err = PMPI_T_cvar_handle_alloc(__var, NULL, &(handle_cvar_list[i]), &(cvar_data->count));
        assert(err == MPI_SUCCESS);
        //assert( cvar_data->mpi_type == MPI_CHAR || cvar_data->count == 1 );
#ifdef MALP_MPI_T_STRING_SUPPORT
        max_string_val = (max_string_val < count) ? count : max_string_val;
#endif /* MALP_MPI_T_STRING_SUPPORT */
    }

    for (i = 0; i < __num_pvars; i++)
    {
        pvar_data = &(__pvar_ids[i].var.pvar);
        const int __var = pvar_data->absolute_id;

        MPI_Comm cw = MPI_COMM_WORLD;

        err = PMPI_T_pvar_handle_alloc(session, __var, cw, &(handle_pvar_list[i]), &pvar_data->count);
        assert(err == MPI_SUCCESS);
#ifdef MALP_MPI_T_STRING_SUPPORT
        max_string_val = (max_string_val < count) ? count : max_string_val;
#endif /* MALP_MPI_T_STRING_SUPPORT */
    }

    err = PMPI_T_pvar_start(session, MPI_T_PVAR_ALL_HANDLES);
    assert((err == MPI_SUCCESS) || (err == MPI_T_ERR_PVAR_NO_STARTSTOP));

#ifdef MALP_MPI_T_STRING_SUPPORT
    __string_buffer = (char *)malloc(sizeof(char) * (max_string_val + 1));
    assert(__string_buffer);
#endif /* MALP_MPI_T_STRING_SUPPORT */

    /* Read all MPI_Handle */
    do
    {
        for (i = 0; i < __num_cvars; i++)
        {
            if (__cvar_ids[i].var.cvar.mpi_type == MPI_CHAR)
            {
#ifdef MALP_MPI_T_STRING_SUPPORT
                err = PMPI_T_cvar_read(handle_cvar_list[i], (void *)__string_buffer);
                assert(err == MPI_SUCCESS);
                __string_buffer[__cvar_ids[i].var.cvar.count] = '\0';
#endif                    /* MALP_MPI_T_STRING_SUPPORT */
                continue; // SKIP MPI_CHAR value
            }

            err = PMPI_T_cvar_read(handle_cvar_list[i], (void *)&(__uint64_t_singleton));
            assert(err == MPI_SUCCESS);

            /* Create and push MPI_T poll cvar */
            event.event.mpit.logical_var_idx = i;
            event.event.mpit.type = MALP_MPI_T_TYPE_POLL_CVAR;
            event.event.mpit.mpi_type = MALP_mpi_t_type_encode(__cvar_ids[i].var.cvar.mpi_type);
            event.event.mpit.value.val_uint64_t = __uint64_t_singleton;

            MALP_Trace_Event_fill(&event, MALP_EVENT_MPI_T, Timer_tsc(), 1);
            Event_Packer_push(&event);

#ifdef MALP_MPI_T_STRING_SUPPORT
            if (__pvar_ids[i].var.pvar.mpi_type == MPI_CHAR)
                memset(__string_buffer, 0, sizeof(char) * (max_string_val + 1));
#endif /* MALP_MPI_T_STRING_SUPPORT */
        }

        for (i = 0; i < __num_pvars; i++)
        {
            if (__pvar_ids[i].var.pvar.mpi_type == MPI_CHAR)
            {
#ifdef MALP_MPI_T_STRING_SUPPORT
                err = PMPI_T_pvar_read(session, handle_pvar_list[i], (void *)__string_buffer);
                assert(err == MPI_SUCCESS);
                __string_buffer[__pvar_ids[i].var.cvar.count] = '\0';
#endif                    /* MALP_MPI_T_STRING_SUPPORT */
                continue; // SKIP MPI_CHAR value
            }

            if (__pvar_ids[i].var.pvar.count != 1)
            {
                continue;
            }

            err = PMPI_T_pvar_read(session, handle_pvar_list[i], (void *)&(__uint64_t_singleton));
            assert(err == MPI_SUCCESS);

            /* Create and push MPI_T poll pvar */
            event.event.mpit.type = MALP_MPI_T_TYPE_POLL_PVAR;
            event.event.mpit.logical_var_idx = i;
            event.event.mpit.value.val_uint64_t = __uint64_t_singleton;
            event.event.mpit.mpi_type = MALP_mpi_t_type_encode(__pvar_ids[i].var.pvar.mpi_type);

            MALP_Trace_Event_fill(&event, MALP_EVENT_MPI_T, Timer_tsc(), 1);
            Event_Packer_push(&event);

#ifdef MALP_MPI_T_STRING_SUPPORT
            if (__pvar_ids[i].var.pvar.mpi_type == MPI_CHAR)
                memset(__string_buffer, 0, sizeof(char) * (max_string_val + 1));
#endif /* MALP_MPI_T_STRING_SUPPORT */
        }
        if (!__malp_mpi_t_thread_active)
            break;
        /* Pause MPI_T_FREQ */
        usleep(frequency);

    } while (__malp_mpi_t_thread_active);

    err = PMPI_T_pvar_stop(session, MPI_T_PVAR_ALL_HANDLES);
    assert((err == MPI_SUCCESS) || (err == MPI_T_ERR_PVAR_NO_STARTSTOP));

    /* Release MPI handlers and session */
    for (i = 0; i < __num_cvars; i++)
    {
        err = PMPI_T_cvar_handle_free(&(handle_cvar_list[i]));
        assert(err == MPI_SUCCESS);
    }

    for (i = 0; i < __num_pvars; i++)
    {
        err = PMPI_T_pvar_handle_free(session, &(handle_pvar_list[i]));
        assert(err == MPI_SUCCESS);
    }

    err = PMPI_T_pvar_session_free(&session);

    return NULL;
}

void malp_mpi_t_start_polling_thread(int frequency)
{
    int err;
    __malp_mpi_t_thread_active = 1;
    int * f = malloc(sizeof(int));
    *f = frequency;
    err = pthread_create(&__malp_mpi_t_thread_data, NULL, __malp_mpi_t_polling_thread_start, (void *)f);
    assert(!err);
}

void malp_mpi_t_stop_polling_thread(void)
{
    int err;
    __malp_mpi_t_thread_active = 0;
    err = pthread_join(__malp_mpi_t_thread_data, NULL);
    assert(!err);
}

#endif /* HAVE_MPI_T */
