#ifndef __OMPT_INSTR_H__
#define __OMPT_INSTR_H__

#include "config.h"
#if HAVE_OMPT_H

#include "OMPT_Events.h"

static inline MALP_ompt_lock_type_t
MALP_ompt_translate_lock_type( ompt_mutex_kind_t kind )
{
   MALP_ompt_lock_type_t retval = 0;

   switch( kind ) 
   {
      case ompt_mutex :
          retval = MALP_OMPT_LOCK_TYPE;
          break;
      case ompt_mutex_lock :
          retval = MALP_OMPT_LOCK_TYPE_LOCK;
          break;
      case ompt_mutex_nest_lock :
          retval = MALP_OMPT_LOCK_TYPE_NEST_LOCK;
          break;
      case ompt_mutex_critical :
          retval = MALP_OMPT_LOCK_TYPE_CRITICAL;
          break;
      case ompt_mutex_atomic :
          retval = MALP_OMPT_LOCK_TYPE_ATOMIC;
          break;
      case ompt_mutex_ordered :
   	  retval = MALP_OMPT_LOCK_TYPE_ORDERED;
          break;
      default:
          retval = MALP_OMPT_LOCK_TYPE_UNDEF;
   } 

   return retval;
}

static void MALP_ompt_push_work_timer_data( ompt_data_t* task_data, int start_timer )
{
   MALP_ompt_task_data_t* __task_data; 
   MALP_ompt_work_timer_data_t *new_work_timer; 

   assert( task_data );
   __task_data = ( MALP_ompt_task_data_t* ) task_data;

   new_work_timer = ( MALP_ompt_work_timer_data_t* ) malloc( sizeof(MALP_ompt_work_timer_data_t ) );
   assert( new_work_timer );
   memset( new_work_timer, 0, sizeof(MALP_ompt_work_timer_data_t ) );

   new_work_timer->start_timer = start_timer;

   if( __task_data->work_timer ) 
       new_work_timer->prev = __task_data->work_timer;
     
   __task_data->work_timer = new_work_timer; 
}

static int MALP_ompt_pop_work_timer_data( ompt_data_t* task_data )
{
   MALP_ompt_task_data_t* __task_data; 
   MALP_ompt_work_timer_data_t *cur_work_timer; 
   
   assert( task_data );
   __task_data = ( MALP_ompt_task_data_t* ) task_data;

   assert( __task_data->work_timer );
   cur_work_timer = __task_data->work_timer;
   __task_data->work_timer = cur_work_timer->prev;

   const int start_timer = cur_work_timer->start_timer;
   free( cur_work_timer );
   return start_timer;
}

#endif /* HAVE_OMPT_H */

#endif /* __OMPT_INSTR_H__ */
