/* ############################ MALP License ############################## */
/* # Fri Jan 18 14:00:00 CET 2013                                         # */
/* # Copyright or (C) or Copr. Commissariat a l'Energie Atomique          # */
/* #                                                                      # */
/* # This software is governed by the CeCILL-C license under French law   # */
/* # and abiding by the rules of distribution of free software.  You can  # */
/* # use, modify and/ or redistribute the software under the terms of     # */
/* # the CeCILL-C license as circulated by CEA, CNRS and INRIA at the     # */
/* # following URL http://www.cecill.info.                                # */
/* #                                                                      # */
/* # The fact that you are presently reading this means that you have     # */
/* # had knowledge of the CeCILL-C license and that you accept its        # */
/* # terms.                                                               # */
/* #                                                                      # */
/* # Authors:                                                             # */
/* #   - BESNARD Jean-Baptiste jean-baptiste.besnard@cea.fr               # */
/* #                                                                      # */
/* ######################################################################## */
#ifndef POSIX_WRAPP_H
#define POSIX_WRAPP_H

#define _GNU_SOURCE
#include <dlfcn.h>
//#include <dlsym.h>

#include "Wrapped_Symbols.h"

/********** Pointer to func **********/
/**
 * @brief Union defining a function pointer
 */
union function_table_elem {
	void *p;            /**< @brief the void* (to be get by dlsym) */
	void (*p_func) ();  /**< @brief the function pointer (to be used as a function) */
};

/********** Resolution func **********/
/**
 * @brief The table containing every wrapped symbols
 */
extern union function_table_elem Tracelib_symbol_table[T_POSIX_COUNT];

/**
 * @brief Gets a symbol with its ID
 * @param id the ID of wanted symbol
 * @return the corresponding function pointed
 */
static inline void * Tracelib_resolve_symbol (int id)
{
	return Tracelib_symbol_table[id].p;
}

/********** useful macros **********/
/**
 * @brief Retrieves the id of a symbol by its name
 */

#define to_string(a) _to_string(a)
#define _to_string(a) #a

#if DO_POSIX_INSTRUM
/**
 * @brief Does the real call of a symbol
 */
#define REAL(a) (( FOOTPRINT_ ## a ) Tracelib_resolve_symbol( T_POSIX_ENUM( a ) ))
#define pREAL(a) ( Tracelib_resolve_symbol( T_POSIX_ENUM(a) ) )

void ___TEST_LOAD_SYMBOL( Wrapped_symbol symb, const char * const symbol );

#define _TEST_LOAD_SYMBOL(a) do{ \
		___TEST_LOAD_SYMBOL( T_POSIX_ENUM( a ), to_string(a) );\
	} while (0)

/**
 * @brief loads the real (not the wrapping function) symbols function pointed and stores it into the table (Tracelib_symbol_table)
 */
#define TEST_LOAD_SYMBOL(a) _TEST_LOAD_SYMBOL(a);

#else /*DO_POSIX_INSTRUM */

/* Do not intercept POSIX */
#define REAL(a) a
#define pREAL(a) a
#define TEST_LOAD_SYMBOL(a) 

#endif /* DO_POSIX_INSTRUM */

void Posix_Wrapp_init();
void Wrapp_enable();
void Wrapp_disable();


#endif /* POSIX_WRAPP_H */
