/* ############################ MALP License ############################## */
/* # Fri Jan 18 14:00:00 CET 2013                                         # */
/* # Copyright or (C) or Copr. Commissariat a l'Energie Atomique          # */
/* #                                                                      # */
/* # This software is governed by the CeCILL-C license under French law   # */
/* # and abiding by the rules of distribution of free software.  You can  # */
/* # use, modify and/ or redistribute the software under the terms of     # */
/* # the CeCILL-C license as circulated by CEA, CNRS and INRIA at the     # */
/* # following URL http://www.cecill.info.                                # */
/* #                                                                      # */
/* # The fact that you are presently reading this means that you have     # */
/* # had knowledge of the CeCILL-C license and that you accept its        # */
/* # terms.                                                               # */
/* #                                                                      # */
/* # Authors:                                                             # */
/* #   - BESNARD Jean-Baptiste jean-baptiste.besnard@cea.fr               # */
/* #                                                                      # */
/* ######################################################################## */
var cache = require('nsc');
var catalog = module.require("malp_catalog").catalog;
var process = require('child_process');
var fs = require("fs");
	
/*
 * 
 *  METADATA
 * 
 * 
 */

function metadata()
{
	
	metadata.prototype.getdesc = function( tid )
	{
		var streams = catalog.getStreams();
		return streams[tid];
	}
	
	metadata.prototype.stream_duration = function( tid )
	{
		var s = this.getdesc( tid );
		
		if( s )
		{
			return (s.end_time - s.start_time);
		}
		
		return 0;
	}
	
	metadata.prototype.partition_duration = function()
	{
		var p = catalog.getDesc();
		
		if( p != undefined )
		{
			return p.duration;
		}
		
		return 0;
	}
	
	metadata.prototype.partition_cputime = function()
	{
		var p = catalog.getDesc();
		
		if( p )
			return p.duration * p.partition_size;

		return 0;
	}

}

var metadata_instance = new metadata();


/*
 * 
 *  UNIT converter
 * 
 * 
 */


function Converter()
{
	Converter.prototype.time_unit = function ( time )
	{
		var r = time;
		var ret = "";

		if( 1 <= r )
		{
			var days = Math.floor( r / (3600 * 24 ) );
			r = r - days*(3600 * 24 );
			var hour = Math.floor( r / (3600) );
			r = r - hour*(3600);
			var m = Math.floor( r / (60) );
			r = r - m*(60);
			var s = r;

			if( days )
				ret += days + " days ";

			if( hour )
				ret += hour + " h ";

			if( m )
				ret += m + " m ";

			if( s )
				ret += s.toFixed(3) + " s";
		}
		else
		{
			if( r < 1e-6 )
				ret = (r * 1e9).toFixed(3) + " ns";
			else
			if( r < 1e-3 )
				ret = (r * 1e6).toFixed(3) + " µs";
			else
				ret = (r * 1e3).toFixed(3) + " ms";
		}

		return ret
	}

	
	Converter.prototype.size_unit = function( size )
	{
		var ret = "";
		var tb = 1024 * 1024 * 1024 * 1024;
		var gb = 1024 * 1024 * 1024;
		var mb = 1024 * 1024;
		var kb = 1024;


		if( tb < size )
			ret = (size/tb).toFixed(3) + " TB";
		else if( gb < size )
			ret = (size/gb).toFixed(3) + " GB";
		else if( mb < size )
			ret = (size/mb).toFixed(3) + " MB";
		else if( kb < size )
			ret = (size/kb).toFixed(3) + " KB";
		else
			ret = size + " B";

		return ret;
	}
}

var converter_instance = new Converter();



/*
 * 
 *  Profile Renderer
 * 
 * 
 */

function profile( res, coll )
{
	this.res = res;
	this.profile_collector = coll;
	this.container_id = 0;

	profile.prototype.render_pie_chart = function( prof, key )
	{
		this.container_id++;
		
		var i;
		
		var data = [];
		
		for( i = 0 ; i < prof.length ; i++ )
		{
			data.push( [ prof[i].name , prof[i][ key ] ] );
		}
		
		
		this.res.write("<script type=\"text/javascript\">");
		this.res.write("$(document).ready(function(){\n");
		
		this.res.write("var plot1 = $.jqplot('chart_" + this.container_id + "', [" + JSON.stringify(data) + "], {\n");
		this.res.write("		gridPadding: {top:0, bottom:38, left:0, right:0},\n");
		this.res.write("		seriesDefaults:{\n");
		this.res.write("			renderer:$.jqplot.PieRenderer,\n");
		this.res.write("			rendererOptions: { padding: 8, showDataLabels: true }\n");
		this.res.write("		},\n");
		this.res.write("		legend:{\n");
		this.res.write("			show:false,\n");
		//this.res.write("			placement: 'outside',\n");
		//this.res.write("			rendererOptions: {\n");
		//this.res.write("				numberRows: 8\n");
		//this.res.write("			},\n");
		//this.res.write("			location:'s',\n");
		//this.res.write("			marginTop: '15px',\n");
		//this.res.write("			border:'none'\n");
		this.res.write("		},      \n");
		this.res.write("		highlighter: {\n");
		this.res.write("		  show: true,\n");
		this.res.write("		  formatString:'%s', \n");
		this.res.write("		  tooltipLocation:'sw', \n");
		this.res.write("		  useAxesFormatters:false\n");
		this.res.write("		}\n");
		this.res.write("	});\n");
		this.res.write("});\n");
		
		this.res.write("</script>\n");	
		
		this.res.write("<div id=\"chart_" + this.container_id + "\" style=\"height:400px;width:300px;border-width: 0px;\"></div>\n");
	}
	
	profile.prototype.pie_chart_array = function( prof )
	{
		this.res.write("<div class='inlinegraph'>\n");
		this.res.write("<h2>Time</h2>\n");
		this.render_pie_chart( prof, "time");
		this.res.write("</div>\n");
		
		this.res.write("<div class='inlinegraph'>\n");
		this.res.write("<h2>Calls</h2>\n");
		this.render_pie_chart( prof, "hits");
		this.res.write("</div>\n");
		
		this.res.write("<div class='inlinegraph'>\n");
		this.res.write("<h2>Size</h2>\n");
		this.render_pie_chart( prof, "size");
		this.res.write("</div>\n");

	}

	profile.prototype._getTop = function( num )
	{
		if( !this.profile_collector )
			return;
		
		var prof = this.profile_collector.profile;
		
		prof = prof.sort( function( a , b )
		{
			if( a.time < b.time )
				return -1;
			if( a.time == b.time )
				return 0;
			if( b.time < a.time )
				return 1;
			
		});
		
		var ret = [];
		
		var i;
		
		for( i = (prof.length - 1); (prof.length - 1 - num ) < i   ; i-- )
		{
			if( ! prof[i] )
				continue;
			
			ret.push( prof[i].name  );
		}
		
		return ret;
	}	



	profile.prototype.render = function( prof, duration )
	{
		if( prof.length == 0 )
			return;

		this.res.write("<table class=\"malptable\">");
		this.res.write("<tr>");
		this.res.write("<th>Function</th>");
		this.res.write("<th>Calls</th>");
		this.res.write("<th>Time</th>");
		this.res.write("<th>%</th>");
		this.res.write("<th>Size</th>");
		this.res.write("</tr>");

		var i = 0;
		
		prof = prof.sort( function( a , b )
		{
			if( a.time < b.time )
				return -1;
			if( a.time == b.time )
				return 0;
			if( b.time < a.time )
				return 1;
			
		});

		for( i = prof.length - 1 ; 0 <= i ; i-- )
		{
			var current_func = prof[i];
			
			if( current_func["name"] == "Other Calls" )
				this.res.write("<tr class='highlight'>");
			else
				this.res.write("<tr>");
			
			this.res.write("<td>" + current_func["name"] + "</td>");
			this.res.write("<td>" + current_func["hits"] + "</td>");

			this.res.write("<td>" + converter_instance.time_unit(current_func["time"]) + "</td>");
			this.res.write("<td>" + ((current_func["time"]/duration)*100).toFixed(3) + "</td>");

			this.res.write("<td>" + converter_instance.size_unit( current_func["size"] ) + "</td>");


			this.res.write("</tr>");	

		}

		this.res.write("</table>");	

	}	
	
	profile.prototype.render_global = function ()
	{
		var d = metadata_instance.partition_cputime();
		this.render( this.profile_collector.profile, d );
	}
	
	profile.prototype.render_per_stream = function ()
	{
		var stream_list = this.profile_collector.streams;
	
		var ranks = this.profile_collector.streams.sort( function (a, b)
			    {
				var a_rank = metadata_instance.getdesc(a.tid).mpi_rank;
				var b_rank = metadata_instance.getdesc(b.tid).mpi_rank;

				if( a_rank < b_rank )
					return -1;
				if(  a_rank == b_rank )
					return 0;
				if( b_rank < a_rank )
					return 1; 
			    });

		var i;
		
		for( i = 0 ; i <  ranks.length; i++ )
		{
			var current_prof = ranks[i];
			var desc = metadata_instance.getdesc(current_prof.tid)
			
			if( desc.type == "process" )
			{
				this.res.write("<h2>Rank " + desc.mpi_rank + "</h2>");
				this.render( current_prof.profile,  desc.end_time - desc.start_time );
			}
		}
	}
	
}

	
function getTop( num, type )
{
	var current_prof = catalog.getProfile();

	if( !current_prof )
	{
		return [];
	}
	
	var collector = current_prof[ type + "_Profile"];
	
	if( !collector ) 
		return [];
	
	var p  = new profile( undefined, collector );
	
	return p._getTop(num);
}



/*
 * 
 *  Time Matrix Renderer
 * 
 * 
 */
 
function timeMatrix( res, map, width, heigth, scale, gauss, logscale )
{
	var TM = {};
	
	if( width != undefined ) 
		TM.width = width;
	
	if( heigth != undefined ) 
		TM.heigth = heigth;
	
	if( scale != undefined ) 
		TM.scale = scale;

	if( gauss != undefined ) 
		TM.gauss = parseFloat(gauss);	
	
	if( logscale != undefined )
		TM.logscale = (logscale=="1")?true:false;
	
	
	if( map == undefined )
	{
		res.end("Error no map provided");
		return;
	}

	TM.map = map;

	/* Set JPEG header */
	res.writeHead(200, {'Content-Type': 'jpeg'});

	/* First check if we are not on the patched node */
	
	if( fs.density != undefined )
	{
		data = fs.density( JSON.stringify(TM) );
		
		res.end( data );
		/* We are done here */
		return;
	}
	
	/* Otherwise render it via a pipe */
	var renderer = undefined;
	
	try
	{
		renderer = process.spawn("malp_density", []);
	}
	catch(e)
	{
		res.end("Failed to launch malp_density process");
		return;
	}
	
	
	renderer.stdout.on('data', function (data) {
		res.write( data );
	});
	
	renderer.stderr.on('data', function (data) {
		console.log( data + "");
	});
	
	renderer.on('close', function (code) {
		res.end();
		return;
	});

	
	renderer.stdin.write(JSON.stringify(TM));
	renderer.stdin.end();
	
}

function timeMatrixScale( res, scale, width, height, logscale )
{
	/* Fill in a linear scale descriptor */
	var TM = {};
	TM.map = [];
	TM.map[0] = [];
	
	/* Retrieve params */
	if( width != undefined )
		TM.width = parseInt(width);	
	
	if( height != undefined )
		TM.height =  parseInt(height);

	if( scale != undefined )
		TM.scale = scale;

	if( logscale != undefined )
		TM.logscale = (logscale=="1")?true:false;

	var i;
	
	for( i = 0 ; i < 500 ; i++ )
		TM["map"][0].push( i );

	/* Set JPEG header */
	res.writeHead(200, {'Content-Type': 'jpeg'});

	/* First check if we are not on the patched node */
	
	if( fs.density != undefined )
	{
		data = fs.density( JSON.stringify(TM) );
		res.end( data );
		/* We are done here */
		return;
	}
	
	/* Otherwise render it via a pipe */
	var renderer = undefined;
	
	try
	{
		renderer = process.spawn("malp_density", []);
	}
	catch(e)
	{
		res.end("Failed to launch malp_density process");
		return;
	}
	
	res.writeHead(200, {'Content-Type': 'image/jpg'});
	
	renderer.stdout.on('data', function (data) {
		res.write( data );
	});
	
	renderer.on('close', function (code) {
		res.end();
		return;
	});

	
	renderer.stdin.write(JSON.stringify(TM));
	renderer.stdin.end();
	
} 


/*
 * 
 *  Exports
 * 
 * 
 */

module.exports.profile = profile;
module.exports.getTop = getTop;
module.exports.metadata = metadata_instance;
module.exports.converter = converter_instance;
module.exports.timeMatrix = timeMatrix;
module.exports.timeMatrixScale = timeMatrixScale;


