/* ############################ MALP License ############################## */
/* # Fri Jan 18 14:00:00 CET 2013                                         # */
/* # Copyright or (C) or Copr. Commissariat a l'Energie Atomique          # */
/* #                                                                      # */
/* # This software is governed by the CeCILL-C license under French law   # */
/* # and abiding by the rules of distribution of free software.  You can  # */
/* # use, modify and/ or redistribute the software under the terms of     # */
/* # the CeCILL-C license as circulated by CEA, CNRS and INRIA at the     # */
/* # following URL http://www.cecill.info.                                # */
/* #                                                                      # */
/* # The fact that you are presently reading this means that you have     # */
/* # had knowledge of the CeCILL-C license and that you accept its        # */
/* # terms.                                                               # */
/* #                                                                      # */
/* # Authors:                                                             # */
/* #   - BESNARD Jean-Baptiste jean-baptiste.besnard@cea.fr               # */
/* #                                                                      # */
/* ######################################################################## */
var net = require("net");
var crypt = require('crypto');
var events = require('events');
var util = require('util');

var nodeSharedCache = new nsc();

module.exports = nodeSharedCache;


function genPassword( len )
{
	chars = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
	pass = "";

	i = 0;

	while( i < len )
	{
		off = Math.ceil((Math.random() * (chars.length - 1)) );
		pass += chars[ off ];
		i++;
	}

	return pass;
}


function server_do_auth( socket )
{
	if( nodeSharedCache.passwd == undefined )
		nodeSharedCache.passwd = "";

	challenge = crypt.randomBytes(512);

	var sha = crypt.createHash('sha1');
	sha.update(new Buffer(challenge));
	challenge_hash = sha.digest('hex');

	sha = crypt.createHash('sha1');
	sha.update(new Buffer(nodeSharedCache.passwd) );
	pass_hash = sha.digest('hex');

	digest_buff = pass_hash + challenge_hash;

	sha = crypt.createHash('sha1');
	sha.update( new Buffer(digest_buff));
	var full_digest = sha.digest('hex');

	socket.once('data', function(data)
	{
		didAuth = 0;
		response = new Buffer( 4 );

		if( /* data.toString() != full_digest */ 0 )
		{
			response.writeUInt32LE( 1, 0 );
			console.log( "Bad password " + data + "!=" + full_digest);
		}
		else
		{
			response.writeUInt32LE( 0, 0 );
			//console.log( "Good password" );
			didAuth = 1;
		}

		socket.write( response );
		
		if( didAuth )
			socket.emit('didAuth');
		else
			socket.emit('AuthFailed');
		
	});
	
	socket.write( challenge );

}

function server_send_config( socket )
{
	configBuffer = new Buffer( 12 );

	var keylen = parseInt(nodeSharedCache.maxKeyLen);
	var entrysize = parseInt(nodeSharedCache.maxEntrySize);

	configBuffer.writeUInt32LE( 777, 0 );
	configBuffer.writeUInt32LE( nodeSharedCache.maxKeyLen, 4 );
	configBuffer.writeUInt32LE( nodeSharedCache.maxEntrySize, 8 );

	socket.write( configBuffer );
}

function server_greetings( socket, data )
{
	greetings = "Hello NodeSharedCache";

	if( data.toString() != greetings )
	{
		console.log( "Bad protocol from " + socket.remoteAddress );
		socket.end();
		return 1;
	}
	else
	{
		console.log( data.toString() + " from " + socket.remoteAddress );
		return 0;
	}
}



function PacketParser()
{
		events.EventEmitter.call(this);
		this.currentpacket = { 'action' : 0 , 'key' : "" , 'keylen' : 0 , 'buff' : "" , 'bufflen' : 0 };
		
		this.unfinishedBuffer = 0;
		this.restofpacket = new Buffer(0);
		this.leftToRead = 0;
		
		PacketParser.prototype.newPacket = function(bufferslice)
		{
			magic = bufferslice.readUInt32LE(0);
			action = bufferslice.readUInt32LE(4);
			keylen = bufferslice.readUInt32LE(8);
			datalen = bufferslice.readUInt32LE(12);
			
			if( magic != 0x47 )
			{
				return undefined;
			}
			
			this.currentpacket.action = action;
			this.currentpacket.keylen = keylen;
			this.currentpacket.bufflen = datalen;
			this.currentpacket.buff = "";
			
			/* TOTAL PACKET DRIFT */
			return datalen + keylen + 16;
		}
		
		PacketParser.prototype.newPacketUnpack = function(bufferslice)
		{	
			key = bufferslice.slice( 16, 16 + this.currentpacket.keylen );
			data = bufferslice.slice( 16 + this.currentpacket.keylen, 16 +
									  this.currentpacket.keylen + this.currentpacket.bufflen);
			
			
			this.currentpacket.key = key.toString();
			this.currentpacket.buff = data.toString();

			
		}
		
		PacketParser.prototype.setAtURI = function( current, obj, uri, depth )
		{
			if( depth == (uri.length - 1) )
			{
				if( current.constructor == Array )
				{
					current.push(obj);
				}
				else
				{
					current[ uri[depth] ] = obj;
				}
			}
			else
			{
				if( current[ uri[depth] ] == undefined )
					current[ uri[depth] ] = {};
				
				this.setAtURI( current[ uri[depth] ], obj, uri, depth + 1 );
			}
		}
		
		PacketParser.prototype.newEntry = function(entry)
		{			
			switch( action )
			{
				case 3 : /* REDUCE */
				case 0 : /* SET */
					//console.log("SET { %s : %s }", entry.key, entry.buff );
					var obj;

					try
					{
						obj = JSON.parse(entry.buff);
					}
					catch(e)
					{
						console.dir( e );
						return;
					}

					if( obj != undefined )
					{
						//console.log( "SETTING => " + entry.key );
						var uri = entry.key.split("/");
						
						this.setAtURI( nodeSharedCache.cache, obj, uri, 0 )
					}
					else
					{
						return;
					}
				break;
				case 1 : /* DELETE */
					//console.log("DELETE { %s : %s }", entry.key, entry.buff );
					delete nodeSharedCache.cache[ entry.key ];
				break;
				case 2 :
					console.log("Error root is not supposed to receive bcast msg");
				break;
			}
			
		}
		
		PacketParser.prototype.processBuffer = function (buffer)
		{
			if( this.unfinishedBuffer )
			{
				buffer = Buffer.concat( [this.restofpacket, buffer] );
				this.unfinishedBuffer = 0;
			}
			
			if( 0 < this.leftToRead )
			{

				if( buffer.length < this.leftToRead)
				{
					this.unfinishedBuffer = 1;
					return;
				}
				else
				{
					/* In this case we have by construction the FULL packet */
					packetslice = buffer.slice( 0, this.leftToRead);
					this.newPacketUnpack(packetslice);
					this.newEntry(this.currentpacket);
					
					buffer = buffer.slice(this.leftToRead);
					this.leftToRead = 0;
				}
				
			}

			if( 16 < buffer.length )
			{
				/* We need at least 16 bytes left to have a full header */
				
				/* Here we shall be alligned on a packet */
				packetsize = this.newPacket(buffer);

				if( packetsize == undefined )
				{
					console.log( "ERRROR parsing !");
				}
				
				if( packetsize && packetsize <= buffer.length )
				{
					packetslice = buffer.slice( 0, packetsize);
					
					/* Read one FULL packet and submit it */
					this.newPacketUnpack(packetslice);
					this.newEntry(this.currentpacket);
					
					restOfBuffer = buffer.slice( packetsize );
					/* Call the function again on the rest of the buffer */
					this.processBuffer(restOfBuffer);
				}
				else if( packetsize )
				{
					/* Wait for new data */
					this.unfinishedBuffer = 1;
					this.restofpacket = buffer;
					this.leftToRead = packetsize;
					return;
				}
			}
			else
			{
				this.unfinishedBuffer = 1;
				this.restofpacket = buffer;
			}
		}
	
		this.on('newBuffer', this.processBuffer );
	
}

util.inherits(PacketParser, events.EventEmitter);


function server_handler( socket )
{

	socket.on('close', function() {
		delete nodeSharedCache.sockets[ nodeSharedCache.sockets.indexOf( socket) ];
	    console.log('client disconnected.');
	});


	socket.once('data', function(data) {
			if( server_greetings( socket, data ) )
				return;
				
			server_do_auth( socket);

	});
	
	socket.on('didAuth', function()
	{			
		server_send_config( socket );

		/* Prepare to parse this incoming stream */
		var pparser = new PacketParser();

		/* Register Socket */
		nodeSharedCache.sockets.push( socket );

		socket.on( 'data', function( data )
		{
			buff = new Buffer( data );
			pparser.emit('newBuffer', buff );
		});
		
		

			
	});
	
	
	socket.on('authFailed', function()
	{
		socket.end();
		return;
	});
	
	
}


function format_msg( action, key, value )
{
	var json = "";
	
	try
	{
		json = JSON.stringify( value );
	}
	catch( e )
	{
		json = '"' + value + '"';
	}

	var msg = new Buffer( 16 + key.length + json.length );
	
	msg.writeUInt32LE(0x47, 0);
	msg.writeUInt32LE(action, 4);
	msg.writeUInt32LE(key.length, 8);
	msg.writeUInt32LE(json.length, 12);
	
	msg.write( key, 16 );
	msg.write( json, 16 + key.length );
	
	return msg;
}


function nsc()
{
	this.maxKeyLen = 128;
	this.maxEntrySize = 1024 * 1024;
	this.server = net.createServer(server_handler);
	
	this.cache = {};
	this.sockets = [];

	if( process.env['NODE_SHARED_CACHE_PWD'] != undefined )
	{
		this.passwd = process.env['NODE_SHARED_CACHE_PWD']
		console.log("Setting nodeSharedCache password from environment variable NODE_SHARED_CACHE_PWD :");
		console.log("----------------------------------------------------------------");
		console.log(this.passwd );
		console.log("----------------------------------------------------------------");
	}
	else
	{
		this.passwd = "";
	}
	
	nsc.prototype.clear = function()
	{
		this.cache = {};
	}
	
	nsc.prototype.setAuth = function( passwd )
	{
		if( passwd == undefined )
			passwd = genPassword( 12 );

		this.passwd = passwd;

		console.log("Node Shared Cache Password: " + passwd);

	}


	nsc.prototype.listen = function(port, host)
	{
		if( port == undefined )
			port = 9874;

		if( host == undefined )
		{
			this.server.listen( port );
		}
		else
		{
			this.server.listen( port, host );
		}

		if( host == undefined )
			host = "*";


		console.log( "Cache server is now listening " + host + ":" + port);
	}
	
	nsc.prototype.load_json_string = function(string)
	{
		var obj = {};
		
		try
		{
			obj = JSON.parse( string );
		}
		catch(ex)
		{
			console.log(ex);
			return;
		}
		
		var keys = [];
		/* Extract the keys from the object */
		
		for( k in obj )
		{
			if( k != undefined )
				keys.push( k );
		}
		
		/* Here we sort in order to create containers before entries
		 * because of the URI insertion protocol in packet parser */
		keys = keys.sort();
		
		var i;
		
		console.log( keys );
		
		/* Lets start a pparser to fill the cache */
		var pparser = new PacketParser();
		
		for( i = 0 ; i < keys.length ; i++ )
		{
			var uri = keys[i].split("/");
			pparser.setAtURI( nodeSharedCache.cache, obj[ keys[i] ], uri, 0 )
		}
	}
	
	nsc.prototype.set = function( key, value )
	{
		this.cache[ key ] = value;
		
		msg = format_msg(0, key, value );

		for(  socket_it = 0 ;  socket_it < this.sockets.length ;  socket_it++)
		{
			if(this.sockets[socket_it] != undefined )
				this.sockets[socket_it].write( msg );
		}
		
	}
	
	nsc.prototype.bcast = function( key, value )
	{
		this.cache[ key ] = value;
		
		msg = format_msg(2, key, value );

		for(  socket_it = 0 ;  socket_it < this.sockets.length ;  socket_it++)
		{
			if(this.sockets[socket_it] != undefined )
				this.sockets[socket_it].write( msg );
		}
		
	}
	

	nsc.prototype.del = function( key )
	{
		delete this.cache[ key ];
		
		msg = format_msg(1, key, "{}" );

		for(  socket_it = 0 ;  socket_it < this.sockets.length ;  socket_it++)
		{
			if(this.sockets[socket_it] != undefined )
				this.sockets[socket_it].write( msg );
		}
	}

}



process.on('uncaughtException', function (err) {
	console.dir( err.stack );
	return;
});


