/* ############################ MALP License ############################## */
/* # Fri Jan 18 14:00:00 CET 2013                                         # */
/* # Copyright or (C) or Copr. Commissariat a l'Energie Atomique          # */
/* #                                                                      # */
/* # This software is governed by the CeCILL-C license under French law   # */
/* # and abiding by the rules of distribution of free software.  You can  # */
/* # use, modify and/ or redistribute the software under the terms of     # */
/* # the CeCILL-C license as circulated by CEA, CNRS and INRIA at the     # */
/* # following URL http://www.cecill.info.                                # */
/* #                                                                      # */
/* # The fact that you are presently reading this means that you have     # */
/* # had knowledge of the CeCILL-C license and that you accept its        # */
/* # terms.                                                               # */
/* #                                                                      # */
/* # Authors:                                                             # */
/* #   - BESNARD Jean-Baptiste jean-baptiste.besnard@cea.fr               # */
/* #                                                                      # */
/* ######################################################################## */
var pageHandler = require('./pageHandler');

//Command line colors (https://github.com/medikoo/cli-color)
var clc = require('cli-color');

module.exports = Page;

function Page( uri, parentUri, title, alt, ctx, pageHandler )
{
	this.pageHandler = pageHandler;

	this.uri = uri;
	this.parentUri = parentUri;

	this.parentPage = undefined;

	if( title == undefined )
	{
		throw Error("A page must have a title");
	}
	else
	{
		this.title = title;
	}	

	this.alt = alt;
	this.ctx = ctx;

	this.views = {};
	this.childrenPages = [];

	this.setPageHandler( pageHandler );
	this.setParent( pageHandler );
	this.setRender( pageHandler );
	
}

Page.prototype.setPageHandler = function( pageHandler )
{
	this.pageHandler = pageHandler;
}


Page.prototype.setParent = function( pageHandler )
{
	//Do we have a parent ?
	if( this.parentUri != "" && (pageHandler != undefined) )
	{

		this.parentPage = pageHandler.pages[ this.parentUri ];

		if( this.parentPage == undefined )
		{
			throw Error("Parent page "+ this.parentUri + " of "+ this.title + " does not exist");
		}

		this.parentPage.childrenPages.push( this );
	}
}

Page.prototype.setRender = function( pageHandler )
{
	if( pageHandler != undefined )
	{
		if( this.head == undefined )
			this.head = pageHandler.head;

		if( this.header == undefined )
			this.header = pageHandler.header;

		if( this.footer == undefined )
			this.footer = pageHandler.footer;

		if( this.navglobal == undefined )
			this.navglobal = pageHandler.navglobal;

		if( this.navlocal == undefined )
			this.navlocal = pageHandler.navlocal;

		if( this.body == undefined )
			this.body = pageHandler.body;
	}
}


Page.prototype.getUri = function()
{
	if( this.uri != "/" )
		return this.uri + "/";
	else
		return "/";
}


function pageView( callback, is_raw )
{
	this.callback = callback;
	this.is_raw = is_raw;
	
}

Page.prototype.addView = function( name, callback, is_raw )
{
	if( is_raw == undefined )
		is_raw = 0;
	
	
	if( this.views[ name ] == undefined )
	{
		//console.log("\t"+ clc.bold.blue("registering view: " + name) );
		this.views[ name ] = new pageView( callback, is_raw );
	}
	else
	{
		throw Error("View " + name + " is already defined in page " + this.uri );
	}
}
