
function ColorScale( dataset )
{
	/* Color scales */
	this.colscale = d3.scale.linear();

	this.colscale.domain([d3.min(dataset), d3.max(dataset)])
		  .range([0, 255]);
	
	this.getRGB = function(d)
	{
		return "rgb("+ Math.round(this.colscale(d)) +","+ 0 +", " + Math.round(255-this.colscale(d)) + ")"
	}
	
}



function MalpGraph( cont, dataset, width , height )
{
	/* Width computation */
	var w = 800;
	var h = 100;
	
	if( width )
		w = width;
	
	if( height )
		h = height;

	/* SVG creation */
	var render = d3.select(cont).append("svg");

	render.attr("width", w )
		   .attr("height", h );
		   
	/* Scales setup */
	var yscale = d3.scale.sqrt();

	yscale.domain([d3.min(dataset), d3.max(dataset)])
		  .range([0, h-20]);
		  
	var xscale = d3.scale.linear();
	var padding = 50;

	xscale.domain([0, dataset.length])
		  .range([padding, w - padding])
		  .nice();

	/* Color scale */
	var color = new ColorScale( dataset );

	/* Actual rendering */
	render.selectAll("rect")
		   .data(dataset)
		   .enter()
		   .append("rect")
		   .attr("x", function(d,i){  return xscale( i ); })
		   .attr("y", function(d) {
				return (h - 20)-yscale(d);
			})
		   .attr("width", function()
		   {
					return w / dataset.length;
		   })
		   .attr("height", function(d){ return yscale(d); })
		   .attr("fill", function(d){  return color.getRGB(d);});

	var xAxis = d3.svg.axis()
				   .scale(xscale)
				   .orient("bottom");

	render.append("g")
			.attr("class", "axis")
			.attr("transform", "translate(0," + (h-20) + ")")
			.call(xAxis);

}
