var catalog = module.require("malp_catalog").catalog;
var meta = require("malp_rendering").metadata;

<page uri="/interacttopo" parent="/" title="Interactive Topology" desc="Interactive network topologies for MALP" />


<body>
<script src='/scripts/jquery.js'></script>
<script src='/scripts/jquery-ui.min.js'></script>
<script type="text/javascript" src="/scripts/d3.js"></script>
<link rel="stylesheet" type="text/css" href="/scripts/jquery-ui.min.css">

<server>

var profile = catalog.getProfile();

if( !profile )
{
    res.write("No profile loaded")
        return
}
</server>


<script>
var target_to_name = {};
<server>
res.write("var the_topology = " + JSON.stringify(profile.Topology) + ";")
</server>
var cnt = 0;
for( var t in the_topology )
{
    if( t == "name")
        continue;
    target_to_name["" + cnt++] =  t
}
</script>


<h1>Communication Topology</h1>

<div id="controls">
<select id="topotype">
<server>


var cnt = 0;
for( var t in profile.Topology )
{
    if( t == "name")
        continue;
    res.write("<option value=\""+ cnt++ + "\">" +  t + "</option>")
}
</server>
</select>

<select id="topometric">
<option value="0">Total Time</option>
<option value="1">Total Size</option>
<option value="2">Call Count</option>
</select>

<button id="reset"> Reset View </button>

<input id="spinner" name="value"></input>
</div>

<div id="topocontainer">
<div id="leftpanel">
<div id="nodeinfo">
<h2 id=nodename>Node Information</h2>
<div id="deletediv" style='text-align:left;'>
<button id='delete'>Delete Node</button>
</div>
<div id="nodedata">
Please click on a node to display its information
</div>
</div>
<div id="graph"></div>
</div>

<div id="topology">
<svg id="topo"></svg>
</div>
</div>

<table id="lgdcont">
<tr>
<td>
<div id="legendmin"></div>
</td>
<td>
<div id="legend"></div>
<svg id="lgd"></svg>
</td>
<td>
<div id="legendmax"></div>
</td>
</tr>
</table>

<div>
<b id="repul"> Repulsion Force <-225>: </b>
<div id="topology-repulsion"></div>
</div>

<br>

<div>
<b id="fric"> Friction <0.5>: </b>
<div id="topology-friction"></div>
</div>

<script src='/interacttopo/topojs'></script>

<script>
$( document ).ready(function() {
    var topo = new topology()
    topo.render()
});
</script>


</body>;








<view uri="topojs" raw="true">

function topology()
{
    this.target = 0;
    this.type = "0";

    this.charge = -225;
    this.fric = 0.9;

    this.the_graph = null

    this.toSize = function( value )
    {
        var tb = 1024 * 1024 * 1024 * 1024;
        var gb = 1024 * 1024 * 1024;
        var mb = 1024 * 1024;
        var kb = 1024; 

        if( tb < value )
            return (value/tb).toFixed(3) + " TB";
        else if( gb < value )
            return (value/gb).toFixed(3) + " GB";
        else if( mb < value )
            return (value/mb).toFixed(3) + " MB";
        else if( kb < value )
            return (value/kb).toFixed(3) + " KB";
        else
            return value + " B";
    }

    this.toTime = function( value )
    {
        var a_day = 3600 * 24;
        var an_hour = 3600;
        var a_minute = 60;

        var ret = "";

        var d = 0;

        if( a_day < value )
        {
            ret += Math.floor( value / a_day ) + " d ";
            value = value%a_day;
        }

        if( an_hour < value )
        {
            ret += Math.floor( value / an_hour ) + " h ";
            value = value%an_hour;
        }

        if( a_minute < value )
        {
            ret += Math.floor( value / a_minute ) + " m ";
            value = value%a_minute;
        }


        if( value )
        {
            if( 1 < value )
                ret += value.toFixed(2) + " s ";
            else
                ret += value.toExponential(2) + " s ";
        }

        return ret;
    }

    this.noConv = function( d )
    {
        return  d;
    }


    this.conv = function ( value )
    {
        var conv = this.noConv;

        switch( this.type )
        {
            case "0":
                conv = this.toTime;
                break;
            case "1":			
                conv = this.toSize;
                break;
        }

        return conv( value );
    }

    this.renderLegend = function( ramp, min, max )
    {

        var cmin = min;
        var cmax = max;

        cmin = this.conv( min );
        cmax = this.conv( max );

        var numRect = 200;
        var width = 650;

        var q = (max - min ) / numRect;
        var xq = width / numRect;

        var data = [];
        var i;

        for( i = 0 ; i < numRect ; i++ )
        {;


            data.push( min + q * i );
        }


        $("body #lgd").empty();

        $("#legendmin").html( this.conv(min) );
        $("#legendmax").html( this.conv(max) );

        var svg = d3.select("body #lgd").attr("width", width).attr("height", 50);

        var pthis = this;

        svg.selectAll(".rect")
            .data(data)
            .enter().append("rect")
            .attr("width", xq  + 1)
            .attr("height", 50)
            .attr("fill", function(d, i){ return ramp(d); })
            .attr("x", function(d, i){ return xq * i; })
            .attr("y", 0)
            .on("mouseover", function(d){ $("#legendval").html( pthis.conv(d) ); })
            .on("mouseout", function(d){ $("#legendval").empty(); });

    }


    this.gen_inOroutbound_list = function( current_topo, id, isIn )
    {

        /* Build list */

        var outboud_list = {};

        var i = 0;

        var keys = [ "hits" , "time" ,  "size" ];


        for( k = 0 ; k  < keys.length ; k++ )
        {
            var key = keys[k];

            for( i = 0 ; i < current_topo.length ; i++ )
            {

                var t = current_topo[i]
                    var ref = -1

                    if( isIn )
                    {
                        if( t.dest == id )
                        {
                            ref = parseInt(t.src)
                        }

                    }
                    else
                    {
                        if( t.src == id )
                        {
                            ref = parseInt(t.dest)
                        }
                    }

                if( 0 <= ref  )
                {

                    if( !outboud_list[ ref ] )
                    {
                        outboud_list[ ref ]  = {};
                    }

                    outboud_list[ ref ][key] = t[key]; 
                }
            }
        }


        var ret = [];

        for( e in outboud_list )
        {
            if( !e )
                continue;

            ret.push( { node : e , hits : outboud_list[e].hits , time : outboud_list[e].time , size : outboud_list[e].size } );
        }


        return ret;
    }

    this.genNodeStats = function( inbound, outbound )
    {
        var nodelist = {};

        var ncount = { 'in' : 0 , 'out' : 0 };
        var hits = { 'in' : 0 , 'out' : 0 };
        var time = { 'in' : 0 , 'out' : 0 };
        var size = { 'in' : 0 , 'out' : 0 };


        var i = 0;


            for( i = 0 ; i < inbound.length ; i++ )
            {
                nodelist[inbound[i].node] = 1;

                hits.in+=inbound[i].hits;
                time.in+=inbound[i].time;
                size.in+=inbound[i].size;

                ncount.in++;
            }

        for( i = 0 ; i < outbound.length ; i++ )
        {
            nodelist[outbound[i].node] = 1;

            hits.out+=outbound[i].hits;
            time.out+=outbound[i].time;
            size.out+=outbound[i].size;

            ncount.out++;
        }

        var nodes = [];

        for( e in nodelist )
        {
            if( !e )
                continue;

            nodes.push( e );
        }

        ncount.total = nodes.length;

        return { 'ncount' : ncount , 'hits' : hits , 'time' : time , 'size' : size };
    }


    this.current_node = -1;


    this.render_node_info = function( current_topo, id )
    {
        $("#deletediv").show();
        $("#nodename").html("Rank " + id );

        this.current_node = id;
        $("#spinner").spinner( "value", id );

        $("circle").css("fill", "black");
        $("circle").attr("r", "4");

        $("circle#" + id).css("fill", "red");
        $("circle#" + id).attr("r", "12");

        var outbound = this.gen_inOroutbound_list( current_topo, id, 0 );
        var inbound = this.gen_inOroutbound_list( current_topo, id, 1 );

        var data = "<h3>Node Statistics</h3>";

        var stats = this.genNodeStats( inbound, outbound );

        data+="<i>Neighbour</i><br><br>";

        data += "<table class='nodetable'><tr><td>In</td><td>Out</td><td>Total</td></tr>"
            data += "<tr><td>" + stats.ncount.in + "</td><td>" + stats.ncount.out +"</td><td>" +  (stats.ncount.total) + "</td></tr>";	
        data += "</table>";

        data+="<br><i>Size</i><br><br>";

        data += "<table class='nodetable'><tr><td>In</td><td>Out</td><td>Total</td></tr>"
            data += "<tr><td>" + this.toSize(stats.size.in) + "</td><td>" + this.toSize(stats.size.out) +"</td><td>" + this.toSize(stats.size.in + stats.size.out) + "</td></tr>";
        data += "</table>";

        data+="<br><i>Time</i><br><br>";

        data += "<table class='nodetable'><tr><td>In</td><td>Out</td><td>Total</td></tr>"
            data += "<tr><td>" + this.toTime(stats.time.in) + "</td><td>" + this.toTime(stats.time.out) +"</td><td>" + this.toTime(stats.time.in + stats.time.out) + "</td></tr>";	
        data += "</table>";


        data+="<br><i>Hits</i><br><br>";

        data += "<table class='nodetable'><tr><td>In</td><td>Out</td><td>Total</td></tr>"
            data += "<tr><td>" + stats.hits.in + "</td><td>" + stats.hits.out +"</td><td>" +  (stats.hits.out  +  stats.hits.in) + "</td></tr>";	
        data += "</table>";


        data+="<h3>Outbound Data</h3>";

        data += "<table class='nodetable'><tr><td>Dest</td><td>Hits</td><td>time</td><td>Size</td></tr>"

            for( i = 0 ; i < outbound.length ; i++ )
            {
                data += "<tr><td>" + outbound[i].node + "</td><td>" + outbound[i].hits + "</td><td>" + this.toTime(outbound[i].time) +"</td><td>" + this.toSize(outbound[i].size) + "</td></tr>";	
            }

        data += "</table>";


        data+="<h3>Inbound Data</h3>";




        data += "<table class='nodetable'><tr><td>Src</td><td>Hits</td><td>time</td><td>Size</td></tr>"

            for( i = 0 ; i < inbound.length ; i++ )
            {
                data += "<tr><td>" + inbound[i].node + "</td><td>" + inbound[i].hits + "</td><td>" + this.toTime(inbound[i].time) +"</td><td>" + this.toSize(inbound[i].size) + "</td></tr>";	
            }

        data += "</table>";

            $("#nodedata").html( data );

    }


    this.getCurrentTopo = function(  )
    {
        var topo = this.topology_data;

        var current_topo = topo[ target_to_name[""+this.target]];

        if( !current_topo )
        {
            alert("No such topology");
            this.target = 0;
        }

        return current_topo;
    }



    this.getCurrentTopoLayout = function(  )
    {
        if( this.the_graph )
            return this.the_graph

        var topo = this.topology_data;

        var current_topo = this.topology_data[ target_to_name[this.target + ""]]

        if( !current_topo )
        {
           alert("No such topology");
           this.target = 0;
        }
        
        var tname = target_to_name[ "" + this.target ];
        var type_name = "time_layout";

        switch( this.type )
        {
            case "0":
                type_name = "time";
                break;
            case "1":
                type_name = "size";
                break;
            case "2":
                type_name = "hits";
                break;
        }


        var aret = {};

        var seen_nodes=[]
        var nodes = []
        var links = []

        var targ = current_topo

        for( var i = 0 ; i < targ.length ; i++ )
        {
                var v = targ[i]

                var src = parseInt(v.src)
                var dest = parseInt(v.dest)
 
                if( seen_nodes[src] == null )
                {
                    var name = "Rank "+ src
                    seen_nodes[src] = 1;
                    var nsrc = {"name": name , "id" : src, "nid" : src };
                    nodes[src] = nsrc
                }

                if( seen_nodes[dest] == null )
                {
                    var name = "Rank "+ dest
                    seen_nodes[dest] = 1;
                    var ndest =  {"name": name , "id" : dest, "nid" : dest } 
                    nodes[dest] = ndest
                }

                var nlink = {"source" : src, "target" : dest , "value" : v[type_name] };  
                links.push( nlink )
            
        }



        aret["nodes"] = nodes
        aret["links"] = links

        this.the_graph = aret

        return aret;
    }


    this.renderThreeTopo = function()
    {
        $("#graph").empty();
        //component("threegraph","graph").render(MPI_profile, this.target, this.type, 250, 250 );
    }

    this.force = undefined;


    this.renderTopo = function()
    {
        var graph = this.getCurrentTopoLayout();

        if( !graph )
        {
            alert("No such data " + type_name + " for " + tname );
            type_name = "time_layout";
        }


        var width = 1000,
            height = 600;

        var color = d3.scale.category20();

        this.force = force = d3.layout.force()
            .size( width, height )
            .charge(this.charge)
            .friction(this.fric)
            .linkStrength(1)
            .size([width, height]);

        $("#topo").empty();


            var svg = d3.select("body #topo")
            .attr("width", width + "px")
            .attr("height", height + "px");

        this.force
            .nodes(graph.nodes)
            .links(graph.links)
            .start();

        var minv = -1;	  
        var maxv = -1;	  
        var i = 0;

        for( i = 0; i < graph.links.length ; i++ )
        {

                if( ( graph.links[i].value < minv ) || (minv == -1) )
                    minv = graph.links[i].value;

            if( ( maxv < graph.links[i].value ) || (maxv == -1) )
                maxv = graph.links[i].value;
        }

        console.log("MIN : " + minv  + " MAX : " + maxv )



            var pthis = this;
        var one_q = (maxv - minv)/5.0;
        var ramp = d3.scale.linear().domain([minv, minv + one_q, minv + one_q*2.0, minv + one_q*3.0, minv + one_q*4.0]).range(["back", "blue", "green", "orange", "red",]);

        this.renderLegend( ramp, minv, maxv );


        var link = svg.selectAll(".link")
            .data(graph.links)
            .enter().append("line")
            .attr("id",function(d){return d.source.id + "-" + d.target.id;})
            .attr("class", "link")
            .attr("title", function(d){ return pthis.conv( d.value ); })
            .style("stroke-width", function(d) { return 3; })
            .style("stroke", function(d) { return ramp(d.value); });




        var node = svg.selectAll(".node")
            .data(graph.nodes)
            .enter().append("circle")
            .attr("class", "node")
            .attr("id", function(d) { return d.id; } )
            .attr("nid", function(d) { return d.nid; } )
            .attr("r", 4)
            .style("fill", function(d) { return "black"; })
            .on("click", function(d)
            {
                pthis.render_node_info( pthis.getCurrentTopo(), d.id );
            })
        .call(this.force.drag);

        node.append("title")
            .text(function(d) { return d.name; });

        this.force.on("tick", function() {
            link.attr("x1", function(d) { return d.source.x; })
            .attr("y1", function(d) { return d.source.y; })
            .attr("x2", function(d) { return d.target.x; })
            .attr("y2", function(d) { return d.target.y; });

        node.attr("cx", function(d) { return d.x; })
            .attr("cy", function(d) { return d.y; });
        });

        if( 0 <= this.current_node )
            pthis.render_node_info( pthis.getCurrentTopo(), this.current_node);

        this.renderThreeTopo();
    }



    this.attachSlider = function( target, min, max, func, is_vert )
    {
        $( "#" + target.trim() ).slider({ orientation: (is_vert)?"vertical" : "horizontal",
            min: min,
        max: max,
        step: ( max - min ) / 100.0,
        value: (min + max)/2,
        change: func
        });							
    }


    this.clone = function( data )
    {
        return $.extend(true, {}, data);
    }

    this.removeNode = function()
    {
        if( this.current_node < 0 )
            return;

        $("#deletediv").hide();


        var topo = this.getCurrentTopoLayout();

        if( !topo )
            return;

        var nodes = [];
        var links = [];

        var i = 0;

        for( i = 0 ; i < topo.nodes.length ; i++ )
        {
            if( topo.nodes[i].id != this.current_node )
                nodes.push( topo.nodes[i] );
        }

        for( i = 0 ; i < topo.links.length ; i++ )
        {
            if( (topo.links[i].source.id != this.current_node) && (topo.links[i].target.id != this.current_node) )
                links.push( topo.links[i] );
        }

        $("#topo svg").remove();

        topo.nodes = nodes;
        topo.links = links;

        this.current_node = -1;	
    }


    this.topology_data = this.clone( the_topology );

    this.render = function( fct, type )
    {
        if( fct )
            this.target = fct;

        if( type )
            this.type = type;


        var pthis = this;

        /* Handle select change */
        $("#topotype").on("change", function()
                {
                    pthis.the_graph = null
                    $("#densities div svg").empty();
                    pthis.target = $(this).val();
                    pthis.renderTopo();
                });

        /* Handle select change */
        $("#topometric").on("change", function()
                {
                    pthis.the_graph = null
                    $("#densities div svg").empty();
                    pthis.type = $(this).val();
                    pthis.renderTopo();
                });

        /* Handle reset */
        $("#reset").on("click", function()
                {
                    pthis.the_graph = null
                    pthis.renderTopo();
                });


        $("#deletediv").hide();

        $("#deletediv").on("click", function()
                {
                    pthis.removeNode();
                    pthis.renderTopo();
                });

        $("#graph").on("click", function()
                {
                    $("#content").empty();
                    //component("threeview","content").render(MPI_profile, pthis.target, pthis.type);
                });

        this.attachSlider( "topology-repulsion" , -1000, 100 , function(event, ui)
                {
                    $("#repul").html( "Repulsion Force <" + ui.value + "> :" );
                    pthis.charge = ui.value;
                    pthis.renderTopo();
                }, 0);

        this.attachSlider( "topology-friction" , 0, 1 , function(event, ui)
                {
                    $("#fric").html( "Friction <" + ui.value + "> :" );
                    pthis.fric = 1- ui.value;
                    pthis.renderTopo();
                }, 0);


        var spinner = $( "#spinner" ).spinner();
        $( "#spinner" ).spinner({
            spin: function( event, ui ) {
                var value = ui.value;
                <server>	
            var app = catalog.getCurrentApp()
            res.write("var size = "+ app.size + " ;")
            </server>
            if ( size <= value ) {
                value = 0;
            } else if ( ui.value < 0 ) {
                value = size - 1;
            }

        $( this ).spinner( "value", value );

        return false;
            },
            change : function( event, ui ) {
                var value = $( this ).spinner( "value" );

                if( value == null )
                    value = 0;
                <server>	
                    var app = catalog.getCurrentApp()
                    res.write("var size = "+ app.size + " ;")
                    </server>

                    if ( size <= value ) {
                        value = size - 1;
                    } else if ( value < 0 ) {
                        value = 0;
                    }

                $( this ).spinner( "value", value );

                pthis.render_node_info( pthis.getCurrentTopo(), value);
            }
        });


        /* Render TOPO */
        pthis.renderTopo();

    }

}
    <server>
res.end()
    </server>

    </view>
