var catalog = module.require("malp_catalog").catalog;
var timeMatrix = require("malp_rendering").timeMatrix;
var timeMatrixScale = require("malp_rendering").timeMatrixScale;
var meta = require("malp_rendering").metadata;
var conv = require("malp_rendering").converter;

<page uri="/flatmatrix" parent="/" title="Flat Matrix" />



<body>

	<server>

	//Acquire the collector

	var current_prof = catalog.getProfile();

	if( current_prof == undefined )
	{
	</server>
		<h1>No profile loaded</h1>
	<server>
		return;
	}

	var collector = current_prof["Flat_Matrix"];

	//Now generate function list 

	var function_list = [];
	var i;

	var list = collector.measures;

	if( list == undefined )
	{
		list = collector.mpi;
	}



	for( i = 0 ; i < list.length ; i++ )
	{
		function_list.push( { "name" : list[i].name , "function_class" : "mpi" } );
	}


	res.write("<script>\n");
	res.write("var current_function = '" + function_list[0].name  + "';\n");
	res.write("var current_type = 'time';\n");
	res.write("</script>\n");



	</server>
	
	<div>
	<select id='function'>
		<server>
			for( i = 0 ; i < function_list.length ; i++ )
			{
				res.write("<option value='" + function_list[i].name + "'>" + function_list[i].name + "</option>");
			}
		</server>
	</select>

	<select id='type'>
		<option value="time">Time</option>
		<option value="hits">Hits</option>
		<option value="size">Size</option>
	</select>

	<button id='load'>Add</button>

	</div> 

	<div id="tms"></div>


	<script type="text/javascript" src="/scripts/jquery.js"></script>
	<script>


	$(document).ready( function()
	{
		
		$('#function').bind('change', function ()
		{
			current_function = this.value;
		});
		
		$('#type').bind('change', function ()
		{
			current_type = this.value;
		});

		
		$('#load').bind('click', function ()
		{
			     $.ajax({url: "/flatmatrix/getmatrix?f="+current_function+"&t="+ current_type + "&body_only=1" }).done(function( data ) {
				 
				 if( data == "null" )
				 {
					 alert("No attribute " + current_type + " for " + current_function );
					 return;
				 }
				 
				 $("#tms").append(data);
				 
				//We bind each time to have effect on new entries 
				$("div.graphcontainer").bind('click', function ()
				{
					var id = pos = $(this).attr("id");
					$("#" + id).remove();
				});
			  });
			 
		});
		
	});



	</script>

</body>



var MAT_ID = 0;

<view uri="getmatrix">
	<script type="text/javascript" src="/scripts/d3.js"></script>
	<link rel="stylesheet" type="text/css" href="/css/nv.d3.css"/>
	<script type="text/javascript" src="/scripts/nv.d3.js"></script>
	
	<server>
		var function_name = GET.f;
		var type = GET.t;
		
		if( !function_name || !type )
		{
			return;
		}
		
		var map = get_flatmat_data( function_name, type );
		
		if( !map )
		{
			res.write("null");
			return;
		}
		
		var local_id = MAT_ID++;
		
		res.write("<div id='mat" + local_id + "' class='graphcontainer' matid='" + local_id + "'><h1>"+ function_name + " in  " + type + "</h1>");		
		res.write("<svg width='80%' height=200 id='graph" + local_id + "'></svg>\n");
		
		res.write("<script>");
		res.write("var graph_id = '#graph" + local_id + "';\n");
		res.write("var quantum = " + map.q + ";\n");
		res.write("var graph_data = " + JSON.stringify(map.data[0]) + "\n");
		res.write("var type = '" + type + "'\n");
		res.write("var function_name = '" + function_name + "'\n");

	</server>
	
	function gen_data_x_y( data, is_time )
	{
			var i =0;
			var ret = [];
			
			for( i = 0 ; i < data.length ; i++ )
			{
				var val = data[i];
				
				if( is_time )
				{	
					val = val*100 / quantum;
				}
				
				ret.push({ x : i * quantum , y : val });
			}
			
			return ret;
	};
			
	
	$(document).ready( function()
	{
		 var chart = nv.models.lineChart()
						.margin({left: 100, right: 100}) //Adjust chart margins to give the x-axis some breathing room.
						.useInteractiveGuideline(true) //We want nice looking tooltips and a guideline!
						.showLegend(true) //Show the legend, allowing users to turn on/off line series.
						.showYAxis(true) //Show the y-axis
						.showXAxis(true) //Show the x-axis
						;
	
		chart.xAxis.axisLabel('Time (s)')
			   .tickFormat(d3.format(',r'));
					   
		if( type == "time" )
		{ 
			chart.yAxis.axisLabel('Time Quantum %')
			           .tickFormat(d3.format('.02f'));
		}else if( type == "hits" )
		{
			chart.yAxis.axisLabel('Hits')
			           .tickFormat(d3.format('d'));	
			
		}else if( type == "size" )
		{
			chart.yAxis.axisLabel('Size (bytes)')
			           .tickFormat(d3.format('d'));	
		}
		 
		/* Done setting the chart up? Time to render it!*/
		var x_y = gen_data_x_y( graph_data, type == "time" );	
		
		var myData = [{
			key : function_name,
			values: x_y
		}];
		 
		d3.select(graph_id) //Select the <svg> element you want to render the chart in.
		.datum(myData) //Populate the <svg> element with chart data...
		.call(chart); //Finally, render the chart!
		 
		//Update the chart when window resizes.
		nv.utils.windowResize(function() { chart.update() });
		
		
	} );
	</script>
	
</view>


function get_flatmat_data( function_name, type )
{
	// Try to load the profile
	var current_prof = catalog.getProfile();

	if( current_prof == undefined )
	{
		return undefined;
	}

	//And now the TM section
	var collector = current_prof["Flat_Matrix"];

	if( collector == undefined )
	{
		return undefined;
	}	
	
	var class_array = collector[ "measures" ];
	

	if( class_array == undefined )
	{
		return undefined;
	}		
	
	var i;
	
	for( i = 0 ; i < class_array.length ; i++ )
	{
		if( class_array[i].name == function_name )
		{
			return class_array[i].measures[ type ];
		}
		
	}
	
	return undefined;
}


