var catalog = module.require("malp_catalog").catalog;
var prof = require("malp_rendering").profile;
var meta = require("malp_rendering").metadata;
var conv = require("malp_rendering").converter;
var timeMatrix = require("malp_rendering").timeMatrix;

<page uri="/mpisection" parent="/" title="MPI Section" />


function sort_ranks()
{
	var current_prof = catalog.getProfile();
	var mpi_section = current_prof["MPI_Section"];
	
	mpi_section = mpi_section.ranks.sort( function (a,b){ return a.rank - b.rank; } );
}


function __unfold_section( section )
{
	var ret = "";

	ret += "<li>";
	ret += " <b>" + section.label + "</b>";
	ret += " [Hits] " + section.hits;
	ret += " [Time] " + section.total_time;
	if( section.exclusive_time )
		ret += " [Exclusive] " + section.exclusive_time;
	ret += "</li>\n";
	if( section.children.length)
	{
		ret += "<ul>\n";
		for( var i = 0 ; i < section.children.length ; i++)
		{
			 ret += __unfold_section( section.children[i], ret );
		}
		ret += "</ul>\n";
	}
	return ret;
}



function unfold_section( section_root)
{

	var ret = "<ul>\n";
	ret += __unfold_section( section_root.children[0] , ret);
	ret += "</ul>\n";
	return ret;
}


function __gen_section_balance( section )
{
	var ret = [];
	
	var x_y = [];
	
	for( var i = 0 ; i < section.balance.length ; i++ )
	{
		var p = section.balance[i];
		
		x_y.push( { 'x' : p[0], 'y' : p[1] });
	}
	
	var entry = { 'key' : section.label , 'values' : x_y };
	
	ret.push( entry );
	
	if( section.children.length)
	{
		for( var i = 0 ; i < section.children.length ; i++)
		{
			 ret = ret.concat(  __gen_section_balance( section.children[i], ret ) );
		}
	}
	
	return ret;
}


function gen_section_balance( section_root )
{
	return __gen_section_balance( section_root.children[0] );
}



var MAT_ID = 0;

<view uri="getperrank" raw="true">

	<script type="text/javascript" src="/scripts/d3.js"></script>
	<link rel="stylesheet" type="text/css" href="/css/nv.d3.css"/>
	<script type="text/javascript" src="/scripts/nv.d3.js"></script>

	<server>
		var local_id = MAT_ID++;
		var rank = GET.r;
		var current_prof = catalog.getProfile();
		var mpi_section = current_prof["MPI_Section"];
		
		var graph_data = gen_section_balance( mpi_section.ranks[rank].sections[0] );
		
		res.write("<div id='mat" + local_id + "cont' class='graphcontainer' matid='" + local_id + "'><h3>Rank "+ rank + "</h3>");
		res.write("<div id='mat" + local_id + "' class='graphcontrol' '>close</div>");
		res.write( unfold_section(mpi_section.ranks[rank].sections[0]) );
		res.write("<svg width='80%' height=500 id='graph" + local_id + "'></svg>\n");
		res.write("</div>");

				
		res.write("<script>");
		res.write("var graph_id = '#graph" + local_id + "';\n");
		res.write("var graph_data = " + JSON.stringify( graph_data ) + ";\n");
		res.write("</script>");
		
	</server>

	<script>
	$(document).ready( function()
	{
		 var chart = nv.models.lineChart()
						.margin({left: 100, right: 100}) //Adjust chart margins to give the x-axis some breathing room.
						.useInteractiveGuideline(true) //We want nice looking tooltips and a guideline!
						.showLegend(true) //Show the legend, allowing users to turn on/off line series.
						.showYAxis(true) //Show the y-axis
						.showXAxis(true) //Show the x-axis
						;
	
		chart.xAxis.axisLabel('Time (s)')
			   .tickFormat(d3.format(',r'));
					   

		chart.yAxis.axisLabel('Hits')
				   .tickFormat(d3.format('d'));	
	 
		var myData =  graph_data;
		 
		d3.select(graph_id) //Select the <svg> element you want to render the chart in.
		.datum(myData) //Populate the <svg> element with chart data...
		.call(chart); //Finally, render the chart!
		 
		//Update the chart when window resizes.
		nv.utils.windowResize(function() { chart.update() });
		
		
	} );
	</script>
	
	<server>
		res.end();
	</server>
</view>


function _get_map( current, keys, depth )
{
	if( depth == keys.length )
	{
		return current;
	}
	
	var key_to_find = keys[depth];
	
	//console.log("Look " + key_to_find + " in " + current.label + " " + JSON.stringify(keys));
	
	if( !key_to_find )
	{
		return null;
	}
	
	if( current.children.length )
	{
		for( var i = 0 ; i < current.children.length ; i++)
		{
				if( current.children[i].label == key_to_find )
				{
					return _get_map( current.children[i], keys, depth + 1);
				}
		}
	}
	else
	{
		return null;
	}

}



function get_map( root, path )
{
	var keys = path.split("/");

	keys[0] = "";


	return _get_map( root, keys, 1 );
}


function gen_section_map_bounds( path )
{
	var current_prof = catalog.getProfile();
	var mpi_section = current_prof["MPI_Section"];
	
	var min_y = 0;
	var max_y = mpi_section.ranks.length;
	
	var min_x = -1;
	var max_x = -1;
	
	var min_z = -1;
	var max_z = -1;

	for( var i = 0 ; i < mpi_section.ranks.length ; i++ )
	{
		var root = mpi_section.ranks[i].sections[0];
		var target_section = get_map( root, path );
		
		if( target_section )
		{
			/* Found */
			for( var j = 0 ; j < target_section.balance.length ; j++ )
			{
				var val = target_section.balance[j];
				var ts = parseFloat(val[0]);
				var hits = parseFloat(val[1]);
				
				if( (min_x < 0 ) || ( ts < min_x ) )
				{
					min_x = ts;
				}
				
				if( (max_x < 0 ) || ( max_x < ts ) )
				{
					max_x = ts;
				}
				
				if( (min_z < 0 ) || ( hits < min_x ) )
				{
					min_z = hits;
				}
				
				if( (max_z < 0 ) || ( max_x < hits ) )
				{
					max_z = hits;
				}
				
			}
			
		}
	}
	
	return {"min_y" : min_y, "max_y" : max_y , "min_x" : min_x , "max_x" : max_x , "min_z" : min_z, "max_z" : max_z };
}



function gen_section_map( path , buckets )
{
	if( buckets == undefined )
		buckets = 64;
	
	var current_prof = catalog.getProfile();
	var mpi_section = current_prof["MPI_Section"];
	
	var bounds = gen_section_map_bounds( path );
	
	var min_y = bounds.min_y;
	var max_y = bounds.max_y;
	
	var min_x = bounds.min_x;
	var max_x = bounds.max_x;
	
	var min_z = bounds.min_z;
	var max_z = bounds.max_z;

	for( var i = 0 ; i < mpi_section.ranks.length ; i++ )
	{
		var root = mpi_section.ranks[i].sections[0];
		var target_section = get_map( root, path );
		
		if( target_section )
		{
			/* Found */
			for( var j = 0 ; j < target_section.balance.length ; j++ )
			{
				var val = target_section.balance[j];
				var ts = parseFloat(val[0]);
				var hits = parseFloat(val[1]);
				
				if( (min_x < 0 ) || ( ts < min_x ) )
				{
					min_x = ts;
				}
				
				if( (max_x < 0 ) || ( max_x < ts ) )
				{
					max_x = ts;
				}
				
				if( (min_z < 0 ) || ( hits < min_x ) )
				{
					min_z = hits;
				}
				
				if( (max_z < 0 ) || ( max_x < hits ) )
				{
					max_z = hits;
				}
				
			}
			
		}
	}

	console.log("min : " + min_x + " max : " + max_x );
	
	var size_x = buckets;
	var q = (max_x - min_x)*1.0 / (size_x - 1);
	
	/* Create the array */
	var array = Array( max_y );
	
	for( var i = 0 ; i < max_y ; i++ )
	{
		array[i] = [];
		for( var j = 0 ; j < size_x; j++ )
		{
			array[i][j] = 0.0;
		}
	}
	
	/* Fill the array */
	for( var i = 0 ; i < mpi_section.ranks.length ; i++ )
	{
		var root = mpi_section.ranks[i].sections[0];
		var target_section = get_map( root, path );
	
		
		
		if( target_section )
		{
			/* Found */
			for( var j = 0 ; j < target_section.balance.length ; j++ )
			{
				var val = target_section.balance[j];
				var ts = parseFloat(val[0]);
				var hits = parseFloat(val[1]);
				
				var target_cell = Math.floor((ts-min_x)/q);
				
				array[i][target_cell] += hits;
			}
			
		}
		else
		{
			console.log("NOT FOUND");
		}
	}

	return { "map" : array , "min" : min_z, "max" : max_z };
}



<view uri="getmap" raw="true">

<server>
	var path = GET.r;
	var buckets = GET.b;
	var gauss = GET.g;
	var log = GET.l;
	var scale = GET.s;
	var mat =  gen_section_map( path , buckets);

	if( mat == undefined )
	{
		res.end("No such map");
		return;
	}

	console.log( mat.map );

	if( !gauss )
		gauss= 0;

	if( !log )
		log= 0;

	if( !buckets )
		buckets= 32;

	if( !scale )
		scale = "temperature";
		
	/* Render TM */
	timeMatrix( res, mat.map, 512, 512, scale, gauss, log );
</server>

</view>





<view uri="getpath" raw="true">

	<script type="text/javascript" src="/scripts/d3.js"></script>
	<link rel="stylesheet" type="text/css" href="/css/nv.d3.css"/>
	<script type="text/javascript" src="/scripts/nv.d3.js"></script>

	<server>
		var local_id = MAT_ID++;

		var path = GET.r;
		var buckets = GET.b;
		var gauss = GET.g;
		var log = GET.l;
		var scale = GET.s;

		if( !gauss )
			gauss= 0;

		if( !buckets )
			buckets= 32;

		if( !log )
			log= 0;

		if( !scale )
			scale = "temperature";

		var current_prof = catalog.getProfile();
		var mpi_section = current_prof["MPI_Section"];
		
		res.write("<div id='mat" + local_id + "cont' class='graphcontainer' matid='" + local_id + "'><h3>"+ path + "</h3>");
		res.write("<div id='mat" + local_id + "' class='graphcontrol' '>close</div>");
		
		/* Content */
		//Generate Matrix 
		res.write("<br><img width='100%' height='400px' src='getmap?r=" + path + "&b=" + buckets + "&g=" + gauss + "&l=" + log + "&s=" + scale +  "'>");

		res.write("</div>");

	</server>
	
	<server>
		res.end();
	</server>
</view>


function __build_path_list( array, path, section )
{
	if( section.children.length)
	{
		for( var i = 0 ; i < section.children.length ; i++)
		{
			var new_path = path + "/" + section.children[i].label;
			array[new_path] = 1;
			__build_path_list( array, new_path, section.children[i] );

		}
	}
}



function build_path_list()
{
	var current_prof = catalog.getProfile();
	var mpi_section = current_prof["MPI_Section"];
	
	
	var array = {}; 
	
	/* Unfold all ranks */
	for( i = 0 ; i < mpi_section.ranks.length ; i++ )
	{
		var root = mpi_section.ranks[i].sections[0];
		__build_path_list( array, "", root );
	}
	
	return array;
}


<body>
	<script src='/scripts/jquery.js'></script>

	<server>
		sort_ranks();

		var current_prof = catalog.getProfile();

                if( !current_prof )
                {
                    res.write("<h2>No profile loaded</h2>")
                    return
                }

		var mpi_section = current_prof["MPI_Section"];
   
                if( mpi_section.ranks.length == 0 )
                {
                    res.write("<h2>No section data in this profile</h2>")
                    return
                }
	</server>

	<h1>Command Pannel</h1>

	<div>
		<h2>Per Rank</h2>
		<select id='ranks'>
			<server>
				for( var i = 0 ; i < mpi_section.ranks.length ; i++ )
				{
					res.write("<option value='" + i + "'> Rank " + i + "</option>");
				}
			</server>
		</select>
		<button id='load'>Add Per Rank Information</button>
	</div>
	
	<server>
		var paths = build_path_list();
	</server>
	
	<div>
		<h2>Per Section</h2>
		<select id='paths'>
			<server>
				var a_path = "";
				for( var e in paths )
				{
					if( a_path == "" )
						a_path = e;
					res.write("<option value='" + e + "'>" + e + "</option>");
				}
			</server>
		</select>

	
	
		<select id='scale'>
			<option value="temperature">Temperature</option>
			<option value="fire">Fire</option>
			<option value="ice">Ice</option>
			<option value="greyscale">Greyscale</option>
			<option value="three levels">Three Levels</option>
			<option value="three levels linear">Three Levels Linear</option>
			<option value="three levels linear narrow">Three Levels Narrow Average</option>
			<option value="good and evil">Good and Evil</option>
		</select>

		<select id='gauss'>
			<option value="0">Gauss 0</option>
			<option value="1">Gauss 1</option>
			<option value="2">Gauss 2</option>
			<option value="3">Gauss 3</option>
			<option value="4">Gauss 4</option>
			<option value="5">Gauss 5</option>
			<option value="6">Gauss 6</option>
			<option value="7">Gauss 7</option>
			<option value="8">Gauss 8</option>
			<option value="9">Gauss 9</option>
		</select>


		<select id='logscale'>
			<option value="0">Linear Scale</option>
			<option value="1">Logarithmic Scale</option>
		</select>

		<button id='section_load'>Analyze MPI_Section</button>
	</div>
	
	<h2>Panels</h2>
	
	<server>
		res.write("<script>\n");
		res.write("var current_rank = 0;\n");
		res.write("var current_path = \"" + a_path + "\"\n");
		res.write("var current_scale = 'temperature';\n");
		res.write("var current_logscale = '0';\n");
		res.write("var current_gauss = '0';\n");
		res.write("</script>\n");
	</server>
		
	<div id="tms"></div>

	<script>


		$(document).ready( function()
		{
			
			$('#ranks').bind('change', function ()
			{
				current_rank = this.value;
			});
			
			$('#paths').bind('change', function ()
			{
				current_path = this.value;
			});
			
			$('#scale').bind('change', function ()
			{
				current_scale = this.value;
			});

			$('#gauss').bind('change', function ()
			{
				current_gauss = this.value;
				
			});
				
			$('#logscale').bind('change', function ()
			{
				current_logscale = this.value;
				
			});
			
			$('#load').bind('click', function ()
			{
					$.ajax({url: "/mpisection/getperrank?r="+current_rank}).done(function( data ) {
	
						 if( data == "null" )
						 {
							 alert("No attribute " + current_type + " for " + current_function );
							 return;
						 }
						 
						 $("#tms").append(data);
						 
						//We bind each time to have effect on new entries 
						$("div.graphcontrol").bind('click', function ()
						{
							var id = pos = $(this).attr("id") + "cont";
							$("#" + id).remove();
							var id = pos = $(this).attr("id");
							$("#" + id).remove();
						});
				   });
				 
			});
			
			$('#section_load').bind('click', function ()
			{
					$.ajax({url: "/mpisection/getpath?r="+current_path+"&s=" + current_scale +"&l=" + current_logscale + "&g=" + current_gauss }).done(function( data ) {
	
						 if( data == "null" )
						 {
							 alert("No attribute " + current_type + " for " + current_function );
							 return;
						 }
						 
						 $("#tms").append(data);
						 
						//We bind each time to have effect on new entries 
						$("div.graphcontrol").bind('click', function ()
						{
							var id = pos = $(this).attr("id") + "cont";
							$("#" + id).remove();
							var id = pos = $(this).attr("id");
							$("#" + id).remove();
						});
				   });
				 
			});
			
		});



	</script>

</body>

