var catalog = module.require("malp_catalog").catalog;
var prof = require("malp_rendering").profile;
var meta = require("malp_rendering").metadata;
var conv = require("malp_rendering").converter;

<page uri="/profile" parent="/" title="Profiles" />

function genProfData()
{
	var current_prof = catalog.getProfile();

	if( current_prof == undefined )
	{
		return undefined;
	}

	var mpip = current_prof["MPI_Profile"];
	var posixp = current_prof["POSIX_Profile"];

	if( !mpip && !posixp )
	{
		return undefined;
	}

	/* BUILD A CONCATENATION OF MPI AND POSIX PROFILES */
	var merged_collector = {};

	merged_collector.profile = mpip.profile.concat( posixp.profile );

	var total_time = meta.partition_cputime();

	var acounted_time = 0.0;
	var posix_time = 0.0;
	var mpi_time = 0.0;

	for( var i = 0 ; i < mpip.profile.length ; i++ )
	{
		mpi_time += mpip.profile[i].time;
	}

	for( var i = 0 ; i < posixp.profile.length ; i++ )
	{
		 posix_time  += posixp.profile[i].time;
	}

	/* Compute OTHER time */
	acounted_time = mpi_time + posix_time;
	var other_time = total_time - acounted_time;

	/* Add it to the profile */
	merged_collector.profile.push( { "name" : "Other Calls" , "time" : other_time, "hits" : 0.0 , "size" : 0.0 } );
	
	var ret = {};
	
	/* Render profile data */
	ret.merged_collector = merged_collector;
	
	/* Prepare pie data */
	ret.pieData = [ ["Other Calls time",other_time] , ["MPI time", mpi_time], ["POSIX time",posix_time] ];

	return ret;
}

var global_profile = undefined;

function getProfileData()
{
	if(!global_profile)
		global_profile = genProfData();
	
	return global_profile;
}


//This generated the breakdown data
<view uri="breakdowndata" raw="true">

<server>
	var data = getProfileData();
	
	if( data == undefined )
	{
		res.end("[]");
		return;
	}

	res.end(JSON.stringify( data.pieData ));
</server>


</view>


//This view generates the breakdown pie chart
<view uri="breakdown">
	<script src='/scripts/jquery.js'></script>
	<script src='/scripts/jquery.jqplot.js'></script>
	<script src='/scripts/jqplot.pieRenderer.js'></script>
	<script src='/scripts/jqplot.highlighter.js'></script>
	<script src='/scripts/jqplot.highlighter.js'></script>
	<link rel="stylesheet" type="text/css" href="/css/jquery.jqplot.css">

	<server>
		if( !getProfileData() )
		{
			res.write("<h1>No profile loaded</h1>");
			return;
		}
	</server>

	<server>
		if(! GET["body_only"] )
		{
			res.write("<h1>Time Breakdown</h1>");
		}
	</server>
	
	<div id="chart_summary" style="height:400px;width:300px;border-width: 0px;"></div>

	<script type="text/javascript">
		//Pie chart
		$(document).ready(function(){
			
			$.getJSON('/profile/breakdowndata', function(data)
			{

				$.jqplot('chart_summary', [
				data
				], {
						gridPadding: {top:0, bottom:38, left:0, right:0},
						seriesDefaults:{
							renderer:$.jqplot.PieRenderer,
							rendererOptions: { padding: 8, showDataLabels: true }
						},  
						highlighter: {
						  show: true,
						  formatString:'%s', 
						  tooltipLocation:'sw', 
						  useAxesFormatters:false
						}
					});	
			  });
		});

	</script>


	<table class="malptable">
		<tr>
			<th>Event Type</th>
			<th>Total time</th>
			<th>%</th>
		</tr>
	
<server>

		//Array sumarizing pie chart data
		var data = getProfileData().pieData.sort( function( a, b )
				      {
						if( a[1] < b[1] )
							return -1;
						if( b[1] < a[1] )
							return 1;

						return 0;
					});

		for( var i = (data.length - 1 ) ;  0 <= i  ; i-- )
		{
				res.write("<tr><td>" + data[i][0] + "</td><td>" + conv.time_unit(data[i][1]) + "</td><td>" + ((data[i][1] / meta.partition_cputime() )*100).toFixed(3)  + "</td></tr>");
		}
</server>

</table>

</view>


<view uri="summary">
	<script src='/scripts/jquery.js'></script>
	<script src='/scripts/jquery.jqplot.js'></script>
	<script src='/scripts/jqplot.pieRenderer.js'></script>
	<script src='/scripts/jqplot.highlighter.js'></script>
	<script src='/scripts/jqplot.highlighter.js'></script>
	<link rel="stylesheet" type="text/css" href="/css/jquery.jqplot.css">
	
		
	<server>
		var p = getProfileData();
		
		if( !p )
		{
			res.write("<h1>No profile loaded</h1>");
			return;
		}
	</server>
	
	<server>
		if(! GET["body_only"] )
		{
			res.write("<h1>Global Profile Summary</h1>");
			res.write("<p>Note that the 'Other Calls' contribution is only taken into account as a time metric and thus only visible in the time chart.</p>");
		}
	</server>
	


	<server>
		var profile  = new prof( res, p.merged_collector );
		profile.pie_chart_array(getProfileData().merged_collector.profile);
	</server>
		
	
</view>

<view uri="profile">
	<script src='/scripts/jquery.js'></script>
		
	<server>
		var p = getProfileData();
		
		if( !p )
		{
			res.write("<h1>No profile loaded</h1>");
			return;
		}
	</server>


	<server>
		if(! GET["body_only"] )
		{
			res.write("<h1>Global Profile</h1>");
			res.write("<p>The 'Other Calls' metric is computed by substracting measured times from the total CPU time.</p>");
		}

		var profile  = new prof( res, p.merged_collector );
		profile.render_global();
	</server>
</view>


<body>
	<script src='/scripts/jquery.js'></script>
	<script src='/scripts/jquery.jqplot.js'></script>
	<script src='/scripts/jqplot.pieRenderer.js'></script>
	<script src='/scripts/jqplot.highlighter.js'></script>
	<script src='/scripts/jqplot.highlighter.js'></script>
	<link rel="stylesheet" type="text/css" href="/css/jquery.jqplot.css">
	<script type="text/javascript" src="/scripts/jquery.js"></script>

	<div id="breakdown"></div>
	<div id="summary"></div>
	<div id="profile"></div>

	<script type="text/javascript">
	$(document).ready( function()
	{
		$("#breakdown").hide();
		$("#summary").hide();
		$("#profile").hide();
		
		$.ajax({url: "./breakdown?body_only=1"})
		.done(function( data ) {
				$("#breakdown").html( data ).slideDown(1500);
			  });
			  
		$.ajax({url: "./summary?body_only=1"})
		.done(function( data ) {
				$("#summary").html( data ).slideDown(1500);
			  });
			  
		$.ajax({url: "./profile?body_only=1"})
		.done(function( data ) {
				$("#profile").html( data ).slideDown(1500);
			  });
	});
	</script>

</body>
