#include <errno.h>
#include <stdio.h>
#include <fcntl.h>
#include <string.h>

#include "allinea_metric_plugin_api.h"


struct io_stat
{
	size_t rchar; /** Char read */
	size_t wchar; /** Char written */
	size_t syscr; /** Syscall r */
	size_t syscw; /** Syscall w */
	size_t read_bytes; /** Actual read */
	size_t write_bytes; /** Actual write */
	size_t cancelled_write_bytes; /** Bytes truncated */
};

int read_io_stat( struct io_stat * st )
{
	char buff[300];
	FILE * f = fopen( "/proc/self/io", "r");


	int ret = fread( buff, 1, 300, f );

	//fwrite( buff, 1, ret, stdout );

	if( sscanf( buff, "rchar: %ld\nwchar: %ld\nsyscr: %ld\nsyscw: %ld\nread_bytes: %ld\nwrite_bytes: %ld\ncancelled_write_bytes: %ld",
	       		&st->rchar, &st->wchar, &st->syscr, &st->syscw, &st->read_bytes, &st->write_bytes, &st->cancelled_write_bytes ) != 7 )
	{
		return 1;
	}
	

	fclose( f );

	return 0;
}



struct io_stat prev_values;



int allinea_plugin_initialize(plugin_id_t plugin_id, void *unused)
{
	memset( &prev_values, 0, sizeof( struct io_stat ) );
}

int allinea_plugin_cleanup(plugin_id_t plugin_id, void *unused)
{
	
}



int rchar_collect(metric_id_t metric_id, struct timespec *in_out_sample_time, double *out_value)
{
	struct io_stat st;
	 
	if( read_io_stat( &st ) )
	{
		allinea_set_metric_error_messagef(metric_id, errno,
		"Error opening /proc/self/io\n");
		return 0;
	}

	
	*out_value = ( st.rchar - prev_values.rchar) / (1024.0 * 1024.0); 
	prev_values.rchar = st.rchar;
        return 1;	
}


int wchar_collect(metric_id_t metric_id, struct timespec *in_out_sample_time, double *out_value)
{
	struct io_stat st;
	 
	if( read_io_stat( &st ) )
	{
		allinea_set_metric_error_messagef(metric_id, errno,
		"Error opening /proc/self/io\n");
	}
	
	
	*out_value = ( st.wchar - prev_values.wchar) / (1024.0 * 1024.0); 
	prev_values.wchar = st.wchar;
}

int syscr_collect(metric_id_t metric_id, struct timespec *in_out_sample_time, uint64_t *out_value)
{
	struct io_stat st;
	 
	if( read_io_stat( &st ) )
	{
		allinea_set_metric_error_messagef(metric_id, errno,
		"Error opening /proc/self/io\n");
	}

	*out_value = st.syscr - prev_values.syscr;
	prev_values.syscr = st.syscr;
}

int syscw_collect(metric_id_t metric_id, struct timespec *in_out_sample_time, uint64_t *out_value)
{
	struct io_stat st;
	 
	if( read_io_stat( &st ) )
	{
		allinea_set_metric_error_messagef(metric_id, errno,
		"Error opening /proc/self/io\n");
	}

	*out_value = st.syscw - prev_values.syscw;
	prev_values.syscw = st.syscw; 
}


int read_bytes_collect(metric_id_t metric_id, struct timespec *in_out_sample_time, double *out_value)
{
	struct io_stat st;
	 
	if( read_io_stat( &st ) )
	{
		allinea_set_metric_error_messagef(metric_id, errno,
		"Error opening /proc/self/io\n");
	}

	*out_value = ( st.read_bytes - prev_values.read_bytes) / (1024.0 * 1024.0); 
	prev_values.read_bytes = st.read_bytes;
}


int write_bytes_collect(metric_id_t metric_id, struct timespec *in_out_sample_time, double *out_value)
{
	struct io_stat st;
	 
	if( read_io_stat( &st ) )
	{
		allinea_set_metric_error_messagef(metric_id, errno,
		"Error opening /proc/self/io\n");
	}
	
	*out_value = ( st.write_bytes - prev_values.write_bytes) / (1024.0 * 1024.0); 
	prev_values.write_bytes = st.write_bytes;
}

int cancelled_write_bytes_collect(metric_id_t metric_id, struct timespec *in_out_sample_time, double *out_value)
{
	struct io_stat st;
	 
	if( read_io_stat( &st ) )
	{
		allinea_set_metric_error_messagef(metric_id, errno,
		"Error opening /proc/self/io\n");
	}
	

	*out_value = (double)st.cancelled_write_bytes / (1024.0 * 1024.0); 
}
