var fs = require("fs");
var path = require("path");

/* First lets build a list of current directory in BFS order */
var list_of_files = [];
var current_id = 0;
var start_dir = process.cwd();

/* Define a file loader */
function File( pathname )
{
	this.full_path = pathname.replace(new RegExp(start_dir,"g"), ".");
	this.id = current_id++;
	
	this.directory = path.dirname( this.full_path );
	this.filename = path.basename( this.full_path );
	this.length = 0;
	
	/* Is it a directory */
	stat = fs.statSync(pathname);
	
	if( stat.isDirectory() )
	{
		this.is_directory = 1;
	}
	else
	{
		/* We have data ! */
		this.is_directory = 0;
		
		try
		{
			this.content = fs.readFileSync(this.full_path);
			this.length = this.content.length;
		}
		catch( e )
		{
			console.log(e);
		}
	}
	
	//console.log("Loaded from " + this.directory + " :::::: " + this.filename  );
}


File.prototype.toCarray = function()
{
	if( this.is_directory )
		return "";
	
	if( !this.content || !this.filename )
		return "";
	
	ret = "/* " + this.filename +" */\n";
	ret += "int file_" + this.id + "_len = " + this.content.length + ";\n";
	ret += "char file_" + this.id +"[" + this.content.length + "] = {";
	
	var i = 0;
	
	for( i = 0 ; i < this.content.length ; i++ )
	{
		ret += " " + this.content.readInt8( i );
		
		if( i != ( this.content.length - 1 ) )
			ret += ",";
			
		if( !(i % 16) )
			ret += "\n";
	}
	
	
	return ret + "};\n\n";
}


File.prototype.dump = function()
{
	var ret = "\t/* " + this.filename +" */\n";
	if( this.is_directory )
	{
		ret += "\tmkdir(\"" + this.full_path + "\", 0777);\n"
	}
	else
	{
	
		if( this.content )
		{
			ret += "\tdump_file( \"" + this.full_path  + "\" , file_" + this.id + ", " + this.length + " );\n";
		}
	
	}

	return ret;
}

File.prototype.remove = function()
{
	var ret = "\t/* " + this.filename +" */\n";
	if( this.is_directory )
	{
		ret += "\trmdir(\"" + this.full_path + "\");\n"
	}
	else
	{
	
		if( this.content )
		{
			ret += "\tunlink( \"" + this.full_path  + "\");\n";
		}
	
	}

	return ret;
}

function push_entry( path )
{
	list_of_files.push( new File( path ) );
}


/* Gen the list  */
function recurse_directory( directory, handler )
{

	var files = fs.readdirSync( directory );

	var i = 0;

	for( i = 0 ; i < files.length ; i++ )
	{
		try
		{
			stat = fs.statSync(directory + "/" + files[i]);
		}
		catch(e)
		{
			continue;
		}
		

		if( !stat.isDirectory() )
		{
			//console.log("Packing " + directory + "/" + files[i] );

			if( handler )
				(handler)(directory + "/" + files[i]);

			
			
				
			//fs.unlink( parsed_template );

		}
	}

	//Then if recursive load subdirectories

	var i = 0;
	
	for( i = 0 ; i < files.length ; i++ )
	{
		try
		{
			stat = fs.statSync(directory + "/" + files[i]);
		}
		catch(e)
		{
			continue;
		}
		
		if( stat.isDirectory() )
		{
				//console.log("Packing DIR " + directory + "/" + files[i] );

				if( handler )
					(handler)(directory + "/" + files[i]);
					
				recurse_directory( directory + "/" + files[i], handler );
		}
	}


}


recurse_directory( process.cwd(), push_entry );


/* We now have to generate packer's code */

/* First encode file objects */

var i = 0;

var c_data = "#include <stdio.h>\n#include <sys/stat.h>\n#include <sys/types.h>\n";

for( i = 0 ; i < list_of_files.length ; i++ )
{
	c_data += list_of_files[i].toCarray();
}

c_data += "void dump_file( char * path, char * data, int len ){\n" +
		  "\t FILE *out = fopen( path, \"w\");\n" +
		  "\t if( !out )\n" +
		  "\t { \n" +
		  "\t\t printf(\"Failed to write %s\\n\", path); \n" +
		  "\t\t return; \n" +
		  "\t } \n" +
		  "\t int i = 0; \n" +
		  "\t fwrite( data, sizeof( char ) , len , out );\n" +
		  "\t fclose( out ); \n" +
		  "}\n\n";
		  

c_data += "\n void dump_node_data()\n{\n";

for( i = 0 ; i < list_of_files.length ; i++ )
{
	c_data += list_of_files[i].dump();
}

c_data += "\n}\n\n";

c_data += "\n void remove_node_data()\n{\n";


/* Lets remove backward to empty dirs first */
for( i = (list_of_files.length - 1 ) ; 0 <= i ; i-- )
{
	c_data += list_of_files[i].remove();
}

c_data += "\n}\n\n";

//c_data += "int main( int argc, char ** argv ){\n" +
//		  "\t dump_node_data();\n" +
//		  "\t return 0;\n"+
//		  "}\n\n";




console.log( c_data );
