#ifndef TREE_BASED_REDUCTION_TREE
#define TREE_BASED_REDUCTION_TREE

#include <stdlib.h>
#include <pthread.h>
#include <stdint.h>

#include "Data_Pack.h"
#include "Conn_Ctx.h"
#include "Node_List.h"
#include "Red_Dispatch_Bootstrap.h"

/*
 * 
 * Reduction tree
 * 
 */


struct Red_Tree
{
	uint64_t instance_id;
	
	/* Unpacker */
	struct Data_Unpacker unpacker;
	
	/* These are client sockets (with pooling threads) */
	int lc_fd;  /** Left child socket */
	pthread_t lc_progress_thread; /** Right child reading thread */
	
	int rc_fd;  /** Right child socket */
	pthread_t rc_progress_thread; /** Left child reading thread */
	
	int pa_socket; /** Parent socket */
	pthread_mutex_t parent_socket_lock; /** Lock to make writing exclusive on the parent */
	
	/* Listening port context */
	char hostname[512];
	struct Connection_ctx listening_socket_ctx;
	int listening_socket; /** Socket on which the tree instance is listening */
	pthread_t accept_thread; /** Thread performing accepts on this socket */
	
	struct Node_list *node_list;
	
	/** Handler called when pushing data on the root */
	void (*data_handler)(struct Red_Tree *rt, char * packet, size_t size );
};

int Red_Tree_init( struct Red_Tree* rt, char * (target_host_handler)(struct Red_Tree *), int port );
int Red_Tree_connect( struct Red_Tree *rt, struct Connection_ctx * target );

int Red_Tree_release( struct Red_Tree *rt );

int Red_Tree_set_handler( struct Red_Tree *rt, void (*data_handler)( struct Red_Tree *, char * , size_t ) );
int Red_Tree_push_pack( struct Red_Tree *rt, struct Data_pack *pack );
int Red_Tree_push_data( struct Red_Tree *rt, void * data, int size );

void * Red_Tree_read_loop( void *pRed_Tree );


#endif /* TREE_BASED_REDUCTION_TREE */
