#include <stdlib.h>
#include <stdio.h>
#include <gd.h>

#include "Density.h"
#include "jsonCache.h"
#include "Color_Scales.h"

#include <math.h>


void walk_map( json_t * map, void (*handler)( int x, int y, double *value, void *arg ), void *arg )
{
	if( !handler )
		return;


	int size_x = json_to_array( map )->size;

	int x,y;
	double val = 0.0;

	for( x = 0 ; x < size_x ; x++ )
	{
		json_t * line = json_array_get( map, x );

		if( line->type != JSON_ARRAY )
		{
			printf("Error map shall only contain arrays\n");
			return;
		}

		int size_y = json_to_array( line )->size;

		for( y = 0 ; y < size_y ; y++ )
		{
			json_t * elem =  json_array_get( line, y );

			if( elem->type == JSON_REAL )
			{
				(handler)( x, y, &(json_to_real(elem)->value) , arg );
			}
			else if( elem->type == JSON_INT )
			{
				val = json_to_int(elem)->value;
				(handler)( x, y, &val , arg );
				json_to_int(elem)->value = val;
			}
			else
			{
				continue;
			}


			json_decref( elem );

		}

		json_decref( line );


	} 
}


struct Map_min_max
{
	double min;
	double max; 
};


void compute_min_max( int x, int y, double *value, void *pminmax )
{
	struct Map_min_max * mm = ( struct Map_min_max * ) pminmax;

	if( ( mm->min == 0 ) || (*value < mm->min ) )
	{
		mm->min = *value;
	}

	if( ( mm->max == 0 ) || ( mm->max < *value ) )
	{
		mm->max = *value;
	} 

} 

struct Map_redering_ctx
{
	struct Map_min_max minmax;
	gdImagePtr image;
	Color_Scale_t scale;

	int gauss;
	int logscale;

	int image_width;
	int image_height;

	int array_width;
	int array_height;

	double qx;
	double qy;
};

void render_map( int x, int y, double *value, void *pctx )
{
	struct Map_redering_ctx * ctx = (struct Map_redering_ctx *)pctx;

	struct Color col;
	
	double computed_value = *value; 
	double min = ctx->minmax.min; 
	double max = ctx->minmax.max - min; 

	if( ctx->logscale )
	{
		computed_value = log(computed_value*1e6 + 1.0);
		min = log( min*1e6 + 1.0 );
		max = log( (max)*1e6 + 1.0 );
	}


	double factor = (computed_value - min) / max ;

	// (double)( value - ctx->minmax.min ) / (double)( ctx->minmax.max - ctx->minmax.min );
	factor_to_color( &col, factor, ctx->scale );


	gdImageFilledRectangle(ctx->image, y * ctx->qy, x * ctx->qx, (y+1) * ctx->qy , (x+1) * ctx->qx, gdTrueColor(col.r, col.g, col.b));
}



void print_cell( int x, int y, double value, void *arg )
{
	printf("%d %d %g\n", x, y , value );
} 









void apply_gauss( struct Map_redering_ctx *ctx )
{
	int gauss_left = ctx->gauss;
	
	static const double gauss_kernel[5][5]={ {1.0,4.0,7.0,4.0,1.0},
											 {4.0,16.0,26.0,16.0,4.0},
										 	 {7.0,26.0,41.0,26.0,7.0},
										  	 {4.0,16.0,26.0,16.0,4.0},
											 {1.0,4.0,7.0,4.0,1.0} };

	while( gauss_left )
	{

		int x,y,i,j;


		for( x = 0 ;  x < ctx->image_height ; x++ )
		{
			for( y = 0 ; y < ctx->image_width ; y++ )
			{
				double impulse[3] = { 0.0, 0.0, 0.0 };
				
				
				for( i = -2 ; i <= 2 ; i++ )
				{
					for( j = -2 ; j <= 2 ; j++ )
					{
						int tx = x + i;
						int ty = y + j;
						
						/* Boundary check */
						
						if( ( ty < 0 ) || ( tx < 0 ) )
						{
							continue;
						}
						
						if( ( ctx->image_height <= tx ) || ( ctx->image_width <= ty ) )
						{
							continue;
						}
						
						int color = gdImageGetTrueColorPixel(ctx->image, tx, ty);
						
						impulse[0] += gauss_kernel[i+2][j+2] * gdTrueColorGetRed( color );
						impulse[1] += gauss_kernel[i+2][j+2] * gdTrueColorGetGreen( color );
						impulse[2] += gauss_kernel[i+2][j+2] * gdTrueColorGetBlue( color );
					}
				}
				
				impulse[0] *= 1.0/273.0;
				impulse[1] *= 1.0/273.0;
				impulse[2] *= 1.0/273.0;


				gdImageSetPixel( ctx->image, x, y, gdTrueColor( (int)impulse[0] ,  (int)impulse[1] , (int)impulse[2] ) );
			}
		}








		gauss_left--;
	}


}


void render_density_map(  struct Density * dens , json_t * content )
{	
	/* Now parse the params
	 * 
	 * 
	 * 
	 *  */

	if( content->type != JSON_OBJECT )
	{
		printf("malp_density expects a JSON object");
		return;
	}

	json_t * scale = json_object_get( content, "scale" );

	if( !scale )
	{
		scale = json_string("temperature");
	}

	if( scale->type != JSON_STRING )
	{
		printf("Bad 'scale' type\n");
		return;
	}

	json_t * gauss = json_object_get( content, "gauss" );

	if( !gauss )
	{
		gauss = json_int(0);
	}


	json_t * width = json_object_get( content, "width" );

	if( !width )
	{
		width = json_int(1024);
	}	

	if( width->type != JSON_INT )
	{
		printf("Bad 'width' type\n");
		return;
	}

	json_t * height = json_object_get( content, "height" );

	if( !height )
	{
		height = json_int(512);
	}

	if( height->type != JSON_INT )
	{
		printf("Bad 'height' type\n");
		return;
	}

	json_t * map = json_object_get( content, "map" );

	if( !map )
	{
		printf("Cannot render a density map without data!\n");
		return;
	}

	if( map->type != JSON_ARRAY )
	{
		printf("Bad 'map' type\n");
		return;
	}
	
	json_t * do_log = json_object_get( content, "logscale" );

	if( !do_log )
	{
		do_log = json_bool(0);
	}
	else
	{
		if( do_log->type != JSON_BOOL )
		{
			printf("Bad 'logscale' type\n");
			return;
		}
	}


	/* We can now start our work =) 
	 * 
	 * Create the target image
	 * Note that we inverted W & H as we did so in the rest
	 * of this rendering phase
	 * 
	 * */

	int v_width = json_to_int(height)->value;
	int v_height = json_to_int(width)->value;

	if( !v_width || !v_height )
	{
		printf("Error Bad width/height values\n");
		return;
	}


	/* Make sure the size is divisible by our array size
	 * 
	 * 
	 * 
	 *  */

	int v_size_x = json_to_array( map )->size;

	json_t * first_array = json_array_get( map, 0 );

	if( !first_array )
	{
		printf("Map seem empty\n");
		return;
	}

	if( first_array->type != JSON_ARRAY )
	{
		printf("Error map shall only contain arrays\n");
		return;
	}

	int v_size_y = json_to_array( first_array )->size;

	json_decref( first_array );

	//printf("W : %d    H : %d SX : %d Sy : %d\n", v_width, v_height, v_size_x, v_size_y );

	if( v_size_x < v_width )
	{
		v_width -= v_width%v_size_x;
	}

	if( v_size_y < v_height )
	{
		v_height -= v_height%v_size_y;
	}


	//printf("W : %d    H : %d\n", v_width, v_height );


	/* Now that we parsed args fill in ctx 
	 * 
	 * 
	 * 
	 * 
	 * */

	struct Map_redering_ctx ctx;

	ctx.minmax.min = 0;
	ctx.minmax.max = 0;
	ctx.logscale = json_to_bool(do_log)->value;;
	ctx.image_width = v_width;
	ctx.image_height = v_height;
	ctx.image = gdImageCreateTrueColor( v_height , v_width );
	
	if( !ctx.image )
	{
		printf("Failled to create image\n");
		return;
	}
	
	/* Set backgroud to black */
	gdImageFill (ctx.image, 0, 0, gdTrueColor(0, 0, 0));

	ctx.scale = Color_Scale_from_name( json_to_string( scale )->value );

	ctx.gauss = json_to_int( gauss )->value;

	ctx.array_width = v_size_x;
	ctx.array_height= v_size_y;

	ctx.qx = (double)ctx.image_width / (double)ctx.array_width;
	ctx.qy = (double)ctx.image_height / (double)ctx.array_height;

	/* We can now start our work =) 
	 * 
	 * Extract points from the array
	 * 
	 * */

	//walk_map( map, print_cell, NULL );

	walk_map( map, compute_min_max, (void *)&ctx.minmax );
	
	/* We do the work only if min is different from MAX 
	 * otherwise send a black box (its enough =) ) */
	if( ctx.minmax.min != ctx.minmax.max )
	{
		walk_map( map, render_map, (void *)&ctx );

		/* If needed apply the gaussian convolution over the image */
		apply_gauss( &ctx );
	}

	/* Render and free the image */

	int image_size = 0;

	void * image = gdImageJpegPtr(ctx.image, &image_size, 100 );

	dens->data = image;
	dens->size = image_size;

	gdImageDestroy(ctx.image);


	/* Free our params */
	json_decref( scale );
	json_decref( gauss );
	json_decref( width );
	json_decref( height );
	json_decref( map );		

}




int Density_init( struct Density * dens , char * json_data )
{
	if( ! dens )
		return 0;
	
	dens->data = NULL;
	dens->size = 0;
	
	/* Parse JSON content */
	json_t * content = json_parse( json_data );

	if( !content )
	{
		fprintf(stderr, "Failed parsing ...\n");
		return 1;
	}
	
	/* Proceed with rendering */
	 render_density_map( dens , content );
	
	/* Release JSON data */
	json_decref( content );
	
	return 0;
}

int Density_release( struct Density * dens  )
{
	gdFree( dens->data );
	dens->size = 0;
}


int Density_dump( struct Density * dens , FILE * dest )
{
	if( !dens->size || !dens->data )
		return 0;
	
	int ret = fwrite( dens->data, sizeof( char ) , dens->size, dest );

	if( ret != dens->size )
	{
		perror("fwrite");
		return 1;
	}
	
	return 0;
}

