/* ############################ MALP License ############################## */
/* # Fri Jan 18 14:00:00 CET 2013                                         # */
/* # Copyright or (C) or Copr. Commissariat a l'Energie Atomique          # */
/* #                                                                      # */
/* # This software is governed by the CeCILL-C license under French law   # */
/* # and abiding by the rules of distribution of free software.  You can  # */
/* # use, modify and/ or redistribute the software under the terms of     # */
/* # the CeCILL-C license as circulated by CEA, CNRS and INRIA at the     # */
/* # following URL http://www.cecill.info.                                # */
/* #                                                                      # */
/* # The fact that you are presently reading this means that you have     # */
/* # had knowledge of the CeCILL-C license and that you accept its        # */
/* # terms.                                                               # */
/* #                                                                      # */
/* # Authors:                                                             # */
/* #   - BESNARD Jean-Baptiste jean-baptiste.besnard@cea.fr               # */
/* #                                                                      # */
/* ######################################################################## */

var fs = require('fs');
var spawn = require('child_process').spawn;

/*************************************************/
/* Extend the string class with useful functions */
/*************************************************/

String.prototype.contains = function( needle )
{
	var match = this.match(".*" + needle +".*");

	if( match )
		return 1;
	
	return 0;
}

String.prototype.trim = function()
{
	return this.replace(/^\s+|\s+$/g, '').replace(/\s+/g, ' ');
}

/*************************************************/
/* Argument Parsing and validation               */
/*************************************************/

var args = process.argv.splice(2);

/*
if( args.length != 3 )
{
	console.log("You must use the mpi wrapper as follows:");
	console.log("wrapper [mpicc] [script.w] [output_file]");
	return;
}
*/

var mpicc = args[0].trim();

var mpicc_args = [];

var sp_mpicc = mpicc.split(" ");

if( sp_mpicc.length != 1 )
{
	mpicc = sp_mpicc[0];
	mpicc_args = sp_mpicc.slice(1);
}


var script = args[1];
var output_file = args[2];

/*************************************************/
/* Simple main preprocessing with MPICC          */
/*************************************************/

/* Create a temporary C file with a simple main */

var tmp_file = "/tmp/temp" + Math.round(Math.random() * 1e6) + ".c";

try
{
	fs.writeFileSync(tmp_file, "#include <mpi.h> int main(int argc, char **argv){}");
}
catch(e)
{
	console.log("Failed to write test program");
	console.log(e);
	process.exit(1);
}

/* Compile this program with -E */

console.log("Launching " + mpicc + " -E " + tmp_file );

var mpi_h_data = "";

try
{
	mpicc_preprocessed_output = spawn(mpicc, mpicc_args.concat(["-E", tmp_file]));
}
catch(e)
{
	console.log("Failed to launch command " + mpicc );
	console.log(e);
	process.exit(1);
}

mpicc_preprocessed_output.stdout.on('data', function (data) {
  mpi_h_data += data;
});


mpicc_preprocessed_output.on('close', function (code)
{
/*************************************************/
/* Proccess mpicc preprocessed output            */
/*************************************************/
	
	if( code != 0 )
	{
		console.log(mpi_h_data);
		console.log('call to ' + mpicc + " exited with " + code);
		process.exit(1);
	}

	fs.unlinkSync(tmp_file);


/*************************************************/
/* Clean input data                              */
/*************************************************/	

	var tmp_mpi_h = mpi_h_data.replace(/\\\n/g, ' ').replace(/(\/\*([\s\S]*?)\*\/)|(\/\/(.*)$)/gm, ' ').split('\n');

	var single_buff_mpih = "";

	/* Filter out lines beginning with # */
	for( i = 0 ; i < tmp_mpi_h.length ; i++ )
	{
		for( j = 0 ; j < tmp_mpi_h[i].length ; j++ )
		{
			if( tmp_mpi_h[i][j] == "#" )
			{
				tmp_mpi_h[i] = "";
				continue;
			}
		}

		single_buff_mpih += tmp_mpi_h[i];
	}

	mpi_h_lines = single_buff_mpih.replace(/\n/g, ' ').replace(/\s+/g, ' ').split(";");
	tmp_mpi_h = ""
	single_buff_mpih = ""

/*************************************************/
/* Parsing functions                             */
/*************************************************/

	var var_counter = 0;
	
	function add_arg( arg )
	{
		ret = ""
		var sp = this.arg.replace(/[\s]+/g, ' ').split(" ");

		if( sp.length == 1 )
		{
			ret = arg + " var" + var_counter++;
		}
		
		return ret;
	}
	

	function MPI_func( ret, name, args )
	{	
		this.name = name.trim();
		this.ret = ret;
		this.args = args.split(",");
	}

	MPI_func.prototype.parse = function()
	{
	/* Now lets do a raw check to be sure that the PMPI symbol exists */
		if( !mpi_h_data.contains("P" + this.name ) )
		{
			console.log("No PMPI found for " + this.name );
			return 0;
		}
		
		var i = 0;
		
		/* Here we remove void arguments */
		for( i = 0 ; i < this.args.length ; i++ )
		{
			this.args[i] = this.args[i].trim();
			if( this.args[i] == "void" )
				delete this.args[i];
		}
		
		/* We we want to squash types and stars to allow easy splitting 
		 * and to check if names are specified in the prototype */
		for( i = 0 ; i < this.args.length ; i++ )
		{
			if( this.args[i] == undefined )
				continue;

			/* Make sure the star is not agains the arg name */
			this.args[i] = this.args[i].replace(/\*/g, '* ')

			this.args[i] = this.args[i].replace(/[\s]+\*/g, '*').replace(/const /g, 'const_').replace(/unsigned /g, 'unsigned_');
			this.args[i] = this.args[i].trim();
			
			/* Now we detect if we do not have a name */
			var sp = this.args[i].replace(/[\s]+/g, ' ').split(" ");

			if( sp.length == 1 )
			{
				this.args[i] = this.args[i] + " var" + var_counter++;
			}
			
			/* Split the arg */
			var tmp = this.args[i].split(" ");
			
			this.args[i] = tmp;
			
			/* This shall not happen */
			if( this.args[i].length != 2 )
			{
				console.log("Error parsing " + this.name );
				console.dir( this );
				return 0;
			}
			
			/* Put back space before the star & co */
			this.args[i][0] = this.args[i][0].replace(/\*/g, ' *').replace(/\*\s\*/g, '**').replace(/\*\s\*/g, '**').replace(/const_/g, 'const ').replace(/unsigned_/g, 'unsigned ');
			
			/* Just in case */
			if( this.args[i][1].replace(/\[.*\]/g, '').trim() == "" )
					this.args[i][1] = "var" + (var_counter++) + this.args[i][1];
		}
		
		/* Cleanup undefined args */
		var tmp = [];
		
		for( i = 0 ; i < this.args.length ; i++ )
		{
			if( this.args[i] != undefined )
				tmp.push( this.args[i] );
		
		}
		
		this.args = tmp;
		
		return 1;
	}
	
/*************************************************/
/* MPI Functions knowledge base                  */
/*************************************************/
	
	function gen_mpi_kb(mpifuncs)
	{
		var MPI_kb = {};
		
		var i;
		
		for( i = 0; i  < mpifuncs.length ; i++ )
		{
			MPI_kb[ mpifuncs[i].name ] = mpifuncs[i].args;
		}
		
		console.log( JSON.stringify( MPI_kb  ) );
	}

	MPI_Functions_kb = {"MPI_Abort":
						[["MPI_Comm","comm"],["int","errorcode"]],"MPI_Accumulate":
						[["void *","origin_addr"],["int","origin_count"],["MPI_Datatype","origin_datatype"],["int","target_rank"],["MPI_Aint","target_disp"],["int","target_count"],["MPI_Datatype","target_datatype"],["MPI_Op","op"],["MPI_Win","win"]],"MPI_Add_error_class":
						[["int *","errorclass"]],"MPI_Add_error_code":
						[["int","errorclass"],["int *","errorcode"]],"MPI_Add_error_string":
						[["int","errorcode"],["char *","string"]],"MPI_Address":
						[["void *","location"],["MPI_Aint *","address"]],"MPI_Allgather":
						[["void *","sendbuf"],["int","sendcount"],["MPI_Datatype","sendtype"],["void *","recvbuf"],["int","recvcount"],["MPI_Datatype","recvtype"],["MPI_Comm","comm"]],"MPI_Allgatherv":
						[["void *","sendbuf"],["int","sendcount"],["MPI_Datatype","sendtype"],["void *","recvbuf"],["int *","recvcounts"],["int *","displs"],["MPI_Datatype","recvtype"],["MPI_Comm","comm"]],"MPI_Alloc_mem":
						[["MPI_Aint","size"],["MPI_Info","info"],["void *","baseptr"]],"MPI_Allreduce":
						[["void *","sendbuf"],["void *","recvbuf"],["int","count"],["MPI_Datatype","datatype"],["MPI_Op","op"],["MPI_Comm","comm"]],"MPI_Alltoall":
						[["void *","sendbuf"],["int","sendcount"],["MPI_Datatype","sendtype"],["void *","recvbuf"],["int","recvcount"],["MPI_Datatype","recvtype"],["MPI_Comm","comm"]],"MPI_Alltoallv":
						[["void *","sendbuf"],["int *","sendcounts"],["int *","sdispls"],["MPI_Datatype","sendtype"],["void *","recvbuf"],["int *","recvcounts"],["int *","rdispls"],["MPI_Datatype","recvtype"],["MPI_Comm","comm"]],"MPI_Alltoallw":
						[["void *","sendbuf"],["int *","sendcounts"],["int *","sdispls"],["MPI_Datatype *","sendtypes"],["void *","recvbuf"],["int *","recvcounts"],["int *","rdispls"],["MPI_Datatype *","recvtypes"],["MPI_Comm","comm"]],"MPI_Attr_delete":
						[["MPI_Comm","comm"],["int","keyval"]],"MPI_Attr_get":
						[["MPI_Comm","comm"],["int","keyval"],["void *","attribute_val"],["int *","flag"]],"MPI_Attr_put":
						[["MPI_Comm","comm"],["int","keyval"],["void *","attribute_val"]],"MPI_Barrier":
						[["MPI_Comm","comm"]],"MPI_Bcast":
						[["void *","buffer"],["int","count"],["MPI_Datatype","datatype"],["int","root"],["MPI_Comm","comm"]],"MPI_Bsend":
						[["void *","buf"],["int","count"],["MPI_Datatype","datatype"],["int","dest"],["int","tag"],["MPI_Comm","comm"]],"MPI_Bsend_init":
						[["void *","buf"],["int","count"],["MPI_Datatype","datatype"],["int","dest"],["int","tag"],["MPI_Comm","comm"],["MPI_Request *","request"]],"MPI_Buffer_attach":
						[["void *","buffer"],["int","size"]],"MPI_Buffer_detach":
						[["void *","buffer"],["int *","size"]],"MPI_Cancel":
						[["MPI_Request *","request"]],"MPI_Cart_coords":
						[["MPI_Comm","comm"],["int","rank"],["int","maxdims"],["int *","coords"]],"MPI_Cart_create":
						[["MPI_Comm","old_comm"],["int","ndims"],["int *","dims"],["int *","periods"],["int","reorder"],["MPI_Comm *","comm_cart"]],"MPI_Cart_get":
						[["MPI_Comm","comm"],["int","maxdims"],["int *","dims"],["int *","periods"],["int *","coords"]],"MPI_Cart_map":
						[["MPI_Comm","comm"],["int","ndims"],["int *","dims"],["int *","periods"],["int *","newrank"]],"MPI_Cart_rank":
						[["MPI_Comm","comm"],["int *","coords"],["int *","rank"]],"MPI_Cart_shift":
						[["MPI_Comm","comm"],["int","direction"],["int","disp"],["int *","rank_source"],["int *","rank_dest"]],"MPI_Cart_sub":
						[["MPI_Comm","comm"],["int *","remain_dims"],["MPI_Comm *","new_comm"]],"MPI_Cartdim_get":
						[["MPI_Comm","comm"],["int *","ndims"]],"MPI_Close_port":
						[["char *","port_name"]],"MPI_Comm_accept":
						[["char *","port_name"],["MPI_Info","info"],["int","root"],["MPI_Comm","comm"],["MPI_Comm *","newcomm"]],"MPI_Comm_c2f":
						[["MPI_Comm","comm"]],"MPI_Comm_call_errhandler":
						[["MPI_Comm","comm"],["int","errorcode"]],"MPI_Comm_compare":
						[["MPI_Comm","comm1"],["MPI_Comm","comm2"],["int *","result"]],"MPI_Comm_connect":
						[["char *","port_name"],["MPI_Info","info"],["int","root"],["MPI_Comm","comm"],["MPI_Comm *","newcomm"]],"MPI_Comm_create_errhandler":
						[["MPI_Comm_errhandler_function *","function"],["MPI_Errhandler *","errhandler"]],"MPI_Comm_create_keyval":
						[["MPI_Comm_copy_attr_function *","comm_copy_attr_fn"],["MPI_Comm_delete_attr_function *","comm_delete_attr_fn"],["int *","comm_keyval"],["void *","extra_state"]],"MPI_Comm_create":
						[["MPI_Comm","comm"],["MPI_Group","group"],["MPI_Comm *","newcomm"]],"MPI_Comm_delete_attr":
						[["MPI_Comm","comm"],["int","comm_keyval"]],"MPI_Comm_disconnect":
						[["MPI_Comm *","comm"]],"MPI_Comm_dup":
						[["MPI_Comm","comm"],["MPI_Comm *","newcomm"]],"MPI_Comm_free_keyval":
						[["int *","comm_keyval"]],"MPI_Comm_free":
						[["MPI_Comm *","comm"]],"MPI_Comm_get_attr":
						[["MPI_Comm","comm"],["int","comm_keyval"],["void *","attribute_val"],["int *","flag"]],"MPI_Comm_get_errhandler":
						[["MPI_Comm","comm"],["MPI_Errhandler *","erhandler"]],"MPI_Comm_get_name":
						[["MPI_Comm","comm"],["char *","comm_name"],["int *","resultlen"]],"MPI_Comm_get_parent":
						[["MPI_Comm *","parent"]],"MPI_Comm_group":
						[["MPI_Comm","comm"],["MPI_Group *","group"]],"MPI_Comm_join":
						[["int","fd"],["MPI_Comm *","intercomm"]],"MPI_Comm_rank":
						[["MPI_Comm","comm"],["int *","rank"]],"MPI_Comm_remote_group":
						[["MPI_Comm","comm"],["MPI_Group *","group"]],"MPI_Comm_remote_size":
						[["MPI_Comm","comm"],["int *","size"]],"MPI_Comm_set_attr":
						[["MPI_Comm","comm"],["int","comm_keyval"],["void *","attribute_val"]],"MPI_Comm_set_errhandler":
						[["MPI_Comm","comm"],["MPI_Errhandler","errhandler"]],"MPI_Comm_set_name":
						[["MPI_Comm","comm"],["char *","comm_name"]],"MPI_Comm_size":
						[["MPI_Comm","comm"],["int *","size"]],"MPI_Comm_spawn":
						[["char *","command"],["char **","argv"],["int","maxprocs"],["MPI_Info","info"],["int","root"],["MPI_Comm","comm"],["MPI_Comm *","intercomm"],["int *","array_of_errcodes"]],"MPI_Comm_spawn_multiple":
						[["int","count"],["char **","array_of_commands"],["char ***","array_of_argv"],["int *","array_of_maxprocs"],["MPI_Info *","array_of_info"],["int","root"],["MPI_Comm","comm"],["MPI_Comm *","intercomm"],["int *","array_of_errcodes"]],"MPI_Comm_split":
						[["MPI_Comm","comm"],["int","color"],["int","key"],["MPI_Comm *","newcomm"]],"MPI_Comm_test_inter":
						[["MPI_Comm","comm"],["int *","flag"]],"MPI_Dims_create":
						[["int","nnodes"],["int","ndims"],["int *","dims"]],"MPI_Errhandler_c2f":
						[["MPI_Errhandler","errhandler"]],"MPI_Errhandler_create":
						[["MPI_Handler_function *","function"],["MPI_Errhandler *","errhandler"]],"MPI_Errhandler_free":
						[["MPI_Errhandler *","errhandler"]],"MPI_Errhandler_get":
						[["MPI_Comm","comm"],["MPI_Errhandler *","errhandler"]],"MPI_Errhandler_set":
						[["MPI_Comm","comm"],["MPI_Errhandler","errhandler"]],"MPI_Error_class":
						[["int","errorcode"],["int *","errorclass"]],"MPI_Error_string":
						[["int","errorcode"],["char *","string"],["int *","resultlen"]],"MPI_Exscan":
						[["void *","sendbuf"],["void *","recvbuf"],["int","count"],["MPI_Datatype","datatype"],["MPI_Op","op"],["MPI_Comm","comm"]],"MPI_File_c2f":
						[["MPI_File","file"]],"MPI_File_call_errhandler":
						[["MPI_File","fh"],["int","errorcode"]],"MPI_File_create_errhandler":
						[["MPI_File_errhandler_function *","function"],["MPI_Errhandler *","errhandler"]],"MPI_File_set_errhandler":
						[["MPI_File","file"],["MPI_Errhandler","errhandler"]],"MPI_File_get_errhandler":
						[["MPI_File","file"],["MPI_Errhandler *","errhandler"]],"MPI_File_open":
						[["MPI_Comm","comm"],["char *","filename"],["int","amode"],["MPI_Info","info"],["MPI_File *","fh"]],"MPI_File_close":
						[["MPI_File *","fh"]],"MPI_File_delete":
						[["char *","filename"],["MPI_Info","info"]],"MPI_File_set_size":
						[["MPI_File","fh"],["MPI_Offset","size"]],"MPI_File_preallocate":
						[["MPI_File","fh"],["MPI_Offset","size"]],"MPI_File_get_size":
						[["MPI_File","fh"],["MPI_Offset *","size"]],"MPI_File_get_group":
						[["MPI_File","fh"],["MPI_Group *","group"]],"MPI_File_get_amode":
						[["MPI_File","fh"],["int *","amode"]],"MPI_File_set_info":
						[["MPI_File","fh"],["MPI_Info","info"]],"MPI_File_get_info":
						[["MPI_File","fh"],["MPI_Info *","info_used"]],"MPI_File_set_view":
						[["MPI_File","fh"],["MPI_Offset","disp"],["MPI_Datatype","etype"],["MPI_Datatype","filetype"],["char *","datarep"],["MPI_Info","info"]],"MPI_File_get_view":
						[["MPI_File","fh"],["MPI_Offset *","disp"],["MPI_Datatype *","etype"],["MPI_Datatype *","filetype"],["char *","datarep"]],"MPI_File_read_at":
						[["MPI_File","fh"],["MPI_Offset","offset"],["void *","buf"],["int","count"],["MPI_Datatype","datatype"],["MPI_Status *","status"]],"MPI_File_read_at_all":
						[["MPI_File","fh"],["MPI_Offset","offset"],["void *","buf"],["int","count"],["MPI_Datatype","datatype"],["MPI_Status *","status"]],"MPI_File_write_at":
						[["MPI_File","fh"],["MPI_Offset","offset"],["void *","buf"],["int","count"],["MPI_Datatype","datatype"],["MPI_Status *","status"]],"MPI_File_write_at_all":
						[["MPI_File","fh"],["MPI_Offset","offset"],["void *","buf"],["int","count"],["MPI_Datatype","datatype"],["MPI_Status *","status"]],"MPI_File_iread_at":
						[["MPI_File","fh"],["MPI_Offset","offset"],["void *","buf"],["int","count"],["MPI_Datatype","datatype"],["MPI_Request *","request"]],"MPI_File_iwrite_at":
						[["MPI_File","fh"],["MPI_Offset","offset"],["void *","buf"],["int","count"],["MPI_Datatype","datatype"],["MPI_Request *","request"]],"MPI_File_read":
						[["MPI_File","fh"],["void *","buf"],["int","count"],["MPI_Datatype","datatype"],["MPI_Status *","status"]],"MPI_File_read_all":
						[["MPI_File","fh"],["void *","buf"],["int","count"],["MPI_Datatype","datatype"],["MPI_Status *","status"]],"MPI_File_write":
						[["MPI_File","fh"],["void *","buf"],["int","count"],["MPI_Datatype","datatype"],["MPI_Status *","status"]],"MPI_File_write_all":
						[["MPI_File","fh"],["void *","buf"],["int","count"],["MPI_Datatype","datatype"],["MPI_Status *","status"]],"MPI_File_iread":
						[["MPI_File","fh"],["void *","buf"],["int","count"],["MPI_Datatype","datatype"],["MPI_Request *","request"]],"MPI_File_iwrite":
						[["MPI_File","fh"],["void *","buf"],["int","count"],["MPI_Datatype","datatype"],["MPI_Request *","request"]],"MPI_File_seek":
						[["MPI_File","fh"],["MPI_Offset","offset"],["int","whence"]],"MPI_File_get_position":
						[["MPI_File","fh"],["MPI_Offset *","offset"]],"MPI_File_get_byte_offset":
						[["MPI_File","fh"],["MPI_Offset","offset"],["MPI_Offset *","disp"]],"MPI_File_read_shared":
						[["MPI_File","fh"],["void *","buf"],["int","count"],["MPI_Datatype","datatype"],["MPI_Status *","status"]],"MPI_File_write_shared":
						[["MPI_File","fh"],["void *","buf"],["int","count"],["MPI_Datatype","datatype"],["MPI_Status *","status"]],"MPI_File_iread_shared":
						[["MPI_File","fh"],["void *","buf"],["int","count"],["MPI_Datatype","datatype"],["MPI_Request *","request"]],"MPI_File_iwrite_shared":
						[["MPI_File","fh"],["void *","buf"],["int","count"],["MPI_Datatype","datatype"],["MPI_Request *","request"]],"MPI_File_read_ordered":
						[["MPI_File","fh"],["void *","buf"],["int","count"],["MPI_Datatype","datatype"],["MPI_Status *","status"]],"MPI_File_write_ordered":
						[["MPI_File","fh"],["void *","buf"],["int","count"],["MPI_Datatype","datatype"],["MPI_Status *","status"]],"MPI_File_seek_shared":
						[["MPI_File","fh"],["MPI_Offset","offset"],["int","whence"]],"MPI_File_get_position_shared":
						[["MPI_File","fh"],["MPI_Offset *","offset"]],"MPI_File_read_at_all_begin":
						[["MPI_File","fh"],["MPI_Offset","offset"],["void *","buf"],["int","count"],["MPI_Datatype","datatype"]],"MPI_File_read_at_all_end":
						[["MPI_File","fh"],["void *","buf"],["MPI_Status *","status"]],"MPI_File_write_at_all_begin":
						[["MPI_File","fh"],["MPI_Offset","offset"],["void *","buf"],["int","count"],["MPI_Datatype","datatype"]],"MPI_File_write_at_all_end":
						[["MPI_File","fh"],["void *","buf"],["MPI_Status *","status"]],"MPI_File_read_all_begin":
						[["MPI_File","fh"],["void *","buf"],["int","count"],["MPI_Datatype","datatype"]],"MPI_File_read_all_end":
						[["MPI_File","fh"],["void *","buf"],["MPI_Status *","status"]],"MPI_File_write_all_begin":
						[["MPI_File","fh"],["void *","buf"],["int","count"],["MPI_Datatype","datatype"]],"MPI_File_write_all_end":
						[["MPI_File","fh"],["void *","buf"],["MPI_Status *","status"]],"MPI_File_read_ordered_begin":
						[["MPI_File","fh"],["void *","buf"],["int","count"],["MPI_Datatype","datatype"]],"MPI_File_read_ordered_end":
						[["MPI_File","fh"],["void *","buf"],["MPI_Status *","status"]],"MPI_File_write_ordered_begin":
						[["MPI_File","fh"],["void *","buf"],["int","count"],["MPI_Datatype","datatype"]],"MPI_File_write_ordered_end":
						[["MPI_File","fh"],["void *","buf"],["MPI_Status *","status"]],"MPI_File_get_type_extent":
						[["MPI_File","fh"],["MPI_Datatype","datatype"],["MPI_Aint *","extent"]],"MPI_File_set_atomicity":
						[["MPI_File","fh"],["int","flag"]],"MPI_File_get_atomicity":
						[["MPI_File","fh"],["int *","flag"]],"MPI_File_sync":
						[["MPI_File","fh"]],"MPI_Finalize":
						[],"MPI_Finalized":
						[["int *","flag"]],"MPI_Free_mem":
						[["void *","base"]],"MPI_Gather":
						[["void *","sendbuf"],["int","sendcount"],["MPI_Datatype","sendtype"],["void *","recvbuf"],["int","recvcount"],["MPI_Datatype","recvtype"],["int","root"],["MPI_Comm","comm"]],"MPI_Gatherv":
						[["void *","sendbuf"],["int","sendcount"],["MPI_Datatype","sendtype"],["void *","recvbuf"],["int *","recvcounts"],["int *","displs"],["MPI_Datatype","recvtype"],["int","root"],["MPI_Comm","comm"]],"MPI_Get_address":
						[["void *","location"],["MPI_Aint *","address"]],"MPI_Get_count":
						[["MPI_Status *","status"],["MPI_Datatype","datatype"],["int *","count"]],"MPI_Get_elements":
						[["MPI_Status *","status"],["MPI_Datatype","datatype"],["int *","count"]],"MPI_Get":
						[["void *","origin_addr"],["int","origin_count"],["MPI_Datatype","origin_datatype"],["int","target_rank"],["MPI_Aint","target_disp"],["int","target_count"],["MPI_Datatype","target_datatype"],["MPI_Win","win"]],"MPI_Get_processor_name":
						[["char *","name"],["int *","resultlen"]],"MPI_Get_version":
						[["int *","version"],["int *","subversion"]],"MPI_Graph_create":
						[["MPI_Comm","comm_old"],["int","nnodes"],["int *","index"],["int *","edges"],["int","reorder"],["MPI_Comm *","comm_graph"]],"MPI_Graph_get":
						[["MPI_Comm","comm"],["int","maxindex"],["int","maxedges"],["int *","index"],["int *","edges"]],"MPI_Graph_map":
						[["MPI_Comm","comm"],["int","nnodes"],["int *","index"],["int *","edges"],["int *","newrank"]],"MPI_Graph_neighbors_count":
						[["MPI_Comm","comm"],["int","rank"],["int *","nneighbors"]],"MPI_Graph_neighbors":
						[["MPI_Comm","comm"],["int","rank"],["int","maxneighbors"],["int *","neighbors"]],"MPI_Graphdims_get":
						[["MPI_Comm","comm"],["int *","nnodes"],["int *","nedges"]],"MPI_Grequest_complete":
						[["MPI_Request","request"]],"MPI_Grequest_start":
						[["MPI_Grequest_query_function *","query_fn"],["MPI_Grequest_free_function *","free_fn"],["MPI_Grequest_cancel_function *","cancel_fn"],["void *","extra_state"],["MPI_Request *","request"]],"MPI_Group_c2f":
						[["MPI_Group","group"]],"MPI_Group_compare":
						[["MPI_Group","group1"],["MPI_Group","group2"],["int *","result"]],"MPI_Group_difference":
						[["MPI_Group","group1"],["MPI_Group","group2"],["MPI_Group *","newgroup"]],"MPI_Group_excl":
						[["MPI_Group","group"],["int","n"],["int *","ranks"],["MPI_Group *","newgroup"]],"MPI_Group_free":
						[["MPI_Group *","group"]],"MPI_Group_incl":
						[["MPI_Group","group"],["int","n"],["int *","ranks"],["MPI_Group *","newgroup"]],"MPI_Group_intersection":
						[["MPI_Group","group1"],["MPI_Group","group2"],["MPI_Group *","newgroup"]],"MPI_Group_range_excl":
						[["MPI_Group","group"],["int","n"],["int **","ranges"],["MPI_Group *","newgroup"]],"MPI_Group_range_incl":
						[["MPI_Group","group"],["int","n"],["int **","ranges"],["MPI_Group *","newgroup"]],"MPI_Group_rank":
						[["MPI_Group","group"],["int *","rank"]],"MPI_Group_size":
						[["MPI_Group","group"],["int *","size"]],"MPI_Group_translate_ranks":
						[["MPI_Group","group1"],["int","n"],["int *","ranks1"],["MPI_Group","group2"],["int *","ranks2"]],"MPI_Group_union":
						[["MPI_Group","group1"],["MPI_Group","group2"],["MPI_Group *","newgroup"]],"MPI_Ibsend":
						[["void *","buf"],["int","count"],["MPI_Datatype","datatype"],["int","dest"],["int","tag"],["MPI_Comm","comm"],["MPI_Request *","request"]],"MPI_Info_c2f":
						[["MPI_Info","info"]],"MPI_Info_create":
						[["MPI_Info *","info"]],"MPI_Info_delete":
						[["MPI_Info","info"],["char *","key"]],"MPI_Info_dup":
						[["MPI_Info","info"],["MPI_Info *","newinfo"]],"MPI_Info_free":
						[["MPI_Info *","info"]],"MPI_Info_get":
						[["MPI_Info","info"],["char *","key"],["int","valuelen"],["char *","value"],["int *","flag"]],"MPI_Info_get_nkeys":
						[["MPI_Info","info"],["int *","nkeys"]],"MPI_Info_get_nthkey":
						[["MPI_Info","info"],["int","n"],["char *","key"]],"MPI_Info_get_valuelen":
						[["MPI_Info","info"],["char *","key"],["int *","valuelen"],["int *","flag"]],"MPI_Info_set":
						[["MPI_Info","info"],["char *","key"],["char *","value"]],"MPI_Init":
						[["int *","argc"],["char ***","argv"]],"MPI_Initialized":
						[["int *","flag"]],"MPI_Init_thread":
						[["int *","argc"],["char ***","argv"],["int","required"],["int *","provided"]],"MPI_Intercomm_create":
						[["MPI_Comm","local_comm"],["int","local_leader"],["MPI_Comm","bridge_comm"],["int","remote_leader"],["int","tag"],["MPI_Comm *","newintercomm"]],"MPI_Intercomm_merge":
						[["MPI_Comm","intercomm"],["int","high"],["MPI_Comm *","newintercomm"]],"MPI_Iprobe":
						[["int","source"],["int","tag"],["MPI_Comm","comm"],["int *","flag"],["MPI_Status *","status"]],"MPI_Irecv":
						[["void *","buf"],["int","count"],["MPI_Datatype","datatype"],["int","source"],["int","tag"],["MPI_Comm","comm"],["MPI_Request *","request"]],"MPI_Irsend":
						[["void *","buf"],["int","count"],["MPI_Datatype","datatype"],["int","dest"],["int","tag"],["MPI_Comm","comm"],["MPI_Request *","request"]],"MPI_Isend":
						[["void *","buf"],["int","count"],["MPI_Datatype","datatype"],["int","dest"],["int","tag"],["MPI_Comm","comm"],["MPI_Request *","request"]],"MPI_Issend":
						[["void *","buf"],["int","count"],["MPI_Datatype","datatype"],["int","dest"],["int","tag"],["MPI_Comm","comm"],["MPI_Request *","request"]],"MPI_Is_thread_main":
						[["int *","flag"]],"MPI_Keyval_create":
						[["MPI_Copy_function *","copy_fn"],["MPI_Delete_function *","delete_fn"],["int *","keyval"],["void *","extra_state"]],"MPI_Keyval_free":
						[["int *","keyval"]],"MPI_Lookup_name":
						[["char *","service_name"],["MPI_Info","info"],["char *","port_name"]],"MPI_Op_c2f":
						[["MPI_Op","op"]],"MPI_Op_commutative":
						[["MPI_Op","op"],["int *","commute"]],"MPI_Op_create":
						[["MPI_User_function *","function"],["int","commute"],["MPI_Op *","op"]],"MPI_Open_port":
						[["MPI_Info","info"],["char *","port_name"]],"MPI_Op_free":
						[["MPI_Op *","op"]],"MPI_Pack_external":
						[["char *","datarep"],["void *","inbuf"],["int","incount"],["MPI_Datatype","datatype"],["void *","outbuf"],["MPI_Aint","outsize"],["MPI_Aint *","position"]],"MPI_Pack_external_size":
						[["char *","datarep"],["int","incount"],["MPI_Datatype","datatype"],["MPI_Aint *","size"]],"MPI_Pack":
						[["void *","inbuf"],["int","incount"],["MPI_Datatype","datatype"],["void *","outbuf"],["int","outsize"],["int *","position"],["MPI_Comm","comm"]],"MPI_Pack_size":
						[["int","incount"],["MPI_Datatype","datatype"],["MPI_Comm","comm"],["int *","size"]],"MPI_Probe":
						[["int","source"],["int","tag"],["MPI_Comm","comm"],["MPI_Status *","status"]],"MPI_Publish_name":
						[["char *","service_name"],["MPI_Info","info"],["char *","port_name"]],"MPI_Put":
						[["void *","origin_addr"],["int","origin_count"],["MPI_Datatype","origin_datatype"],["int","target_rank"],["MPI_Aint","target_disp"],["int","target_count"],["MPI_Datatype","target_datatype"],["MPI_Win","win"]],"MPI_Query_thread":
						[["int *","provided"]],"MPI_Recv_init":
						[["void *","buf"],["int","count"],["MPI_Datatype","datatype"],["int","source"],["int","tag"],["MPI_Comm","comm"],["MPI_Request *","request"]],"MPI_Recv":
						[["void *","buf"],["int","count"],["MPI_Datatype","datatype"],["int","source"],["int","tag"],["MPI_Comm","comm"],["MPI_Status *","status"]],"MPI_Reduce":
						[["void *","sendbuf"],["void *","recvbuf"],["int","count"],["MPI_Datatype","datatype"],["MPI_Op","op"],["int","root"],["MPI_Comm","comm"]],"MPI_Reduce_local":
						[["void *","inbuf"],["void *","inoutbuf"],["int","count"],["MPI_Datatype","datatype"],["MPI_Op","op"]],"MPI_Reduce_scatter":
						[["void *","sendbuf"],["void *","recvbuf"],["int *","recvcounts"],["MPI_Datatype","datatype"],["MPI_Op","op"],["MPI_Comm","comm"]],"MPI_Register_datarep":
						[["char *","datarep"],["MPI_Datarep_conversion_function *","read_conversion_fn"],["MPI_Datarep_conversion_function *","write_conversion_fn"],["MPI_Datarep_extent_function *","dtype_file_extent_fn"],["void *","extra_state"]],"MPI_Request_c2f":
						[["MPI_Request","request"]],"MPI_Request_free":
						[["MPI_Request *","request"]],"MPI_Request_get_status":
						[["MPI_Request","request"],["int *","flag"],["MPI_Status *","status"]],"MPI_Rsend":
						[["void *","ibuf"],["int","count"],["MPI_Datatype","datatype"],["int","dest"],["int","tag"],["MPI_Comm","comm"]],"MPI_Rsend_init":
						[["void *","buf"],["int","count"],["MPI_Datatype","datatype"],["int","dest"],["int","tag"],["MPI_Comm","comm"],["MPI_Request *","request"]],"MPI_Scan":
						[["void *","sendbuf"],["void *","recvbuf"],["int","count"],["MPI_Datatype","datatype"],["MPI_Op","op"],["MPI_Comm","comm"]],"MPI_Scatter":
						[["void *","sendbuf"],["int","sendcount"],["MPI_Datatype","sendtype"],["void *","recvbuf"],["int","recvcount"],["MPI_Datatype","recvtype"],["int","root"],["MPI_Comm","comm"]],"MPI_Scatterv":
						[["void *","sendbuf"],["int *","sendcounts"],["int *","displs"],["MPI_Datatype","sendtype"],["void *","recvbuf"],["int","recvcount"],["MPI_Datatype","recvtype"],["int","root"],["MPI_Comm","comm"]],"MPI_Send_init":
						[["void *","buf"],["int","count"],["MPI_Datatype","datatype"],["int","dest"],["int","tag"],["MPI_Comm","comm"],["MPI_Request *","request"]],"MPI_Send":
						[["void *","buf"],["int","count"],["MPI_Datatype","datatype"],["int","dest"],["int","tag"],["MPI_Comm","comm"]],"MPI_Sendrecv":
						[["void *","sendbuf"],["int","sendcount"],["MPI_Datatype","sendtype"],["int","dest"],["int","sendtag"],["void *","recvbuf"],["int","recvcount"],["MPI_Datatype","recvtype"],["int","source"],["int","recvtag"],["MPI_Comm","comm"],["MPI_Status *","status"]],"MPI_Sendrecv_replace":
						[["void *","buf"],["int","count"],["MPI_Datatype","datatype"],["int","dest"],["int","sendtag"],["int","source"],["int","recvtag"],["MPI_Comm","comm"],["MPI_Status *","status"]],"MPI_Ssend_init":
						[["void *","buf"],["int","count"],["MPI_Datatype","datatype"],["int","dest"],["int","tag"],["MPI_Comm","comm"],["MPI_Request *","request"]],"MPI_Ssend":
						[["void *","buf"],["int","count"],["MPI_Datatype","datatype"],["int","dest"],["int","tag"],["MPI_Comm","comm"]],"MPI_Start":
						[["MPI_Request *","request"]],"MPI_Startall":
						[["int","count"],["MPI_Request *","array_of_requests"]],"MPI_Status_set_cancelled":
						[["MPI_Status *","status"],["int","flag"]],"MPI_Status_set_elements":
						[["MPI_Status *","status"],["MPI_Datatype","datatype"],["int","count"]],"MPI_Testall":
						[["int","count"],["MPI_Request *","array_of_requests"],["int *","flag"],["MPI_Status *","array_of_statuses"]],"MPI_Testany":
						[["int","count"],["MPI_Request *","array_of_requests"],["int *","index"],["int *","flag"],["MPI_Status *","status"]],"MPI_Test":
						[["MPI_Request *","request"],["int *","flag"],["MPI_Status *","status"]],"MPI_Test_cancelled":
						[["MPI_Status *","status"],["int *","flag"]],"MPI_Testsome":
						[["int","incount"],["MPI_Request *","array_of_requests"],["int *","outcount"],["int *","array_of_indices"],["MPI_Status *","array_of_statuses"]],"MPI_Topo_test":
						[["MPI_Comm","comm"],["int *","status"]],"MPI_Type_c2f":
						[["MPI_Datatype","datatype"]],"MPI_Type_commit":
						[["MPI_Datatype *","type"]],"MPI_Type_contiguous":
						[["int","count"],["MPI_Datatype","oldtype"],["MPI_Datatype *","newtype"]],"MPI_Type_create_darray":
						[["int","size"],["int","rank"],["int","ndims"],["int *","gsize_array"],["int *","distrib_array"],["int *","darg_array"],["int *","psize_array"],["int","order"],["MPI_Datatype","oldtype"],["MPI_Datatype *","newtype"]],"MPI_Type_create_hindexed":
						[["int","count"],["int *","array_of_blocklengths"],["MPI_Aint *","array_of_displacements"],["MPI_Datatype","oldtype"],["MPI_Datatype *","newtype"]],"MPI_Type_create_hvector":
						[["int","count"],["int","blocklength"],["MPI_Aint","stride"],["MPI_Datatype","oldtype"],["MPI_Datatype *","newtype"]],"MPI_Type_create_keyval":
						[["MPI_Type_copy_attr_function *","type_copy_attr_fn"],["MPI_Type_delete_attr_function *","type_delete_attr_fn"],["int *","type_keyval"],["void *","extra_state"]],"MPI_Type_create_indexed_block":
						[["int","count"],["int","blocklength"],["int *","array_of_displacements"],["MPI_Datatype","oldtype"],["MPI_Datatype *","newtype"]],"MPI_Type_create_struct":
						[["int","count"],["int *","array_of_block_lengths"],["MPI_Aint *","array_of_displacements"],["MPI_Datatype *","array_of_types"],["MPI_Datatype *","newtype"]],"MPI_Type_create_subarray":
						[["int","ndims"],["int *","size_array"],["int *","subsize_array"],["int *","start_array"],["int","order"],["MPI_Datatype","oldtype"],["MPI_Datatype *","newtype"]],"MPI_Type_create_resized":
						[["MPI_Datatype","oldtype"],["MPI_Aint","lb"],["MPI_Aint","extent"],["MPI_Datatype *","newtype"]],"MPI_Type_delete_attr":
						[["MPI_Datatype","type"],["int","type_keyval"]],"MPI_Type_dup":
						[["MPI_Datatype","type"],["MPI_Datatype *","newtype"]],"MPI_Type_extent":
						[["MPI_Datatype","type"],["MPI_Aint *","extent"]],"MPI_Type_free":
						[["MPI_Datatype *","type"]],"MPI_Type_free_keyval":
						[["int *","type_keyval"]],"MPI_Type_get_attr":
						[["MPI_Datatype","type"],["int","type_keyval"],["void *","attribute_val"],["int *","flag"]],"MPI_Type_get_contents":
						[["MPI_Datatype","mtype"],["int","max_integers"],["int","max_addresses"],["int","max_datatypes"],["int *","array_of_integers"],["MPI_Aint *","array_of_addresses"],["MPI_Datatype *","array_of_datatypes"]],"MPI_Type_get_envelope":
						[["MPI_Datatype","type"],["int *","num_integers"],["int *","num_addresses"],["int *","num_datatypes"],["int *","combiner"]],"MPI_Type_get_extent":
						[["MPI_Datatype","type"],["MPI_Aint *","lb"],["MPI_Aint *","extent"]],"MPI_Type_get_name":
						[["MPI_Datatype","type"],["char *","type_name"],["int *","resultlen"]],"MPI_Type_get_true_extent":
						[["MPI_Datatype","datatype"],["MPI_Aint *","true_lb"],["MPI_Aint *","true_extent"]],"MPI_Type_hindexed":
						[["int","count"],["int *","array_of_blocklengths"],["MPI_Aint *","array_of_displacements"],["MPI_Datatype","oldtype"],["MPI_Datatype *","newtype"]],"MPI_Type_hvector":
						[["int","count"],["int","blocklength"],["MPI_Aint","stride"],["MPI_Datatype","oldtype"],["MPI_Datatype *","newtype"]],"MPI_Type_indexed":
						[["int","count"],["int *","array_of_blocklengths"],["int *","array_of_displacements"],["MPI_Datatype","oldtype"],["MPI_Datatype *","newtype"]],"MPI_Type_lb":
						[["MPI_Datatype","type"],["MPI_Aint *","lb"]],"MPI_Type_match_size":
						[["int","typeclass"],["int","size"],["MPI_Datatype *","type"]],"MPI_Type_set_attr":
						[["MPI_Datatype","type"],["int","type_keyval"],["void *","attr_val"]],"MPI_Type_set_name":
						[["MPI_Datatype","type"],["char *","type_name"]],"MPI_Type_size":
						[["MPI_Datatype","type"],["int *","size"]],"MPI_Type_struct":
						[["int","count"],["int *","array_of_blocklengths"],["MPI_Aint *","array_of_displacements"],["MPI_Datatype *","array_of_types"],["MPI_Datatype *","newtype"]],"MPI_Type_ub":
						[["MPI_Datatype","mtype"],["MPI_Aint *","ub"]],"MPI_Type_vector":
						[["int","count"],["int","blocklength"],["int","stride"],["MPI_Datatype","oldtype"],["MPI_Datatype *","newtype"]],"MPI_Unpack":
						[["void *","inbuf"],["int","insize"],["int *","position"],["void *","outbuf"],["int","outcount"],["MPI_Datatype","datatype"],["MPI_Comm","comm"]],"MPI_Unpublish_name":
						[["char *","service_name"],["MPI_Info","info"],["char *","port_name"]],"MPI_Unpack_external":
						[["char *","datarep"],["void *","inbuf"],["MPI_Aint","insize"],["MPI_Aint *","position"],["void *","outbuf"],["int","outcount"],["MPI_Datatype","datatype"]],"MPI_Waitall":
						[["int","count"],["MPI_Request *","array_of_requests"],["MPI_Status *","array_of_statuses"]],"MPI_Waitany":
						[["int","count"],["MPI_Request *","array_of_requests"],["int *","index"],["MPI_Status *","status"]],"MPI_Wait":
						[["MPI_Request *","request"],["MPI_Status *","status"]],"MPI_Waitsome":
						[["int","incount"],["MPI_Request *","array_of_requests"],["int *","outcount"],["int *","array_of_indices"],["MPI_Status *","array_of_statuses"]],"MPI_Win_c2f":
						[["MPI_Win","win"]],"MPI_Win_call_errhandler":
						[["MPI_Win","win"],["int","errorcode"]],"MPI_Win_complete":
						[["MPI_Win","win"]],"MPI_Win_create":
						[["void *","base"],["MPI_Aint","size"],["int","disp_unit"],["MPI_Info","info"],["MPI_Comm","comm"],["MPI_Win *","win"]],"MPI_Win_create_errhandler":
						[["MPI_Win_errhandler_function *","function"],["MPI_Errhandler *","errhandler"]],"MPI_Win_create_keyval":
						[["MPI_Win_copy_attr_function *","win_copy_attr_fn"],["MPI_Win_delete_attr_function *","win_delete_attr_fn"],["int *","win_keyval"],["void *","extra_state"]],"MPI_Win_delete_attr":
						[["MPI_Win","win"],["int","win_keyval"]],"MPI_Win_fence":
						[["int","assert"],["MPI_Win","win"]],"MPI_Win_free":
						[["MPI_Win *","win"]],"MPI_Win_free_keyval":
						[["int *","win_keyval"]],"MPI_Win_get_attr":
						[["MPI_Win","win"],["int","win_keyval"],["void *","attribute_val"],["int *","flag"]],"MPI_Win_get_errhandler":
						[["MPI_Win","win"],["MPI_Errhandler *","errhandler"]],"MPI_Win_get_group":
						[["MPI_Win","win"],["MPI_Group *","group"]],"MPI_Win_get_name":
						[["MPI_Win","win"],["char *","win_name"],["int *","resultlen"]],"MPI_Win_lock":
						[["int","lock_type"],["int","rank"],["int","assert"],["MPI_Win","win"]],"MPI_Win_post":
						[["MPI_Group","group"],["int","assert"],["MPI_Win","win"]],"MPI_Win_set_attr":
						[["MPI_Win","win"],["int","win_keyval"],["void *","attribute_val"]],"MPI_Win_set_errhandler":
						[["MPI_Win","win"],["MPI_Errhandler","errhandler"]],"MPI_Win_set_name":
						[["MPI_Win","win"],["char *","win_name"]],"MPI_Win_start":
						[["MPI_Group","group"],["int","assert"],["MPI_Win","win"]],"MPI_Win_test":
						[["MPI_Win","win"],["int *","flag"]],"MPI_Win_unlock":
						[["int","rank"],["MPI_Win","win"]],"MPI_Win_wait":
						[["MPI_Win","win"]],"MPI_Wtick":
						[],"MPI_Wtime":
						[]};
	
	MPI_func.prototype.check_args = function()
	{
		var ref = MPI_Functions_kb[ this.name ];
		
		if( ref == undefined )
			return -1;
		
		/* Check that arg types match */
		if( this.args.length != ref.length )
		{
			//console.log("length mismatch");
			return;
		}
		
		var i;
		
		for( i = 0 ; i < this.args.length ; i++ )
		{
			if( this.args[i][0].trim() != ref[i][0].trim() )
			{
				//console.log("type mismatch " + this.args[i][0] + " vs "  +  ref[i][0].trim());
				return;
			}
		}
		
		/* If we are here functions are matching */
		for( i = 0 ; i < this.args.length ; i++ )
		{
			this.args[i][1] = ref[i][1].trim();
		}

                return 0
	}
	
	
/*************************************************/
/* Actual function parsing                       */
/*************************************************/

	/* Parse each MPI function */

	var MPI_Functions = [];

	var i = 0;

	for( i = 0 ; i < mpi_h_lines.length ; i++ )
	{
		if( mpi_h_lines[i].contains("typedef")
		||  mpi_h_lines[i].contains("\\.\\.\\.") )
			continue;
		
		match = mpi_h_lines[i].match(".*(int|double|MPI_Aint)\\s+(MPI_.*?)\\((.*?)\\)")
		
		if( match != null )
		{
			var func = new MPI_func(match[1], match[2], match[3]);
			
			if( func.parse() )
			{
	/*			console.log( "Wrapping " + func.name + "...");  */
				
				/* Here we replace args with those from the base
				 * when possible in order to ensure consistensy */
				cret = func.check_args();
				
                                if( cret < 0 )
                                    continue;

				/* Check if we do not have it yet */
				var k;
				var is_dup = 0;
				
				for( k = 0 ; k < MPI_Functions.length ; k++ )
				{
					if( MPI_Functions[k].name == func.name )
					{
						console.log("\t Duplicated function" );
						is_dup = 1;
						break;
					}
				}
				
				if( !is_dup )
				{
					MPI_Functions.push( func );
				/* 	console.log( "\t Done."); */ 
				}
			}
			else
			{
				console.log("\t Failed to parse " + match[1] );
			}
			
		}
	}

	console.log("\033[1;31mSuccessfully wrapped " + MPI_Functions.length + " MPI functions...\033[0m");

	if( MPI_Functions.length == 0 )
	{
		console.log("We did not find anything using " + mpicc );
		process.exit(1);
	}

/*************************************************/
/* Wrapper script loading                        */
/*************************************************/
	/* READ SCRIPT */
	var script_data = "";

	try
	{
		script_data = fs.readFileSync( script, { encoding : "ascii" }  );
	}
	catch(e)
	{
		console.log("Failed to open script " + script );
		console.log(e);
		process.exit(1);
	}
		
	var script_lines = script_data.split('\n');

/*************************************************/
/* Wrapper script compilation                    */
/*************************************************/

	/* Script entity */

	function script_action( type, line, filter, params )
	{	
		this.type = type;
		this.line = line;

		this.is_whitelisted = 0;
		this.whitelist = {};
		this.blacklist = {};
		
		if( filter != "" )
		{
			var sp = filter.split(",");
			
			var i;
			
			for( i = 0; i < sp.length ; i++ )
			{
				sp[i] = sp[i].trim();
		
				if( sp[i][0] == "!" )
				{
					
					if( this.is_whitelisted )
					{
						console.log("You cannot whitelist while blacklisting: " + filter );
						process.exit(1);
					}

					this.blacklist[ sp[i].slice(1) ] = 1;
				}
				else
				{
					this.is_whitelisted = 1;
					this.whitelist[ sp[i] ] = 1;
				}
				
			}
		
		}

		this.params = params;
		
		this.actions = [];
	}

	/* Parser object */

	function script_parser()
	{
		this.is_in_wrapped_section = 0;
		this.actions = [];
		
		this.output = "";
	}

	script_parser.prototype.parseline = function(line )
	{
		/* Section Start */
		var match = line.match("<MALPW(.*)>");
		
		if( match != null )
		{
			var filter = match[1];
			
			this.is_in_wrapped_section = 1;
			
			this.actions.push( new script_action( "section", line, filter ) );
			
			return;
		}
		
		match = line.match("</MALPW>");
		
		if( match != null )
		{
			this.is_in_wrapped_section = 0;
			
			return;
		}
		
		if( !this.is_in_wrapped_section )
		{
			/* Out of sections anything is a text line */
			this.actions.push( new script_action( "text", line, "" ) );
		}
		else
		{
			var current_action = this.actions[this.actions.length - 1 ];
			
			if( current_action.type != "section" )
			{
				console.log("You can only put commands in sections " + script + ":" + (i + 1) );
				process.exit(1); 
			}
			
			match = line.match("MALPW_PROTOTYPE\\(\\);");
			
			if( match != null )
			{
				current_action.actions.push( new script_action( "prototype", "", "" ) );
				return;
			}

			match = line.match("MALPW_WRAP\\((.*),(.*),(.*)\\);");
			
			if( match != null )
			{
				if( match.length != 4 )
				{
					console.log("Missing arguments to MALPW_WRAP" + script + ":" + (i + 1) );
					process.exit(1);
				}
				
				var type = match[1];
				var filter = match[2];
				var macro = match[3];
				
				current_action.actions.push( new script_action( "wrap", "", "", { "type" : type , "filter" : filter, "macro" : macro } ) );
				return;
			}
					
			match = line.match("MALPW_WNAMECASE_UP\\((.*)\\);");
			
			if( match != null )
			{
				var prefix = "";
				
				if( match.length == 2 )
				{
					prefix = match[1].trim();				
				}
				
				current_action.actions.push( new script_action( "caseup", "", "", { "prefix" : prefix } ) );
				return;
			}
			
			match = line.match("MALPW_WNAME\\((.*)\\);");
			
			if( match != null )
			{
				var prefix = ""
				
				if( match.length == 2 )
				{
					prefix = match[1].trim();
				}
	
				current_action.actions.push( new script_action( "wname", "", "", { "prefix" : prefix } ) );
				return;
			}		
	
			match = line.match("MALPW_CALL\\(.*\\);");
			
			if( match != null )
			{

				var macro = match[1];
				
				current_action.actions.push( new script_action( "call", "", "" ) );
				return;
			}
			
			match = line.match("MALPW_HAS_ARG_B\\((.*),(.*)\\);");
			
			if( match != null )
			{
	
				if( match.length != 3 )
				{
					console.log("Missing arguments to MALPW_HAS_ARG_B" + script + ":" + (i + 1) );
					process.exit(1);
				}
				
				var type = match[1];
				var name = match[2];
				
				current_action.actions.push( new script_action( "argb", "", "", { "type" : type, "name" : name } ) );
				return;
			}
			
			match = line.match("MALPW_HAS_ARG_E(.*);");
			
			if( match != null )
			{

				var type = match[1];
				var name = match[2];
				
				current_action.actions.push( new script_action( "arge", "", "", { "type" : type, "name" : name } ) );
				return;
			}

			/* IF it did not match it is text */
			current_action.actions.push( new script_action( "text", line, "" ) );
		}	
	}
	
	
	script_parser.prototype.do_action = function( action, current_func )
	{
		var ret = "";
		var i;
		
		switch( action.type )
		{
				case "text" :
					ret = action.line;
					break;
				case "prototype" :
					if( current_func == undefined )
						return;
						
					ret = current_func.ret + " " + current_func.name + "( ";
					
					for( i=0; i < current_func.args.length ; i++ )
					{
						ret += current_func.args[i][0] + " " + current_func.args[i][1];
						if( i != (current_func.args.length - 1 ) )
							ret += ", ";
						else
							ret += " ";
					}
					
					ret += ")"
					
					break;
				case "wrap" :
					if( current_func == undefined )
						return;

					for( i=0; i < current_func.args.length ; i++ )
					{
						if( current_func.args[i][0].trim() == action.params['type'].trim()
						    && ( (current_func.args[i][1] == action.params['filter'].trim()) || action.params['filter'].trim() == "*" )
						  )
						{
							ret += "\t" + action.params['macro'] + "(" + current_func.args[i][1].trim() + ")";
						}
					}
					
					break;
				case "wname" :
					if( action.params['prefix'] != "" )
						ret += "\t" + action.params['prefix'] + "(" + current_func.name.trim() + ")";
					else
						ret += "\t" + current_func.name.trim();
					break;
				case "caseup" :
					if( action.params['prefix'] != "" )
						ret += "\t" + action.params['prefix'] + "(" + current_func.name.toUpperCase().trim() + ")";
					else
						ret += "\t" + current_func.name.toUpperCase().trim();
					break;
				case "call" :
					ret = "\t" + current_func.ret + " ret = P" + current_func.name + "( ";
					
					for( i=0; i < current_func.args.length ; i++ )
					{
						ret += current_func.args[i][1].replace(/\[.*\]/g, '');
						if( i != (current_func.args.length - 1 ) )
							ret += ", ";
						else
							ret += " ";
					}
					
					ret += ");"
					break;			
				case "argb" :
					if( current_func == undefined )
						return;

					var found = 0;

					for( i=0; i < current_func.args.length ; i++ )
					{
						if( current_func.args[i][0].trim() == action.params['type'].trim()
						    && ( (current_func.args[i][1] == action.params['name'].trim()) || action.params['name'].trim() == "*" )
						  )
						{
							found = 1;
							break;
						}
					}
					
					ret = "\t#if " + found;
					break;
				case "arge" :
					ret = "\t#endif";
					break;		
		}
		
		this.output += ret + "\n";
	}	
	

	script_parser.prototype.gen = function()
	{
		var i;
		var j;
		var k;
		
		for( i = 0 ; i < this.actions.length ; i++ )
		{
			if( this.actions[i].type != "section" )
			{
					this.do_action( this.actions[i] );
			}
			else
			{
				for( j = 0 ; j < MPI_Functions.length ; j++ )
				{
					if( this.actions[i].is_whitelisted )
					{
						if( this.actions[i].whitelist[ MPI_Functions[j].name ] == undefined )
							continue;
					}
					else
					{
						if( this.actions[i].blacklist[ MPI_Functions[j].name ] != undefined )
						{
							continue;
						}
					}
						
					for( k = 0 ; k < this.actions[i].actions.length ; k++ )
					{
						this.do_action( this.actions[i].actions[k],  MPI_Functions[j] );
					}
				}
			}
			
			
		}
		
		
		
		
	}

/*************************************************/
/* Final output generation                       */
/*************************************************/

	var sc = new script_parser();


	for( i = 0 ; i < script_lines.length ; i++ )
	{
		sc.parseline( script_lines[i] );
	}

	sc.gen();
	
	//gen_mpi_kb( MPI_Functions );
	
	//console.log( sc.output );
	try
	{
		fs.writeFileSync(output_file, sc.output);
	}
	catch(e)
	{
		console.log("Failed to write output program");
		console.log(e);
		process.exit(1);
	}
});
